@@ -12,28 +12,30 @@
 # Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
 # Copyright Buildbot Team Members
-
-from __future__ import with_statement
-
-
 # Based on the work of Dave Peticolas for the P4poll
 # Changed to svn (using xml.dom.minidom) by Niklaus Giger
 # Hacked beyond recognition by Brian Warner
 
+import os
+import xml.dom.minidom
+from urllib.parse import quote_plus as urlquote_plus
+
+from twisted.internet import defer
+from twisted.internet import utils
 from twisted.python import log
-from twisted.internet import defer, utils
 
 from buildbot import util
 from buildbot.changes import base
-
-import xml.dom.minidom
-import os, urllib
+from buildbot.util import bytes2unicode
 
 # these split_file_* functions are available for use as values to the
 # split_file= argument.
+
+
 def split_file_alwaystrunk(path):
     return dict(path=path)
 
+
 def split_file_branches(path):
     # turn "trunk/subdir/file.c" into (None, "subdir/file.c")
     # and "trunk/subdir/" into (None, "subdir/")
@@ -46,12 +48,13 @@ def split_file_branches(path):
         return (None, '/'.join(pieces[1:]))
     elif len(pieces) > 2 and pieces[0] == 'branches':
         return ('/'.join(pieces[0:2]), '/'.join(pieces[2:]))
-    else:
-        return None
+    return None
+
 
 def split_file_projects_branches(path):
-    # turn projectname/trunk/subdir/file.c into dict(project=projectname, branch=trunk, path=subdir/file.c)
-    if not "/" in path:
+    # turn projectname/trunk/subdir/file.c into dict(project=projectname,
+    # branch=trunk, path=subdir/file.c)
+    if "/" not in path:
         return None
     project, path = path.split("/", 1)
     f = split_file_branches(path)
@@ -62,37 +65,43 @@ def split_file_projects_branches(path):
         return info
     return f
 
+
 class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
+
     """
     Poll a Subversion repository for changes and submit them to the change
     master.
     """
 
-    compare_attrs = ["svnurl", "split_file",
-                     "svnuser", "svnpasswd", "project",
-                     "pollInterval", "histmax",
-                     "svnbin", "category", "cachepath"]
-
-    parent = None # filled in when we're added
+    compare_attrs = ("repourl", "split_file", "svnuser", "svnpasswd", "project", "pollInterval",
+                     "histmax", "svnbin", "category", "cachepath", "pollAtLaunch",
+                     "pollRandomDelayMin", "pollRandomDelayMax")
+    secrets = ("svnuser", "svnpasswd")
+    parent = None  # filled in when we're added
     last_change = None
     loop = None
 
-    def __init__(self, svnurl, split_file=None,
-                 svnuser=None, svnpasswd=None,
-                 pollInterval=10*60, histmax=100,
-                 svnbin='svn', revlinktmpl='', category=None, 
-                 project='', cachepath=None, pollinterval=-2,
-                 extra_args=None):
+    def __init__(self, repourl, split_file=None, svnuser=None, svnpasswd=None,
+                 pollInterval=10 * 60, histmax=100, svnbin="svn", revlinktmpl="",
+                 category=None, project="", cachepath=None, pollinterval=-2,
+                 extra_args=None, name=None, pollAtLaunch=False, pollRandomDelayMin=0,
+                 pollRandomDelayMax=0):
 
         # for backward compatibility; the parameter used to be spelled with 'i'
         if pollinterval != -2:
             pollInterval = pollinterval
 
-        base.PollingChangeSource.__init__(self, name=svnurl, pollInterval=pollInterval)
+        if name is None:
+            name = repourl
 
-        if svnurl.endswith("/"):
-            svnurl = svnurl[:-1] # strip the trailing slash
-        self.svnurl = svnurl
+        super().__init__(name=name, pollInterval=pollInterval, pollAtLaunch=pollAtLaunch,
+                         svnuser=svnuser, svnpasswd=svnpasswd,
+                         pollRandomDelayMin=pollRandomDelayMin,
+                         pollRandomDelayMax=pollRandomDelayMax)
+
+        if repourl.endswith("/"):
+            repourl = repourl[:-1]  # strip the trailing slash
+        self.repourl = repourl
         self.extra_args = extra_args
         self.split_file = split_file or split_file_alwaystrunk
         self.svnuser = svnuser
@@ -100,38 +109,40 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
 
         self.revlinktmpl = revlinktmpl
 
-        self.environ = os.environ.copy() # include environment variables
-                                         # required for ssh-agent auth
+        # include environment variables required for ssh-agent auth
+        self.environ = os.environ.copy()
 
         self.svnbin = svnbin
         self.histmax = histmax
         self._prefix = None
-        self.category = category
-        self.project = project
+        self.category = category if callable(
+            category) else util.bytes2unicode(category)
+        self.project = util.bytes2unicode(project)
 
         self.cachepath = cachepath
         if self.cachepath and os.path.exists(self.cachepath):
             try:
                 with open(self.cachepath, "r") as f:
                     self.last_change = int(f.read().strip())
-                    log.msg("SVNPoller: SVNPoller(%s) setting last_change to %s" % (self.svnurl, self.last_change))
+                    log.msg(("SVNPoller: SVNPoller({}) setting last_change to {}"
+                             ).format(self.repourl, self.last_change))
                 # try writing it, too
                 with open(self.cachepath, "w") as f:
                     f.write(str(self.last_change))
-            except:
+            except Exception:
                 self.cachepath = None
-                log.msg(("SVNPoller: SVNPoller(%s) cache file corrupt or unwriteable; " +
-                        "skipping and not using") % self.svnurl)
+                log.msg(("SVNPoller: SVNPoller({}) cache file corrupt or unwriteable; " +
+                         "skipping and not using").format(self.repourl))
                 log.err()
 
     def describe(self):
-        return "SVNPoller: watching %s" % self.svnurl
+        return "SVNPoller: watching {}".format(self.repourl)
 
     def poll(self):
         # Our return value is only used for unit testing.
 
         # we need to figure out the repository root, so we can figure out
-        # repository-relative pathnames later. Each SVNURL is in the form
+        # repository-relative pathnames later. Each REPOURL is in the form
         # (ROOT)/(PROJECT)/(BRANCH)/(FILEPATH), where (ROOT) is something
         # like svn://svn.twistedmatrix.com/svn/Twisted (i.e. there is a
         # physical repository at /svn/Twisted on that host), (PROJECT) is
@@ -142,17 +153,17 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
         # "trunk" or "branches/2.0.x", and (FILEPATH) is a tree-relative
         # filename like "twisted/internet/defer.py".
 
-        # our self.svnurl attribute contains (ROOT)/(PROJECT) combined
+        # our self.repourl attribute contains (ROOT)/(PROJECT) combined
         # together in a way that we can't separate without svn's help. If the
-        # user is not using the split_file= argument, then self.svnurl might
+        # user is not using the split_file= argument, then self.repourl might
         # be (ROOT)/(PROJECT)/(BRANCH) . In any case, the filenames we will
         # get back from 'svn log' will be of the form
         # (PROJECT)/(BRANCH)/(FILEPATH), but we want to be able to remove
         # that (PROJECT) prefix from them. To do this without requiring the
-        # user to tell us how svnurl is split into ROOT and PROJECT, we do an
+        # user to tell us how repourl is split into ROOT and PROJECT, we do an
         # 'svn info --xml' command at startup. This command will include a
         # <root> element that tells us ROOT. We then strip this prefix from
-        # self.svnurl to determine PROJECT, and then later we strip the
+        # self.repourl to determine PROJECT, and then later we strip the
         # PROJECT prefix from the filenames reported by 'svn log --xml' to
         # get a (BRANCH)/(FILEPATH) that can be passed to split_file() to
         # turn into separate BRANCH and FILEPATH values.
@@ -166,10 +177,11 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
 
         d = defer.succeed(None)
         if not self._prefix:
-            d.addCallback(lambda _ : self.get_prefix())
+            d.addCallback(lambda _: self.get_prefix())
+
+            @d.addCallback
             def set_prefix(prefix):
                 self._prefix = prefix
-            d.addCallback(set_prefix)
 
         d.addCallback(self.get_logs)
         d.addCallback(self.parse_logs)
@@ -177,7 +189,8 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
         d.addCallback(self.create_changes)
         d.addCallback(self.submit_changes)
         d.addCallback(self.finished_ok)
-        d.addErrback(log.err, 'SVNPoller: Error in  while polling') # eat errors
+        # eat errors
+        d.addErrback(log.err, 'SVNPoller: Error in  while polling')
         return d
 
     def getProcessOutput(self, args):
@@ -186,20 +199,21 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
         return d
 
     def get_prefix(self):
-        args = ["info", "--xml", "--non-interactive", self.svnurl]
+        args = ["info", "--xml", "--non-interactive", self.repourl]
         if self.svnuser:
-            args.extend(["--username=%s" % self.svnuser])
-        if self.svnpasswd:
-            args.extend(["--password=%s" % self.svnpasswd])
+            args.append("--username={}".format(self.svnuser))
+        if self.svnpasswd is not None:
+            args.append("--password={}".format(self.svnpasswd))
         if self.extra_args:
             args.extend(self.extra_args)
         d = self.getProcessOutput(args)
+
+        @d.addCallback
         def determine_prefix(output):
             try:
                 doc = xml.dom.minidom.parseString(output)
             except xml.parsers.expat.ExpatError:
-                log.msg("SVNPoller: SVNPoller._determine_prefix_2: ExpatError in '%s'"
-                        % output)
+                log.msg("SVNPoller: SVNPoller.get_prefix: ExpatError in '{}'".format(output))
                 raise
             rootnodes = doc.getElementsByTagName("root")
             if not rootnodes:
@@ -210,30 +224,29 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
             rootnode = rootnodes[0]
             root = "".join([c.data for c in rootnode.childNodes])
             # root will be a unicode string
-            if not self.svnurl.startswith(root):
-                log.msg(format="svnurl='%(svnurl)s' doesn't start with <root>='%(root)s'",
-                        svnurl=self.svnurl, root=root)
-                raise RuntimeError("Can't handle redirected svn connections!? "
-                        "This shouldn't happen.")
-            prefix = self.svnurl[len(root):]
+            if not self.repourl.startswith(root):
+                log.msg(format="Got root %(root)r from `svn info`, but it is "
+                               "not a prefix of the configured repourl",
+                        repourl=self.repourl, root=root)
+                raise RuntimeError("Configured repourl doesn't match svn root")
+            prefix = self.repourl[len(root):]
             if prefix.startswith("/"):
                 prefix = prefix[1:]
-            log.msg("SVNPoller: svnurl=%s, root=%s, so prefix=%s" %
-                    (self.svnurl, root, prefix))
+            log.msg("SVNPoller: repourl={}, root={}, so prefix={}".format(self.repourl, root,
+                                                                          prefix))
             return prefix
-        d.addCallback(determine_prefix)
         return d
 
     def get_logs(self, _):
         args = []
         args.extend(["log", "--xml", "--verbose", "--non-interactive"])
         if self.svnuser:
-            args.extend(["--username=%s" % self.svnuser])
-        if self.svnpasswd:
-            args.extend(["--password=%s" % self.svnpasswd])
+            args.extend(["--username={}".format(self.svnuser)])
+        if self.svnpasswd is not None:
+            args.extend(["--password={}".format(self.svnpasswd)])
         if self.extra_args:
             args.extend(self.extra_args)
-        args.extend(["--limit=%d" % (self.histmax), self.svnurl])
+        args.extend(["--limit=%d" % (self.histmax), self.repourl])
         d = self.getProcessOutput(args)
         return d
 
@@ -242,12 +255,11 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
         try:
             doc = xml.dom.minidom.parseString(output)
         except xml.parsers.expat.ExpatError:
-            log.msg("SVNPoller: SVNPoller.parse_logs: ExpatError in '%s'" % output)
+            log.msg("SVNPoller: SVNPoller.parse_logs: ExpatError in '{}'".format(output))
             raise
         logentries = doc.getElementsByTagName("logentry")
         return logentries
 
-
     def get_new_logentries(self, logentries):
         last_change = old_last_change = self.last_change
 
@@ -264,7 +276,7 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
                 # if this is the first time we've been run, ignore any changes
                 # that occurred before now. This prevents a build at every
                 # startup.
-                log.msg('SVNPoller: starting at change %s' % new_last_change)
+                log.msg('SVNPoller: starting at change {}'.format(new_last_change))
             elif last_change == new_last_change:
                 # an unmodified repository will hit this case
                 log.msg('SVNPoller: no changes')
@@ -273,20 +285,18 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
                     if last_change == int(el.getAttribute("revision")):
                         break
                     new_logentries.append(el)
-                new_logentries.reverse() # return oldest first
+                new_logentries.reverse()  # return oldest first
 
         self.last_change = new_last_change
-        log.msg('SVNPoller: _process_changes %s .. %s' %
-                (old_last_change, new_last_change))
+        log.msg('SVNPoller: _process_changes {} .. {}'.format(old_last_change, new_last_change))
         return new_logentries
 
-
     def _get_text(self, element, tag_name):
         try:
             child_nodes = element.getElementsByTagName(tag_name)[0].childNodes
             text = "".join([t.data for t in child_nodes])
-        except:
-            text = "<unknown>"
+        except IndexError:
+            text = "unknown"
         return text
 
     def _transform_path(self, path):
@@ -294,14 +304,14 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
             log.msg(format="SVNPoller: ignoring path '%(path)s' which doesn't"
                     "start with prefix '%(prefix)s'",
                     path=path, prefix=self._prefix)
-            return
+            return None
         relative_path = path[len(self._prefix):]
         if relative_path.startswith("/"):
             relative_path = relative_path[1:]
         where = self.split_file(relative_path)
         # 'where' is either None, (branch, final_path) or a dict
         if not where:
-            return
+            return None
         if isinstance(where, tuple):
             where = dict(branch=where[0], path=where[1])
         return where
@@ -312,14 +322,14 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
         for el in new_logentries:
             revision = str(el.getAttribute("revision"))
 
-            revlink=''
+            revlink = ''
 
-            if self.revlinktmpl:
-                if revision:
-                    revlink = self.revlinktmpl % urllib.quote_plus(revision)
+            if self.revlinktmpl and revision:
+                revlink = self.revlinktmpl % urlquote_plus(revision)
+                revlink = str(revlink)
 
-            log.msg("Adding change revision %s" % (revision,))
-            author   = self._get_text(el, "author")
+            log.msg("Adding change revision {}".format(revision))
+            author = self._get_text(el, "author")
             comments = self._get_text(el, "msg")
             # there is a "date" field, but it provides localtime in the
             # repository's timezone, whereas we care about buildmaster's
@@ -329,7 +339,7 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
             branches = {}
             try:
                 pathlist = el.getElementsByTagName("paths")[0]
-            except IndexError: # weird, we got an empty revision
+            except IndexError:  # weird, we got an empty revision
                 log.msg("ignoring commit with no paths")
                 continue
 
@@ -337,11 +347,6 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
                 kind = p.getAttribute("kind")
                 action = p.getAttribute("action")
                 path = "".join([t.data for t in p.childNodes])
-                # the rest of buildbot is certaily not yet ready to handle
-                # unicode filenames, because they get put in RemoteCommands
-                # which get sent via PB to the buildslave, and PB doesn't
-                # handle unicode.
-                path = path.encode("ascii")
                 if path.startswith("/"):
                     path = path[1:]
                 if kind == "dir" and not path.endswith("/"):
@@ -353,8 +358,9 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
                 if where:
                     branch = where.get("branch", None)
                     filename = where["path"]
-                    if not branch in branches:
-                        branches[branch] = { 'files': [], 'number_of_directories': 0}
+                    if branch not in branches:
+                        branches[branch] = {
+                            'files': [], 'number_of_directories': 0}
                     if filename == "":
                         # root directory of branch
                         branches[branch]['files'].append(filename)
@@ -366,34 +372,42 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
                     else:
                         branches[branch]['files'].append(filename)
 
-                    if not branches[branch].has_key('action'):
+                    if "action" not in branches[branch]:
                         branches[branch]['action'] = action
 
                     for key in ("repository", "project", "codebase"):
                         if key in where:
                             branches[branch][key] = where[key]
 
-            for branch in branches.keys():
+            for branch in branches:
                 action = branches[branch]['action']
-                files  = branches[branch]['files']
+                files = branches[branch]['files']
 
-                number_of_directories_changed = branches[branch]['number_of_directories']
+                number_of_directories_changed = branches[
+                    branch]['number_of_directories']
                 number_of_files_changed = len(files)
 
-                if action == u'D' and number_of_directories_changed == 1 and number_of_files_changed == 1 and files[0] == '':
-                    log.msg("Ignoring deletion of branch '%s'" % branch)
+                if (action == 'D' and number_of_directories_changed == 1 and
+                        number_of_files_changed == 1 and files[0] == ''):
+                    log.msg("Ignoring deletion of branch '{}'".format(branch))
                 else:
                     chdict = dict(
-                            author=author,
-                            files=files,
-                            comments=comments,
-                            revision=revision,
-                            branch=branch,
-                            revlink=revlink,
-                            category=self.category,
-                            repository=branches[branch].get('repository', self.svnurl),
-                            project=branches[branch].get('project', self.project),
-                            codebase=branches[branch].get('codebase', None))
+                        author=author,
+                        committer=None,
+                        # weakly assume filenames are utf-8
+                        files=[bytes2unicode(f, 'utf-8', 'replace')
+                               for f in files],
+                        comments=comments,
+                        revision=revision,
+                        branch=util.bytes2unicode(branch),
+                        revlink=revlink,
+                        category=self.category,
+                        repository=util.bytes2unicode(
+                            branches[branch].get('repository', self.repourl)),
+                        project=util.bytes2unicode(
+                            branches[branch].get('project', self.project)),
+                        codebase=util.bytes2unicode(
+                            branches[branch].get('codebase', None)))
                     changes.append(chdict)
 
         return changes
@@ -401,12 +415,12 @@ class SVNPoller(base.PollingChangeSource, util.ComparableMixin):
     @defer.inlineCallbacks
     def submit_changes(self, changes):
         for chdict in changes:
-            yield self.master.addChange(src='svn', **chdict)
+            yield self.master.data.updates.addChange(src='svn', **chdict)
 
     def finished_ok(self, res):
         if self.cachepath:
             with open(self.cachepath, "w") as f:
                 f.write(str(self.last_change))
 
-        log.msg("SVNPoller: finished polling %s" % res)
+        log.msg("SVNPoller: finished polling {}".format(res))
         return res
