@@ -55,24 +55,42 @@ class ChangeSource(service.ClusteredBuildbotService):
 class ReconfigurablePollingChangeSource(ChangeSource):
     pollInterval = None
     pollAtLaunch = None
+    pollRandomDelayMin = None
+    pollRandomDelayMax = None
 
-    def checkConfig(self, name=None, pollInterval=60 * 10, pollAtLaunch=False):
-        ChangeSource.checkConfig(self, name=name)
+    def checkConfig(self, name=None, pollInterval=60 * 10, pollAtLaunch=False,
+                    pollRandomDelayMin=0, pollRandomDelayMax=0):
+        super().checkConfig(name=name)
         if pollInterval < 0:
             config.error("interval must be >= 0: {}".format(pollInterval))
+        if pollRandomDelayMin < 0:
+            config.error("min random delay must be >= 0: {}".format(pollRandomDelayMin))
+        if pollRandomDelayMax < 0:
+            config.error("max random delay must be >= 0: {}".format(pollRandomDelayMax))
+        if pollRandomDelayMin > pollRandomDelayMax:
+            config.error("min random delay must be <= {}: {}".format(pollRandomDelayMax,
+                                                                     pollRandomDelayMin))
+        if pollRandomDelayMax >= pollInterval:
+            config.error("max random delay must be < {}: {}".format(pollInterval,
+                                                                    pollRandomDelayMax))
 
     @defer.inlineCallbacks
-    def reconfigService(self, name=None, pollInterval=60 * 10, pollAtLaunch=False):
+    def reconfigService(self, name=None, pollInterval=60 * 10, pollAtLaunch=False,
+                        pollRandomDelayMin=0, pollRandomDelayMax=0):
         self.pollInterval, prevPollInterval = pollInterval, self.pollInterval
         self.pollAtLaunch = pollAtLaunch
-        yield ChangeSource.reconfigService(self, name=name)
+        self.pollRandomDelayMin = pollRandomDelayMin
+        self.pollRandomDelayMax = pollRandomDelayMax
+        yield super().reconfigService(name=name)
 
         # pollInterval change is the only value which makes sense to reconfigure check.
         if prevPollInterval != pollInterval and self.doPoll.started:
             yield self.doPoll.stop()
             # As a implementation detail, poller will 'pollAtReconfigure' if poll interval changes
             # and pollAtLaunch=True
-            yield self.doPoll.start(interval=self.pollInterval, now=self.pollAtLaunch)
+            yield self.doPoll.start(interval=self.pollInterval, now=self.pollAtLaunch,
+                                    random_delay_min=self.pollRandomDelayMin,
+                                    random_delay_max=self.pollRandomDelayMax)
 
     def poll(self):
         pass
@@ -87,7 +105,9 @@ class ReconfigurablePollingChangeSource(ChangeSource):
         self.doPoll()
 
     def activate(self):
-        self.doPoll.start(interval=self.pollInterval, now=self.pollAtLaunch)
+        self.doPoll.start(interval=self.pollInterval, now=self.pollAtLaunch,
+                          random_delay_min=self.pollRandomDelayMin,
+                          random_delay_max=self.pollRandomDelayMax)
 
     def deactivate(self):
         return self.doPoll.stop()
@@ -95,13 +115,17 @@ class ReconfigurablePollingChangeSource(ChangeSource):
 
 class PollingChangeSource(ReconfigurablePollingChangeSource):
     # Legacy code will be very painful to port to BuildbotService life cycle
-    # because the unit tests keep doing shortcuts for the Service life cycle (i.e by no calling startService)
-    # instead of porting everything at once, we make a class to support legacy
+    # because the unit tests keep doing shortcuts for the Service life cycle (i.e by no calling
+    # startService) instead of porting everything at once, we make a class to support legacy
 
-    def checkConfig(self, name=None, pollInterval=60 * 10, pollAtLaunch=False, **kwargs):
-        ReconfigurablePollingChangeSource.checkConfig(self, name=name, pollInterval=60 * 10, pollAtLaunch=False)
+    def checkConfig(self, name=None, pollInterval=60 * 10, pollAtLaunch=False,
+                    pollRandomDelayMin=0, pollRandomDelayMax=0, **kwargs):
+        super().checkConfig(name=name, pollInterval=60 * 10, pollAtLaunch=False,
+                            pollRandomDelayMin=0, pollRandomDelayMax=0)
         self.pollInterval = pollInterval
         self.pollAtLaunch = pollAtLaunch
+        self.pollRandomDelayMin = pollRandomDelayMin
+        self.pollRandomDelayMax = pollRandomDelayMax
 
     def reconfigService(self, *args, **kwargs):
         # BuildbotServiceManager will detect such exception and swap old service with new service,
