@@ -22,7 +22,7 @@ from twisted.internet import defer
 from buildbot.data import exceptions
 
 
-class ResourceType(object):
+class ResourceType:
     name = None
     plural = None
     endpoints = []
@@ -69,7 +69,7 @@ class ResourceType(object):
                 self.master.mq.produce(routingKey, msg)
 
 
-class Endpoint(object):
+class Endpoint:
     pathPatterns = ""
     rootLinkName = None
     isCollection = False
@@ -93,7 +93,7 @@ class Endpoint(object):
         return "endpoint for " + self.pathPatterns
 
 
-class BuildNestingMixin(object):
+class BuildNestingMixin:
 
     """
     A mixin for methods to decipher the many ways a build, step, or log can be
@@ -109,12 +109,12 @@ class BuildNestingMixin(object):
         else:
             builderid = yield self.getBuilderId(kwargs)
             if builderid is None:
-                return
+                return None
             build = yield self.master.db.builds.getBuildByNumber(
                 builderid=builderid,
                 number=kwargs['build_number'])
             if not build:
-                return
+                return None
             return build['id']
 
     @defer.inlineCallbacks
@@ -124,14 +124,14 @@ class BuildNestingMixin(object):
         else:
             buildid = yield self.getBuildid(kwargs)
             if buildid is None:
-                return
+                return None
 
             dbdict = yield self.master.db.steps.getStep(buildid=buildid,
                                                         number=kwargs.get(
                                                             'step_number'),
                                                         name=kwargs.get('step_name'))
             if not dbdict:
-                return
+                return None
             return dbdict['id']
 
     def getBuilderId(self, kwargs):
@@ -146,7 +146,7 @@ class ListResult(UserList):
 
     def __init__(self, values,
                  offset=None, total=None, limit=None):
-        UserList.__init__(self, values)
+        super().__init__(values)
 
         # if set, this is the index in the overall results of the first element of
         # this list
