@@ -22,7 +22,7 @@ from buildbot.data import types
 from buildbot.db.changesources import ChangeSourceAlreadyClaimedError
 
 
-class Db2DataMixin(object):
+class Db2DataMixin:
 
     @defer.inlineCallbacks
     def db2data(self, dbdict):
@@ -51,7 +51,7 @@ class ChangeSourceEndpoint(Db2DataMixin, base.Endpoint):
             kwargs['changesourceid'])
         if 'masterid' in kwargs:
             if dbdict['masterid'] != kwargs['masterid']:
-                return
+                return None
         return (yield self.db2data(dbdict)) if dbdict else None
 
 
