@@ -14,9 +14,6 @@
 # Copyright Buildbot Team Members
 
 # See "Type Validation" in master/docs/developer/tests.rst
-from future.utils import integer_types
-from future.utils import text_type
-
 import datetime
 import json
 import re
@@ -25,7 +22,7 @@ from buildbot import util
 from buildbot.util import bytes2unicode
 
 
-class Type(object):
+class Type:
 
     name = None
     doc = None
@@ -102,8 +99,7 @@ class Instance(Type):
 
     def validate(self, name, object):
         if not isinstance(object, self.types):
-            yield "%s (%r) is not a %s" % (
-                name, object, self.name or repr(self.types))
+            yield "{} ({}) is not a {}".format(name, repr(object), self.name or repr(self.types))
 
     def toRaml(self):
         return self.ramlType
@@ -112,7 +108,7 @@ class Instance(Type):
 class Integer(Instance):
 
     name = "integer"
-    types = integer_types
+    types = (int,)
     ramlType = "integer"
 
     def valueFromString(self, arg):
@@ -120,15 +116,32 @@ class Integer(Instance):
 
 
 class DateTime(Instance):
+
     name = "datetime"
-    types = (datetime.datetime)
+    types = (datetime.datetime,)
+
+    def valueFromString(self, arg):
+        return int(arg)
+
+    def validate(self, name, object):
+        if isinstance(object, datetime.datetime):
+            return
+        if isinstance(object, int):
+            try:
+                datetime.datetime.fromtimestamp(object)
+            except (OverflowError, OSError):
+                pass
+            else:
+                return
+        yield "{} ({}) is not a valid timestamp".format(name, object)
+
     ramlType = "date"
 
 
 class String(Instance):
 
     name = "string"
-    types = (text_type,)
+    types = (str,)
     ramlType = "string"
 
     def valueFromString(self, arg):
@@ -159,11 +172,11 @@ class Boolean(Instance):
 class Identifier(Type):
 
     name = "identifier"
-    identRe = re.compile('^[a-zA-Z_-][a-zA-Z0-9_-]*$')
+    identRe = re.compile('^[a-zA-Z_-][a-zA-Z0-9._-]*$')
     ramlType = "string"
 
     def __init__(self, len=None, **kwargs):
-        Type.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.len = len
 
     def valueFromString(self, arg):
@@ -173,15 +186,14 @@ class Identifier(Type):
         return val
 
     def validate(self, name, object):
-        if not isinstance(object, text_type):
-            yield "%s - %r - is not a unicode string" % (name, object)
+        if not isinstance(object, str):
+            yield "{} - {} - is not a unicode string".format(name, repr(object))
         elif not self.identRe.match(object):
-            yield "%s - %r - is not an identifier" % (name, object)
+            yield "{} - {} - is not an identifier".format(name, repr(object))
         elif not object:
-            yield "%s - identifiers cannot be an empty string" % (name,)
+            yield "{} - identifiers cannot be an empty string".format(name)
         elif len(object) > self.len:
-            yield "%s - %r - is longer than %d characters" % (name, object,
-                                                              self.len)
+            yield "{} - {} - is longer than {} characters".format(name, repr(object), self.len)
 
     def toRaml(self):
         return {'type': self.ramlType,
@@ -198,16 +210,16 @@ class List(Type):
         return self.of.ramlname
 
     def __init__(self, of=None, **kwargs):
-        Type.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self.of = of
 
     def validate(self, name, object):
         if not isinstance(object, list):  # we want a list, and NOT a subclass
-            yield "%s (%r) is not a %s" % (name, object, self.name)
+            yield "{} ({}) is not a {}".format(name, repr(object), self.name)
             return
 
         for idx, elt in enumerate(object):
-            for msg in self.of.validate("%s[%d]" % (name, idx), elt):
+            for msg in self.of.validate("{}[{}]".format(name, idx), elt):
                 yield msg
 
     def valueFromString(self, arg):
@@ -237,21 +249,21 @@ class SourcedProperties(Type):
 
     def validate(self, name, object):
         if not isinstance(object, dict):  # we want a dict, and NOT a subclass
-            yield "%s is not sourced properties (not a dict)" % (name,)
+            yield "{} is not sourced properties (not a dict)".format(name)
             return
         for k, v in object.items():
-            if not isinstance(k, text_type):
-                yield "%s property name %r is not unicode" % (name, k)
+            if not isinstance(k, str):
+                yield "{} property name {} is not unicode".format(name, repr(k))
             if not isinstance(v, tuple) or len(v) != 2:
-                yield "%s property value for '%s' is not a 2-tuple" % (name, k)
+                yield "{} property value for '{}' is not a 2-tuple".format(name, k)
                 return
             propval, propsrc = v
-            if not isinstance(propsrc, text_type):
-                yield "%s[%s] source %r is not unicode" % (name, k, propsrc)
+            if not isinstance(propsrc, str):
+                yield "{}[{}] source {} is not unicode".format(name, k, repr(propsrc))
             try:
                 json.loads(bytes2unicode(propval))
             except ValueError:
-                yield "%s[%r] value is not JSON-able" % (name, k)
+                yield "{}[{}] value is not JSON-able".format(name, repr(k))
 
     def toRaml(self):
         return {'type': "object",
@@ -277,25 +289,23 @@ class Dict(Type):
 
     def validate(self, name, object):
         if not isinstance(object, dict):
-            yield "%s (%r) is not a dictionary (got type %s)" \
-                % (name, object, type(object))
+            yield "{} ({}) is not a dictionary (got type {})".format(name, repr(object),
+                                                                     type(object))
             return
 
         gotNames = set(object.keys())
 
         unexpected = gotNames - self.keys
         if unexpected:
-            yield "%s has unexpected keys %s" % (name,
-                                                 ", ".join([repr(n) for n in unexpected]))
+            yield "{} has unexpected keys {}".format(name, ", ".join([repr(n) for n in unexpected]))
 
         missing = self.keys - gotNames
         if missing:
-            yield "%s is missing keys %s" % (name,
-                                             ", ".join([repr(n) for n in missing]))
+            yield "{} is missing keys {}".format(name, ", ".join([repr(n) for n in missing]))
 
         for k in gotNames & self.keys:
             f = self.contents[k]
-            for msg in f.validate("%s[%r]" % (name, k), object[k]):
+            for msg in f.validate("{}[{}]".format(name, repr(k)), object[k]):
                 yield msg
 
     def getSpec(self):
@@ -317,15 +327,15 @@ class JsonObject(Type):
 
     def validate(self, name, object):
         if not isinstance(object, dict):
-            yield "%s (%r) is not a dictionary (got type %s)" \
-                % (name, object, type(object))
+            yield "{} ({}) is not a dictionary (got type {})".format(name, repr(object),
+                                                                     type(object))
             return
 
         # make sure JSON can represent it
         try:
             json.dumps(object)
         except Exception as e:
-            yield "%s is not JSON-able: %s" % (name, e)
+            yield "{} is not JSON-able: {}".format(name, e)
             return
 
     def toRaml(self):
@@ -355,25 +365,23 @@ class Entity(Type):
     def validate(self, name, object):
         # this uses isinstance, allowing dict subclasses as used by the DB API
         if not isinstance(object, dict):
-            yield "%s (%r) is not a dictionary (got type %s)" \
-                % (name, object, type(object))
+            yield "{} ({}) is not a dictionary (got type {})".format(name, repr(object),
+                                                                     type(object))
             return
 
         gotNames = set(object.keys())
 
         unexpected = gotNames - self.fieldNames
         if unexpected:
-            yield "%s has unexpected keys %s" % (name,
-                                                 ", ".join([repr(n) for n in unexpected]))
+            yield "{} has unexpected keys {}".format(name, ", ".join([repr(n) for n in unexpected]))
 
         missing = self.fieldNames - gotNames
         if missing:
-            yield "%s is missing keys %s" % (name,
-                                             ", ".join([repr(n) for n in missing]))
+            yield "{} is missing keys {}".format(name, ", ".join([repr(n) for n in missing]))
 
         for k in gotNames & self.fieldNames:
             f = self.fields[k]
-            for msg in f.validate("%s[%r]" % (name, k), object[k]):
+            for msg in f.validate("{}[{}]".format(name, repr(k)), object[k]):
                 yield msg
 
     def getSpec(self):
