@@ -22,7 +22,7 @@ from buildbot.data import types
 from buildbot.util import identifiers
 
 
-class Db2DataMixin(object):
+class Db2DataMixin:
 
     def db2data(self, dbdict):
         return {
@@ -64,6 +64,7 @@ class WorkerEndpoint(Db2DataMixin, base.Endpoint):
             builderid=kwargs.get('builderid'))
         if sldict:
             return self.db2data(sldict)
+        return None
 
     @defer.inlineCallbacks
     def control(self, action, args, kwargs):
