@@ -19,7 +19,6 @@ import json
 import sqlalchemy as sa
 
 from twisted.internet import defer
-from twisted.internet import reactor
 
 from buildbot.db import NULL
 from buildbot.db import base
@@ -73,6 +72,7 @@ class BuildsConnectorComponent(base.DBConnectorComponent):
         rv = None
         tbl = self.db.model.builds
         offset = 0
+        increment = 1000
         matchssBuild = {(ss['repository'],
                          ss['branch'],
                          ss['codebase']) for ss in ssBuild}
@@ -82,7 +82,7 @@ class BuildsConnectorComponent(base.DBConnectorComponent):
                                                                   (tbl.c.number < number) &
                                                                   (tbl.c.results == 0)),
                                                      offset=offset,
-                                                     limit=10)
+                                                     limit=increment)
             if not prevBuilds:
                 break
             for prevBuild in prevBuilds:
@@ -94,12 +94,41 @@ class BuildsConnectorComponent(base.DBConnectorComponent):
                     # repository/branch/codebase was found !
                     rv = prevBuild
                     break
-            offset += 10
+            offset += increment
 
         return rv
 
+    def getBuildsForChange(self, changeid):
+        assert changeid > 0
+
+        def thd(conn):
+            # Get builds for the change
+            changes_tbl = self.db.model.changes
+            bsets_tbl = self.db.model.buildsets
+            bsss_tbl = self.db.model.buildset_sourcestamps
+            reqs_tbl = self.db.model.buildrequests
+            builds_tbl = self.db.model.builds
+
+            from_clause = changes_tbl.join(bsss_tbl,
+                                           changes_tbl.c.sourcestampid == bsss_tbl.c.sourcestampid)
+            from_clause = from_clause.join(bsets_tbl,
+                                           bsss_tbl.c.buildsetid == bsets_tbl.c.id)
+            from_clause = from_clause.join(reqs_tbl,
+                                           bsets_tbl.c.id == reqs_tbl.c.buildsetid)
+            from_clause = from_clause.join(builds_tbl,
+                                           reqs_tbl.c.id == builds_tbl.c.buildrequestid)
+
+            q = sa.select([builds_tbl]).select_from(
+                from_clause).where(changes_tbl.c.changeid == changeid)
+            res = conn.execute(q)
+            return [self._builddictFromRow(row)
+                    for row in res.fetchall()]
+
+        return self.db.pool.do(thd)
+
     # returns a Deferred that returns a value
-    def getBuilds(self, builderid=None, buildrequestid=None, workerid=None, complete=None, resultSpec=None):
+    def getBuilds(self, builderid=None, buildrequestid=None, workerid=None, complete=None,
+                  resultSpec=None):
         def thd(conn):
             tbl = self.db.model.builds
             q = tbl.select()
@@ -125,8 +154,8 @@ class BuildsConnectorComponent(base.DBConnectorComponent):
 
     # returns a Deferred that returns a value
     def addBuild(self, builderid, buildrequestid, workerid, masterid,
-                 state_string, _reactor=reactor, _race_hook=None):
-        started_at = _reactor.seconds()
+                 state_string, _race_hook=None):
+        started_at = int(self.master.reactor.seconds())
 
         def thd(conn):
             tbl = self.db.model.builds
@@ -166,12 +195,12 @@ class BuildsConnectorComponent(base.DBConnectorComponent):
         return self.db.pool.do(thd)
 
     # returns a Deferred that returns None
-    def finishBuild(self, buildid, results, _reactor=reactor):
+    def finishBuild(self, buildid, results):
         def thd(conn):
             tbl = self.db.model.builds
             q = tbl.update(whereclause=(tbl.c.id == buildid))
             conn.execute(q,
-                         complete_at=_reactor.seconds(),
+                         complete_at=int(self.master.reactor.seconds()),
                          results=results)
         return self.db.pool.do(thd)
 
@@ -214,10 +243,6 @@ class BuildsConnectorComponent(base.DBConnectorComponent):
         yield self.db.pool.do(thd)
 
     def _builddictFromRow(self, row):
-        def mkdt(epoch):
-            if epoch:
-                return epoch2datetime(epoch)
-
         return dict(
             id=row.id,
             number=row.number,
@@ -225,7 +250,7 @@ class BuildsConnectorComponent(base.DBConnectorComponent):
             buildrequestid=row.buildrequestid,
             workerid=row.workerid,
             masterid=row.masterid,
-            started_at=mkdt(row.started_at),
-            complete_at=mkdt(row.complete_at),
+            started_at=epoch2datetime(row.started_at),
+            complete_at=epoch2datetime(row.complete_at),
             state_string=row.state_string,
             results=row.results)
