@@ -23,6 +23,8 @@ Define the interfaces that are implemented by various buildbot classes.
 # pylint: disable=no-method-argument
 # pylint: disable=inherit-non-class
 
+from twisted.python.deprecate import deprecatedModuleAttribute
+from twisted.python.versions import Version
 from zope.interface import Attribute
 from zope.interface import Interface
 
@@ -33,10 +35,19 @@ class BuilderInUseError(Exception):
     pass
 
 
-class WorkerTooOldError(Exception):
+class WorkerSetupError(Exception):
     pass
 
 
+WorkerTooOldError = WorkerSetupError
+deprecatedModuleAttribute(
+    Version("buildbot", 2, 9, 0),
+    message="Use WorkerSetupError instead.",
+    moduleName="buildbot.interfaces",
+    name="WorkerTooOldError",
+)
+
+
 class LatentWorkerFailedToSubstantiate(Exception):
     pass
 
@@ -329,6 +340,12 @@ class IConfigured(Interface):
         pass
 
 
+class IReportGenerator(Interface):
+
+    def generate(self, master, reporter, key, build):
+        pass
+
+
 # #################### Deprecated Status Interfaces   ####################
 
 
@@ -1062,8 +1079,10 @@ class IHttpResponse(Interface):
         """
         :returns: json decoded content of the response via deferred
         """
-    master = Attribute('code',
-                       "http status code of the request's response (e.g 200)")
+    code = Attribute('code',
+                     "http status code of the request's response (e.g 200)")
+    url = Attribute('url',
+                    "request's url (e.g https://api.github.com/endpoint')")
 
 
 class IConfigurator(Interface):
@@ -1072,8 +1091,8 @@ class IConfigurator(Interface):
         """
         Alter the buildbot config_dict, as defined in master.cfg
 
-        like the master.cfg, this is run out of the main reactor thread, so this can block, but this can't
-        call most Buildbot facilities.
+        like the master.cfg, this is run out of the main reactor thread, so this can block, but
+        this can't call most Buildbot facilities.
 
         :returns: None
         """
