@@ -43,7 +43,7 @@ from buildbot.process.botmaster import BotMaster
 from buildbot.process.users.manager import UserManagerManager
 from buildbot.schedulers.manager import SchedulerManager
 from buildbot.secrets.manager import SecretManager
-from buildbot.status.master import Status
+from buildbot.status.master_compat import Status
 from buildbot.util import check_functional_environment
 from buildbot.util import service
 from buildbot.util.eventual import eventually
@@ -125,8 +125,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
             self.hostname = socket.getfqdn()
 
         # public attributes
-        self.name = ("%s:%s" % (self.hostname,
-                                os.path.abspath(self.basedir or '.')))
+        self.name = ("{}:{}".format(self.hostname, os.path.abspath(self.basedir or '.')))
         if isinstance(self.name, bytes):
             self.name = self.name.decode('ascii', 'replace')
         self.masterid = None
@@ -220,8 +219,7 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
             yield self._services_d
             self._services_d = None
 
-        log.msg("Starting BuildMaster -- buildbot.version: %s" %
-                buildbot.version)
+        log.msg("Starting BuildMaster -- buildbot.version: {}".format(buildbot.version))
 
         # Set umask
         if self.umask is not None:
@@ -270,11 +268,13 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
 
             yield self.mq.setup()
 
+            # the buildbot scripts send the SIGHUP signal to reconfig master
             if hasattr(signal, "SIGHUP"):
                 def sighup(*args):
                     eventually(self.reconfig)
                 signal.signal(signal.SIGHUP, sighup)
 
+            # the buildbot scripts send the SIGUSR1 signal to stop master
             if hasattr(signal, "SIGUSR1"):
                 def sigusr1(*args):
                     eventually(self.botmaster.cleanShutdown)
@@ -364,9 +364,9 @@ class BuildMaster(service.ReconfigurableServiceMixin, service.MasterService):
 
         # notify every 10 seconds that the reconfig is still going on, although
         # reconfigs should not take that long!
-        self.reconfig_notifier = task.LoopingCall(lambda:
-                                                  log.msg("reconfig is ongoing for %d s" %
-                                                          (self.reactor.seconds() - self.reconfig_active)))
+        self.reconfig_notifier = task.LoopingCall(
+            lambda: log.msg("reconfig is ongoing for {} s".format(self.reactor.seconds() -
+                                                                  self.reconfig_active)))
         self.reconfig_notifier.start(10, now=False)
 
         timer = metrics.Timer("BuildMaster.reconfig")
