@@ -15,8 +15,6 @@
 #
 # pylint: disable=C0111
 
-from future.utils import string_types
-
 import traceback
 from pkg_resources import iter_entry_points
 
@@ -30,7 +28,7 @@ from buildbot.interfaces import IPlugin
 _NAMESPACE_BASE = 'buildbot'
 
 
-class _PluginEntry(object):
+class _PluginEntry:
 
     def __init__(self, group, entry, loader):
         self._group = group
@@ -96,7 +94,7 @@ class _PluginEntryProxy(_PluginEntry):
         return self._plugin_entry.value
 
 
-class _NSNode(object):
+class _NSNode:
     # pylint: disable=W0212
 
     def __init__(self):
@@ -107,8 +105,7 @@ class _NSNode(object):
             child.load()
 
     def add(self, name, entry):
-        assert isinstance(name, string_types) and isinstance(entry,
-                                                             _PluginEntry)
+        assert isinstance(name, str) and isinstance(entry, _PluginEntry)
         self._add(name, entry)
 
     def _add(self, name, entry):
@@ -121,11 +118,10 @@ class _NSNode(object):
             if child is not None:
                 assert isinstance(child, _PluginEntry)
                 if child != entry:
-                    raise PluginDBError('Duplicate entry point for "%s:%s".\n'
-                                        '  Previous definition %s\n'
-                                        '  This definition %s' %
-                                        (child.group, child.name, child.info,
-                                         entry.info))
+                    raise PluginDBError(('Duplicate entry point for "{}:{}".\n'
+                                         '  Previous definition {}\n'
+                                         '  This definition {}').format(child.group, child.name,
+                                                                        child.info, entry.info))
             else:
                 self._children[key] = entry
         else:
@@ -138,19 +134,19 @@ class _NSNode(object):
     def __getattr__(self, name):
         child = self._children.get(name)
         if child is None:
-            raise PluginDBError('Unknown component name: %s' % name)
+            raise PluginDBError('Unknown component name: {}'.format(name))
 
         if isinstance(child, _PluginEntry):
             return child.value
         return child
 
     def info(self, name):
-        assert isinstance(name, string_types)
+        assert isinstance(name, str)
 
         return self._get(name).info
 
     def get(self, name):
-        assert isinstance(name, string_types)
+        assert isinstance(name, str)
 
         return self._get(name).value
 
@@ -162,11 +158,10 @@ class _NSNode(object):
 
         if isinstance(child, _PluginEntry):
             if not is_leaf:
-                raise PluginDBError('Excessive namespace specification: %s' %
-                                    path[0])
+                raise PluginDBError('Excessive namespace specification: {}'.format(path[0]))
             return child
         elif child is None:
-            raise PluginDBError('Unknown component name: %s' % name)
+            raise PluginDBError('Unknown component name: {}'.format(name))
         else:
             return child._get(path[0])
 
@@ -177,7 +172,7 @@ class _NSNode(object):
                 result.append((key, child.info))
             else:
                 result.extend([
-                    ('%s.%s' % (key, name), value)
+                    ('{}.{}'.format(key, name), value)
                     for name, value in child.info_all().items()
                 ])
         return result
@@ -186,7 +181,7 @@ class _NSNode(object):
         return dict(self._info_all())
 
 
-class _Plugins(object):
+class _Plugins:
 
     """
     represent plugins within a namespace
@@ -196,7 +191,7 @@ class _Plugins(object):
         if interface is not None:
             assert interface.isOrExtends(IPlugin)
 
-        self._group = '%s.%s' % (_NAMESPACE_BASE, namespace)
+        self._group = '{}.{}'.format(_NAMESPACE_BASE, namespace)
 
         self._interface = interface
         self._check_extras = check_extras
@@ -208,25 +203,23 @@ class _Plugins(object):
         if self._check_extras:
             try:
                 entry.require()
-            except Exception as err:
-                raise PluginDBError('Requirements are not satisfied '
-                                    'for %s:%s: %s' % (self._group,
-                                                       entry.name,
-                                                       str(err)))
+            except Exception as e:
+                raise PluginDBError(('Requirements are not satisfied '
+                                     'for {}:{}: {}').format(
+                                         self._group, entry.name, str(e))) from e
         try:
             result = entry.load()
-        except Exception as err:
+        except Exception as e:
             # log full traceback of the bad entry to help support
             traceback.print_exc()
-            raise PluginDBError('Unable to load %s:%s: %s' %
-                                (self._group, entry.name, str(err)))
+            raise PluginDBError('Unable to load {}:{}: {}'.format(self._group, entry.name,
+                                                                  str(e))) from e
         if self._interface:
             try:
                 verifyClass(self._interface, result)
-            except Invalid as err:
-                raise PluginDBError('Plugin %s:%s does not implement %s: %s' %
-                                    (self._group, entry.name,
-                                     self._interface.__name__, str(err)))
+            except Invalid as e:
+                raise PluginDBError('Plugin {}:{} does not implement {}: {}'.format(self._group,
+                        entry.name, self._interface.__name__, str(e))) from e
         return result
 
     @property
@@ -274,11 +267,11 @@ class _Plugins(object):
     def __getattr__(self, name):
         try:
             return getattr(self._tree, name)
-        except PluginDBError as err:
-            raise AttributeError(str(err))
+        except PluginDBError as e:
+            raise AttributeError(str(e)) from e
 
 
-class _PluginDB(object):
+class _PluginDB:
 
     """
     Plugin infrastructure support for Buildbot
