@@ -18,7 +18,6 @@ from functools import reduce
 
 from twisted.internet import defer
 from twisted.internet import error
-from twisted.python import components
 from twisted.python import failure
 from twisted.python import log
 from twisted.python.failure import Failure
@@ -107,6 +106,15 @@ class Build(properties.PropertiesMixin):
         self._locks_released = False
         self._build_finished = False
 
+        # tracks execution during substantiation
+        self._is_substantiating = False
+
+        # tracks the config version for locks
+        self.config_version = None
+
+    def getProperties(self):
+        return self.properties
+
     def setBuilder(self, builder):
         """
         Set the given builder as our builder.
@@ -115,11 +123,12 @@ class Build(properties.PropertiesMixin):
         """
         self.builder = builder
         self.master = builder.master
+        self.config_version = builder.config_version
 
+    @defer.inlineCallbacks
     def setLocks(self, lockList):
-        # convert all locks into their real forms
-        self.locks = [(self.builder.botmaster.getLockFromLockAccess(access), access)
-                      for access in lockList]
+        self.locks = yield self.builder.botmaster.getLockFromLockAccesses(lockList,
+                                                                          self.config_version)
 
     def setWorkerEnvironment(self, env):
         # TODO: remove once we don't have anything depending on this method or attribute
@@ -154,12 +163,12 @@ class Build(properties.PropertiesMixin):
         return files
 
     def __repr__(self):
-        return "<Build %s number:%r results:%s>" % (
-            self.builder.name, self.number, statusToString(self.results))
+        return "<Build {} number:{} results:{}>".format(self.builder.name, repr(self.number),
+                                                        statusToString(self.results))
 
     def blamelist(self):
-        # Note that this algorithm is also implemented in buildbot.reporters.utils.getResponsibleUsersForBuild,
-        # but using the data api.
+        # Note that this algorithm is also implemented in
+        # buildbot.reporters.utils.getResponsibleUsersForBuild, but using the data api.
         # it is important for the UI to have the blamelist easily available.
         # The best way is to make sure the owners property is set to full blamelist
         blamelist = []
@@ -267,6 +276,8 @@ class Build(properties.PropertiesMixin):
                 tags = self.getProperty(
                     self.VIRTUAL_BUILDERTAGS_PROP,
                     self.builder.config.tags)
+                if type(tags) == type([]) and '_virtual_' not in tags:
+                    tags.append('_virtual_')
 
                 self.master.data.updates.updateBuilderInfo(self._builderid,
                                                            description,
@@ -286,7 +297,7 @@ class Build(properties.PropertiesMixin):
 
         worker = workerforbuilder.worker
 
-        log.msg("%s.startBuild" % self)
+        log.msg("{}.startBuild".format(self))
 
         self.build_status = build_status
         # TODO: this will go away when build collapsing is implemented; until
@@ -304,7 +315,8 @@ class Build(properties.PropertiesMixin):
                                                                       str(self.buildid),
                                                                       "stop"))
 
-        # the preparation step counts the time needed for preparing the worker and getting the locks.
+        # the preparation step counts the time needed for preparing the worker and getting the
+        # locks.
         # we cannot use a real step as we don't have a worker yet.
         self.preparation_step = buildstep.BuildStep(name="worker_preparation")
         self.preparation_step.setBuild(self)
@@ -319,7 +331,7 @@ class Build(properties.PropertiesMixin):
 
         # make sure properties are available to people listening on 'new'
         # events
-        yield self._flushProperties(None)
+        yield self.master.data.updates.setBuildProperties(self.buildid, self)
         self.build_status.buildStarted(self)
         yield self.master.data.updates.setBuildStateString(self.buildid, 'starting')
         yield self.master.data.updates.generateNewBuildEvent(self.buildid)
@@ -328,29 +340,34 @@ class Build(properties.PropertiesMixin):
             self.setupBuild()  # create .steps
         except Exception:
             yield self.buildPreparationFailure(Failure(), "setupBuild")
-            self.buildFinished(['Build.setupBuild', 'failed'], EXCEPTION)
+            yield self.buildFinished(['Build.setupBuild', 'failed'], EXCEPTION)
             return
 
         # flush properties in the beginning of the build
-        yield self._flushProperties(None)
-
+        yield self.master.data.updates.setBuildProperties(self.buildid, self)
         yield self.master.data.updates.setBuildStateString(self.buildid,
                                                            'preparing worker')
         try:
-            ready_or_failure = yield workerforbuilder.prepare(self)
+            ready_or_failure = False
+            if workerforbuilder.worker and workerforbuilder.worker.acquireLocks():
+                self._is_substantiating = True
+                ready_or_failure = yield workerforbuilder.substantiate_if_needed(self)
         except Exception:
             ready_or_failure = Failure()
+        finally:
+            self._is_substantiating = False
 
         # If prepare returns True then it is ready and we start a build
         # If it returns failure then we don't start a new build.
         if ready_or_failure is not True:
             yield self.buildPreparationFailure(ready_or_failure, "worker_prepare")
             if self.stopped:
-                self.buildFinished(["worker", "cancelled"], self.results)
-            elif isinstance(ready_or_failure, Failure) and ready_or_failure.check(interfaces.LatentWorkerCannotSubstantiate):
-                self.buildFinished(["worker", "cannot", "substantiate"], EXCEPTION)
+                yield self.buildFinished(["worker", "cancelled"], self.results)
+            elif isinstance(ready_or_failure, Failure) and \
+                    ready_or_failure.check(interfaces.LatentWorkerCannotSubstantiate):
+                yield self.buildFinished(["worker", "cannot", "substantiate"], EXCEPTION)
             else:
-                self.buildFinished(["worker", "not", "available"], RETRY)
+                yield self.buildFinished(["worker", "not", "available"], RETRY)
             return
 
         # ping the worker to make sure they're still there. If they've
@@ -363,8 +380,7 @@ class Build(properties.PropertiesMixin):
         # us in a build.
         yield self.master.data.updates.setBuildStateString(self.buildid,
                                                            'pinging worker')
-        log.msg("starting build %s.. pinging the worker %s"
-                % (self, workerforbuilder))
+        log.msg("starting build {}.. pinging the worker {}".format(self, workerforbuilder))
         try:
             ping_success_or_failure = yield workerforbuilder.ping()
         except Exception:
@@ -372,7 +388,7 @@ class Build(properties.PropertiesMixin):
 
         if ping_success_or_failure is not True:
             yield self.buildPreparationFailure(ping_success_or_failure, "worker_ping")
-            self.buildFinished(["worker", "not", "pinged"], RETRY)
+            yield self.buildFinished(["worker", "not", "pinged"], RETRY)
             return
 
         self.conn = workerforbuilder.worker.conn
@@ -390,14 +406,15 @@ class Build(properties.PropertiesMixin):
             yield self.conn.remoteStartBuild(self.builder.name)
         except Exception:
             yield self.buildPreparationFailure(Failure(), "start_build")
-            self.buildFinished(["worker", "not", "building"], RETRY)
+            yield self.buildFinished(["worker", "not", "building"], RETRY)
             return
 
         yield self.master.data.updates.setBuildStateString(self.buildid,
                                                            'acquiring locks')
         yield self.acquireLocks()
-        yield self.master.data.updates.setStepStateString(self.preparation_step.stepid,
-                                                          "worker ready")
+
+        readymsg = "worker {} ready".format(self.getWorkerName())
+        yield self.master.data.updates.setStepStateString(self.preparation_step.stepid, readymsg)
         yield self.master.data.updates.finishStep(self.preparation_step.stepid, SUCCESS, False)
 
         yield self.master.data.updates.setBuildStateString(self.buildid,
@@ -408,13 +425,21 @@ class Build(properties.PropertiesMixin):
 
     @defer.inlineCallbacks
     def buildPreparationFailure(self, why, state_string):
-        log.err(why, "while " + state_string)
-        self.workerforbuilder.worker.putInQuarantine()
-        if isinstance(why, failure.Failure):
-            yield self.preparation_step.addLogWithFailure(why)
-        yield self.master.data.updates.setStepStateString(self.preparation_step.stepid,
-                                                          "error while " + state_string)
-        yield self.master.data.updates.finishStep(self.preparation_step.stepid, EXCEPTION, False)
+        if self.stopped:
+            # if self.stopped, then this failure is a LatentWorker's failure to substantiate
+            # which we triggered on purpose in stopBuild()
+            log.msg("worker stopped while " + state_string, why)
+            yield self.master.data.updates.finishStep(self.preparation_step.stepid,
+                                                      CANCELLED, False)
+        else:
+            log.err(why, "while " + state_string)
+            self.workerforbuilder.worker.putInQuarantine()
+            if isinstance(why, failure.Failure):
+                yield self.preparation_step.addLogWithFailure(why)
+            yield self.master.data.updates.setStepStateString(self.preparation_step.stepid,
+                                                            "error while " + state_string)
+            yield self.master.data.updates.finishStep(self.preparation_step.stepid,
+                                                      EXCEPTION, False)
 
     @staticmethod
     def _canAcquireLocks(lockList, workerforbuilder):
@@ -431,10 +456,10 @@ class Build(properties.PropertiesMixin):
             return defer.succeed(None)
         if self.stopped:
             return defer.succeed(None)
-        log.msg("acquireLocks(build %s, locks %s)" % (self, self.locks))
+        log.msg("acquireLocks(build {}, locks {})".format(self, self.locks))
         for lock, access in self.locks:
             if not lock.isAvailable(self, access):
-                log.msg("Build %s waiting for lock %s" % (self, lock))
+                log.msg("Build {} waiting for lock {}".format(self, lock))
                 d = lock.waitUntilMaybeAvailable(self, access)
                 d.addCallback(self.acquireLocks)
                 self._acquiringLock = (lock, access, d)
@@ -452,7 +477,7 @@ class Build(properties.PropertiesMixin):
             count = self.stepnames[name]
             count += 1
             self.stepnames[name] = count
-            name = "%s_%d" % (step.name, count)
+            name = "{}_{}".format(step.name, count)
         else:
             self.stepnames[name] = 0
         step.name = name
@@ -460,10 +485,9 @@ class Build(properties.PropertiesMixin):
     def setupBuildSteps(self, step_factories):
         steps = []
         for factory in step_factories:
-            step = factory.buildStep()
+            step = buildstep.create_step_from_step_or_factory(factory)
             step.setBuild(self)
             step.setWorker(self.workerforbuilder.worker)
-            self.setUniqueStepName(step)
             steps.append(step)
 
             if self.useProgress:
@@ -483,18 +507,14 @@ class Build(properties.PropertiesMixin):
             self.setProperty('owners', sorted(owners), 'Build')
         self.text = []  # list of text string lists (text2)
 
-    def _addBuildSteps(self, step_factories):
-        factories = [interfaces.IBuildStepFactory(s) for s in step_factories]
-        return self.setupBuildSteps(factories)
-
     def addStepsAfterCurrentStep(self, step_factories):
         # Add the new steps after the step that is running.
         # The running step has already been popped from self.steps
-        self.steps[0:0] = self._addBuildSteps(step_factories)
+        self.steps[0:0] = self.setupBuildSteps(step_factories)
 
     def addStepsAfterLastStep(self, step_factories):
         # Add the new steps to the end.
-        self.steps.extend(self._addBuildSteps(step_factories))
+        self.steps.extend(self.setupBuildSteps(step_factories))
 
     def getNextStep(self):
         """This method is called to obtain the next BuildStep for this build.
@@ -524,27 +544,34 @@ class Build(properties.PropertiesMixin):
             return self.allStepsDone()
         self.executedSteps.append(s)
         self.currentStep = s
-        d = defer.maybeDeferred(s.startStep, self.conn)
-        d.addBoth(self._flushProperties)
-        d.addCallback(self._stepDone, s)
-        d.addErrback(self.buildException)
+
+        # the following function returns a deferred, but we don't wait for it
+        self._start_next_step_impl(s)
+        return defer.succeed(None)
 
     @defer.inlineCallbacks
-    def _flushProperties(self, results):
-        # `results` is just passed on to the next callback
-        yield self.master.data.updates.setBuildProperties(self.buildid, self)
+    def _start_next_step_impl(self, step):
+        try:
+            results = yield step.startStep(self.conn)
+            yield self.master.data.updates.setBuildProperties(self.buildid, self)
 
-        return results
+            self.currentStep = None
+            if self.finished:
+                return  # build was interrupted, don't keep building
 
-    @defer.inlineCallbacks
-    def _stepDone(self, results, step):
-        self.currentStep = None
-        if self.finished:
-            return  # build was interrupted, don't keep building
-        terminate = yield self.stepDone(results, step)  # interpret/merge results
-        if terminate:
-            self.terminate = True
-        yield self.startNextStep()
+            terminate = yield self.stepDone(results, step)  # interpret/merge results
+            if terminate:
+                self.terminate = True
+            yield self.startNextStep()
+
+        except Exception as e:
+            log.msg("{} build got exception when running step {}".format(self, step))
+            log.err(e)
+
+            yield self.master.data.updates.setBuildProperties(self.buildid, self)
+
+            # Note that buildFinished can't throw exception
+            yield self.buildFinished(["build", "exception"], EXCEPTION)
 
     @defer.inlineCallbacks
     def stepDone(self, results, step):
@@ -560,7 +587,7 @@ class Build(properties.PropertiesMixin):
         summary = yield step.getBuildResultSummary()
         if 'build' in summary:
             text = [summary['build']]
-        log.msg(" step '%s' complete: %s (%s)" % (step.name, statusToString(results), text))
+        log.msg(" step '{}' complete: {} ({})".format(step.name, statusToString(results), text))
         if text:
             self.text.extend(text)
             self.master.data.updates.setBuildStateString(self.buildid,
@@ -577,7 +604,7 @@ class Build(properties.PropertiesMixin):
         # the worker went away. There are several possible reasons for this,
         # and they aren't necessarily fatal. For now, kill the build, but
         # TODO: see if we can resume the build when it reconnects.
-        log.msg("%s.lostRemote" % self)
+        log.msg("{}.lostRemote".format(self))
         self.conn = None
         self.text = ["lost", "connection"]
         self.results = RETRY
@@ -591,7 +618,6 @@ class Build(properties.PropertiesMixin):
             if self._acquiringLock:
                 lock, access, d = self._acquiringLock
                 lock.stopWaitingUntilAvailable(self, access, d)
-                d.callback(None)
 
     def controlStopBuild(self, key, params):
         return self.stopBuild(**params)
@@ -604,7 +630,7 @@ class Build(properties.PropertiesMixin):
         # build as failed quickly rather than waiting for the worker's
         # timeout to kill it on its own.
 
-        log.msg(" %s: stopping build: %s %d" % (self, reason, results))
+        log.msg(" {}: stopping build: {} {}".format(self, reason, results))
         if self.finished:
             return
         # TODO: include 'reason' in this point event
@@ -617,7 +643,10 @@ class Build(properties.PropertiesMixin):
         if self._acquiringLock:
             lock, access, d = self._acquiringLock
             lock.stopWaitingUntilAvailable(self, access, d)
-            d.callback(None)
+        elif self._is_substantiating:
+            # We're having a latent worker that hasn't been substantiated yet. We need to abort
+            # that to not have a latent worker without an associated build
+            self.workerforbuilder.insubstantiate_if_needed()
 
     def allStepsDone(self):
         if self.results == FAILURE:
@@ -635,16 +664,6 @@ class Build(properties.PropertiesMixin):
         text.extend(self.text)
         return self.buildFinished(text, self.results)
 
-    def buildException(self, why):
-        log.msg("%s.buildException" % self)
-        log.err(why)
-        # try to finish the build, but since we've already faced an exception,
-        # this may not work well.
-        try:
-            self.buildFinished(["build", "exception"], EXCEPTION)
-        except Exception:
-            log.err(Failure(), 'while finishing a build with an exception')
-
     @defer.inlineCallbacks
     def buildFinished(self, text, results):
         """This method must be called when the last Step has completed. It
@@ -656,7 +675,9 @@ class Build(properties.PropertiesMixin):
 
         If 'results' is SUCCESS or WARNINGS, we will permit any dependent
         builds to start. If it is 'FAILURE', those builds will be
-        abandoned."""
+        abandoned.
+
+        This method never throws."""
         try:
             self.stopBuildConsumer.stopConsuming()
             self.finished = True
@@ -664,7 +685,7 @@ class Build(properties.PropertiesMixin):
                 self.subs.unsubscribe()
                 self.subs = None
                 self.conn = None
-            log.msg(" %s: build finished" % self)
+            log.msg(" {}: build finished".format(self))
             self.results = worst_status(self.results, results)
             self.build_status.setText(text)
             self.build_status.setResults(self.results)
@@ -698,7 +719,7 @@ class Build(properties.PropertiesMixin):
 
     def releaseLocks(self):
         if self.locks:
-            log.msg("releaseLocks(%s): %s" % (self, self.locks))
+            log.msg("releaseLocks({}): {}".format(self, self.locks))
 
         for lock, access in self.locks:
             if lock.isOwner(self, access):
@@ -751,14 +772,12 @@ class Build(properties.PropertiesMixin):
             ('builds', str(self.buildid), 'finished'),
             lambda: self.finished)
 
+    def getWorkerInfo(self):
+        return self.workerforbuilder.worker.worker_status.info
+
     # IBuildControl
 
     def getStatus(self):
         return self.build_status
 
     # stopBuild is defined earlier
-
-
-components.registerAdapter(
-    lambda build: interfaces.IProperties(build.properties),
-    Build, interfaces.IProperties)
