@@ -13,23 +13,22 @@
 #
 # Copyright Buildbot Team Members
 
-from future.utils import raise_with_traceback
-from future.utils import string_types
-from future.utils import text_type
-
+import inspect
 import re
+import sys
+from io import StringIO
 
 from twisted.internet import defer
 from twisted.internet import error
-from twisted.python import components
 from twisted.python import deprecate
 from twisted.python import failure
 from twisted.python import log
 from twisted.python import util as twutil
 from twisted.python import versions
-from twisted.python.compat import NativeStringIO
+from twisted.python.deprecate import deprecatedModuleAttribute
 from twisted.python.failure import Failure
 from twisted.python.reflect import accumulateClassList
+from twisted.python.versions import Version
 from twisted.web.util import formatFailure
 from zope.interface import implementer
 
@@ -37,7 +36,7 @@ from buildbot import config
 from buildbot import interfaces
 from buildbot import util
 from buildbot.interfaces import IRenderable
-from buildbot.interfaces import WorkerTooOldError
+from buildbot.interfaces import WorkerSetupError
 from buildbot.process import log as plog
 from buildbot.process import logobserver
 from buildbot.process import properties
@@ -58,6 +57,8 @@ from buildbot.process.results import worst_status
 from buildbot.util import bytes2unicode
 from buildbot.util import debounce
 from buildbot.util import flatten
+from buildbot.util.test_result_submitter import TestResultSubmitter
+from buildbot.warnings import warn_deprecated
 
 
 class BuildStepFailed(Exception):
@@ -76,14 +77,52 @@ class CallableAttributeError(Exception):
 
 # old import paths for these classes
 RemoteCommand = remotecommand.RemoteCommand
+deprecatedModuleAttribute(
+    Version("buildbot", 2, 10, 1),
+    message="Use buildbot.process.remotecommand.RemoteCommand instead.",
+    moduleName="buildbot.process.buildstep",
+    name="RemoteCommand",
+)
+
 LoggedRemoteCommand = remotecommand.LoggedRemoteCommand
+deprecatedModuleAttribute(
+    Version("buildbot", 2, 10, 1),
+    message="Use buildbot.process.remotecommand.LoggedRemoteCommand instead.",
+    moduleName="buildbot.process.buildstep",
+    name="LoggedRemoteCommand",
+)
+
 RemoteShellCommand = remotecommand.RemoteShellCommand
+deprecatedModuleAttribute(
+    Version("buildbot", 2, 10, 1),
+    message="Use buildbot.process.remotecommand.RemoteShellCommand instead.",
+    moduleName="buildbot.process.buildstep",
+    name="RemoteShellCommand",
+)
+
 LogObserver = logobserver.LogObserver
+deprecatedModuleAttribute(
+    Version("buildbot", 2, 10, 1),
+    message="Use buildbot.process.logobserver.LogObserver instead.",
+    moduleName="buildbot.process.buildstep",
+    name="LogObserver",
+)
+
 LogLineObserver = logobserver.LogLineObserver
+deprecatedModuleAttribute(
+    Version("buildbot", 2, 10, 1),
+    message="Use buildbot.util.LogLineObserver instead.",
+    moduleName="buildbot.process.buildstep",
+    name="LogLineObserver",
+)
+
 OutputProgressObserver = logobserver.OutputProgressObserver
-_hush_pyflakes = [
-    RemoteCommand, LoggedRemoteCommand, RemoteShellCommand,
-    LogObserver, LogLineObserver, OutputProgressObserver]
+deprecatedModuleAttribute(
+    Version("buildbot", 2, 10, 1),
+    message="Use buildbot.process.logobserver.OutputProgressObserver instead.",
+    moduleName="buildbot.process.buildstep",
+    name="OutputProgressObserver",
+)
 
 
 @implementer(interfaces.IBuildStepFactory)
@@ -105,8 +144,8 @@ class _BuildStepFactory(util.ComparableMixin):
         try:
             return self.factory(*self.args, **self.kwargs)
         except Exception:
-            log.msg("error while creating step, factory=%s, args=%s, kwargs=%s"
-                    % (self.factory, self.args, self.kwargs))
+            log.msg("error while creating step, factory={}, args={}, kwargs={}".format(self.factory,
+                    self.args, self.kwargs))
             raise
 
 
@@ -224,7 +263,7 @@ class SyncLogFileWrapper(logobserver.LogObserver):
 
     def readlines(self):
         alltext = "".join(self.getChunks([self.STDOUT], onlyText=True))
-        io = NativeStringIO(alltext)
+        io = StringIO(alltext)
         return io.readlines()
 
     def getChunks(self, channels=None, onlyText=False):
@@ -245,15 +284,31 @@ class SyncLogFileWrapper(logobserver.LogObserver):
         self._maybeFinished()
 
 
-class BuildStepStatus(object):
+class BuildStepStatus:
     # used only for old-style steps
     pass
 
 
+def get_factory_from_step_or_factory(step_or_factory):
+    if hasattr(step_or_factory, 'get_step_factory'):
+        factory = step_or_factory.get_step_factory()
+    else:
+        factory = step_or_factory
+    # make sure the returned value actually implements IBuildStepFactory
+    return interfaces.IBuildStepFactory(factory)
+
+
+def create_step_from_step_or_factory(step_or_factory):
+    return get_factory_from_step_or_factory(step_or_factory).buildStep()
+
+
 @implementer(interfaces.IBuildStep)
 class BuildStep(results.ResultComputingConfigMixin,
                 properties.PropertiesMixin,
                 util.ComparableMixin):
+    # Note that the BuildStep is at the same time a template from which per-build steps are
+    # constructed. This works by creating a new IBuildStepFactory in __new__, retrieving it via
+    # get_step_factory() and then calling buildStep() on that factory.
 
     alwaysRun = False
     doStepIf = True
@@ -328,8 +383,8 @@ class BuildStep(results.ResultComputingConfigMixin,
                 setattr(self, p, kwargs.pop(p))
 
         if kwargs:
-            config.error("%s.__init__ got unexpected keyword argument(s) %s"
-                         % (self.__class__, list(kwargs)))
+            config.error("{}.__init__ got unexpected keyword argument(s) {}".format(self.__class__,
+                                                                                    list(kwargs)))
         self._pendingLogObservers = []
 
         if not isinstance(self.name, str) and not IRenderable.providedBy(self.name):
@@ -343,7 +398,8 @@ class BuildStep(results.ResultComputingConfigMixin,
         if isinstance(self.descriptionSuffix, str):
             self.descriptionSuffix = [self.descriptionSuffix]
 
-        if self.updateBuildSummaryPolicy is None:  # compute default value for updateBuildSummaryPolicy
+        if self.updateBuildSummaryPolicy is None:
+            # compute default value for updateBuildSummaryPolicy
             self.updateBuildSummaryPolicy = [EXCEPTION, RETRY, CANCELLED]
             if self.flunkOnFailure or self.haltOnFailure or self.warnOnFailure:
                 self.updateBuildSummaryPolicy.append(FAILURE)
@@ -357,7 +413,7 @@ class BuildStep(results.ResultComputingConfigMixin,
             config.error("BuildStep updateBuildSummaryPolicy must be "
                          "a list of result ids or boolean but it is %r" %
                          (self.updateBuildSummaryPolicy,))
-        self._acquiringLock = None
+        self._acquiringLocks = []
         self.stopped = False
         self.master = None
         self.statistics = {}
@@ -366,6 +422,7 @@ class BuildStep(results.ResultComputingConfigMixin,
         self.stepid = None
         self.results = None
         self._start_unhandled_deferreds = None
+        self._test_result_submitters = {}
 
     def __new__(klass, *args, **kwargs):
         self = object.__new__(klass)
@@ -408,7 +465,8 @@ class BuildStep(results.ResultComputingConfigMixin,
                     # python thinks it is actually workdir that is not existing.
                     # python will then swallow the attribute error and call
                     # __getattr__ from worker_transition
-                    raise raise_with_traceback(CallableAttributeError(e))
+                    _, _, traceback = sys.exc_info()
+                    raise CallableAttributeError(e).with_traceback(traceback)
                     # we re-raise the original exception by changing its type,
                     # but keeping its stacktrace
             else:
@@ -418,11 +476,10 @@ class BuildStep(results.ResultComputingConfigMixin,
     def workdir(self, workdir):
         self._workdir = workdir
 
-    def addFactoryArguments(self, **kwargs):
-        # this is here for backwards compatibility
-        pass
+    def getProperties(self):
+        return self.build.getProperties()
 
-    def _getStepFactory(self):
+    def get_step_factory(self):
         return self._factory
 
     def setupProgress(self):
@@ -451,14 +508,15 @@ class BuildStep(results.ResultComputingConfigMixin,
             stepsumm = 'finished'
 
         if self.results != SUCCESS:
-            stepsumm += ' (%s)' % Results[self.results]
+            stepsumm += ' ({})'.format(Results[self.results])
 
         return {'step': stepsumm}
 
     @defer.inlineCallbacks
     def getBuildResultSummary(self):
         summary = yield self.getResultSummary()
-        if self.results in self.updateBuildSummaryPolicy and 'build' not in summary and 'step' in summary:
+        if self.results in self.updateBuildSummaryPolicy and \
+                'build' not in summary and 'step' in summary:
             summary['build'] = summary['step']
         return summary
 
@@ -466,10 +524,9 @@ class BuildStep(results.ResultComputingConfigMixin,
     @defer.inlineCallbacks
     def updateSummary(self):
         def methodInfo(m):
-            import inspect
             lines = inspect.getsourcelines(m)
-            return "\nat %s:%s:\n %s" % (
-                inspect.getsourcefile(m), lines[1], "\n".join(lines[0]))
+            return "\nat {}:{}:\n {}".format(inspect.getsourcefile(m), lines[1],
+                                             "\n".join(lines[0]))
         if not self._running:
             summary = yield self.getResultSummary()
             if not isinstance(summary, dict):
@@ -482,7 +539,7 @@ class BuildStep(results.ResultComputingConfigMixin,
                                 methodInfo(self.getCurrentSummary))
 
         stepResult = summary.get('step', 'finished')
-        if not isinstance(stepResult, text_type):
+        if not isinstance(stepResult, str):
             raise TypeError("step result string must be unicode (got %r)"
                             % (stepResult,))
         if self.stepid is not None:
@@ -493,7 +550,7 @@ class BuildStep(results.ResultComputingConfigMixin,
 
         if not self._running:
             buildResult = summary.get('build', None)
-            if buildResult and not isinstance(buildResult, text_type):
+            if buildResult and not isinstance(buildResult, str):
                 raise TypeError("build result string must be unicode")
     # updateSummary gets patched out for old-style steps, so keep a copy we can
     # call internally for such steps
@@ -504,6 +561,7 @@ class BuildStep(results.ResultComputingConfigMixin,
         # create and start the step, noting that the name may be altered to
         # ensure uniqueness
         self.name = yield self.build.render(self.name)
+        self.build.setUniqueStepName(self)
         self.stepid, self.number, self.name = yield self.master.data.updates.addStep(
             buildid=self.build.buildid,
             name=util.bytes2unicode(self.name))
@@ -517,17 +575,19 @@ class BuildStep(results.ResultComputingConfigMixin,
         self.locks = yield self.build.render(self.locks)
 
         # convert all locks into their real form
-        self.locks = [(self.build.builder.botmaster.getLockFromLockAccess(access), access)
-                      for access in self.locks]
+        botmaster = self.build.builder.botmaster
+        self.locks = yield botmaster.getLockFromLockAccesses(self.locks, self.build.config_version)
+
         # then narrow WorkerLocks down to the worker that this build is being
         # run on
-        self.locks = [(l.getLock(self.build.workerforbuilder.worker), la)
+        self.locks = [(l.getLockForWorker(self.build.workerforbuilder.worker),
+                       la)
                       for l, la in self.locks]
 
         for l, la in self.locks:
             if l in self.build.locks:
-                log.msg("Hey, lock %s is claimed by both a Step (%s) and the"
-                        " parent Build (%s)" % (l, self, self.build))
+                log.msg(("Hey, lock {} is claimed by both a Step ({}) and the"
+                         " parent Build ({})").format(l, self, self.build))
                 raise RuntimeError("lock claimed by both Step and Build")
 
         try:
@@ -562,6 +622,7 @@ class BuildStep(results.ResultComputingConfigMixin,
 
             # run -- or skip -- the step
             if doStep:
+                yield self.addTestResultSets()
                 try:
                     self._running = True
                     self.results = yield self.run()
@@ -596,11 +657,6 @@ class BuildStep(results.ResultComputingConfigMixin,
             if self.results != CANCELLED:
                 self.results = EXCEPTION
 
-        # update the summary one last time, make sure that completes,
-        # and then don't update it any more.
-        self.realUpdateSummary()
-        yield self.realUpdateSummary.stop()
-
         # determine whether we should hide this step
         hidden = self.hideStepIf
         if callable(hidden):
@@ -615,45 +671,79 @@ class BuildStep(results.ResultComputingConfigMixin,
 
         yield self.master.data.updates.finishStep(self.stepid, self.results,
                                                   hidden)
-        # finish unfinished logs
-        all_finished = yield self.finishUnfinishedLogs()
-        if not all_finished:
+        # perform final clean ups
+        success = yield self._cleanup_logs()
+        if not success:
             self.results = EXCEPTION
+
+        # update the summary one last time, make sure that completes,
+        # and then don't update it any more.
+        self.realUpdateSummary()
+        yield self.realUpdateSummary.stop()
+
+        for sub in self._test_result_submitters.values():
+            yield sub.finish()
+
         self.releaseLocks()
 
         return self.results
 
     @defer.inlineCallbacks
-    def finishUnfinishedLogs(self):
-        ok = True
-        not_finished_logs = [v for (k, v) in self.logs.items()
-                             if not v.finished]
+    def _cleanup_logs(self):
+        all_success = True
+        not_finished_logs = [v for (k, v) in self.logs.items() if not v.finished]
         finish_logs = yield defer.DeferredList([v.finish() for v in not_finished_logs],
                                                consumeErrors=True)
         for success, res in finish_logs:
             if not success:
                 log.err(res, "when trying to finish a log")
-                ok = False
-        return ok
+                all_success = False
+
+        for log_ in self.logs.values():
+            if log_.had_errors():
+                all_success = False
+
+        return all_success
+
+    def addTestResultSets(self):
+        return defer.succeed(None)
+
+    @defer.inlineCallbacks
+    def addTestResultSet(self, description, category, value_unit):
+        sub = TestResultSubmitter()
+        yield sub.setup(self, description, category, value_unit)
+        setid = sub.get_test_result_set_id()
+        self._test_result_submitters[setid] = sub
+        return setid
+
+    def addTestResult(self, setid, value, test_name=None, test_code_path=None, line=None,
+                      duration_ns=None):
+        self._test_result_submitters[setid].add_test_result(value, test_name=test_name,
+                                                            test_code_path=test_code_path,
+                                                            line=line, duration_ns=duration_ns)
 
     def acquireLocks(self, res=None):
-        self._acquiringLock = None
         if not self.locks:
             return defer.succeed(None)
         if self.stopped:
             return defer.succeed(None)
-        log.msg("acquireLocks(step %s, locks %s)" % (self, self.locks))
+        log.msg("acquireLocks(step {}, locks {})".format(self, self.locks))
         for lock, access in self.locks:
+            for waited_lock, _, _ in self._acquiringLocks:
+                if lock is waited_lock:
+                    continue
+
             if not lock.isAvailable(self, access):
                 self._waitingForLocks = True
-                log.msg("step %s waiting for lock %s" % (self, lock))
+                log.msg("step {} waiting for lock {}".format(self, lock))
                 d = lock.waitUntilMaybeAvailable(self, access)
+                self._acquiringLocks.append((lock, access, d))
                 d.addCallback(self.acquireLocks)
-                self._acquiringLock = (lock, access, d)
                 return d
         # all locks are available, claim them all
         for lock, access in self.locks:
             lock.claim(self, access)
+        self._acquiringLocks = []
         self._waitingForLocks = False
         return defer.succeed(None)
 
@@ -744,10 +834,10 @@ class BuildStep(results.ResultComputingConfigMixin,
 
     def interrupt(self, reason):
         self.stopped = True
-        if self._acquiringLock:
-            lock, access, d = self._acquiringLock
-            lock.stopWaitingUntilAvailable(self, access, d)
-            d.callback(None)
+        if self._acquiringLocks:
+            for (lock, access, d) in self._acquiringLocks:
+                lock.stopWaitingUntilAvailable(self, access, d)
+            self._acquiringLocks = []
 
         if self._waitingForLocks:
             self.addCompleteLog(
@@ -760,7 +850,7 @@ class BuildStep(results.ResultComputingConfigMixin,
             d.addErrback(log.err, 'while cancelling command')
 
     def releaseLocks(self):
-        log.msg("releaseLocks(%s): %s" % (self, self.locks))
+        log.msg("releaseLocks({}): {}".format(self, self.locks))
         for lock, access in self.locks:
             if lock.isOwner(self, access):
                 lock.release(self, access)
@@ -783,16 +873,18 @@ class BuildStep(results.ResultComputingConfigMixin,
 
     def checkWorkerHasCommand(self, command):
         if not self.workerVersion(command):
-            message = "worker is too old, does not know about %s" % command
-            raise WorkerTooOldError(message)
+            message = "worker is too old, does not know about {}".format(command)
+            raise WorkerSetupError(message)
 
     def getWorkerName(self):
         return self.build.getWorkerName()
 
     def addLog(self, name, type='s', logEncoding=None):
+        if self.stepid is None:
+            raise BuildStepCancelled
         d = self.master.data.updates.addLog(self.stepid,
                                             util.bytes2unicode(name),
-                                            text_type(type))
+                                            str(type))
 
         @d.addCallback
         def newLog(logid):
@@ -820,6 +912,8 @@ class BuildStep(results.ResultComputingConfigMixin,
     @_maybeUnhandled
     @defer.inlineCallbacks
     def addCompleteLog(self, name, text):
+        if self.stepid is None:
+            raise BuildStepCancelled
         logid = yield self.master.data.updates.addLog(self.stepid,
                                                       util.bytes2unicode(name), 't')
         _log = self._newLog(name, 't', logid)
@@ -829,6 +923,8 @@ class BuildStep(results.ResultComputingConfigMixin,
     @_maybeUnhandled
     @defer.inlineCallbacks
     def addHTMLLog(self, name, html):
+        if self.stepid is None:
+            raise BuildStepCancelled
         logid = yield self.master.data.updates.addLog(self.stepid,
                                                       util.bytes2unicode(name), 'h')
         _log = self._newLog(name, 'h', logid)
@@ -873,7 +969,7 @@ class BuildStep(results.ResultComputingConfigMixin,
     @_maybeUnhandled
     @defer.inlineCallbacks
     def addURL(self, name, url):
-        yield self.master.data.updates.addStepURL(self.stepid, text_type(name), text_type(url))
+        yield self.master.data.updates.addStepURL(self.stepid, str(name), str(url))
         return None
 
     @defer.inlineCallbacks
@@ -913,13 +1009,11 @@ class BuildStep(results.ResultComputingConfigMixin,
             desc += self.descriptionSuffix
         return desc
 
-
-components.registerAdapter(
-    BuildStep._getStepFactory,
-    BuildStep, interfaces.IBuildStepFactory)
-components.registerAdapter(
-    lambda step: interfaces.IProperties(step.build),
-    BuildStep, interfaces.IProperties)
+    def warn_deprecated_if_oldstyle_subclass(self, name):
+        if self.__class__.__name__ != name:
+            warn_deprecated('2.9.0', ('Subclassing old-style step {0} in {1} is deprecated, '
+                                      'please migrate to new-style equivalent {0}NewStyle'
+                                      ).format(name, self.__class__.__name__))
 
 
 class LoggingBuildStep(BuildStep):
@@ -934,7 +1028,7 @@ class LoggingBuildStep(BuildStep):
 
     def __init__(self, logfiles=None, lazylogfiles=False, log_eval_func=None,
                  *args, **kwargs):
-        BuildStep.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
         if logfiles is None:
             logfiles = {}
@@ -968,7 +1062,7 @@ class LoggingBuildStep(BuildStep):
     def startCommand(self, cmd, errorMessages=None):
         if errorMessages is None:
             errorMessages = []
-        log.msg("ShellCommand.startCommand(cmd=%s)" % (cmd,))
+        log.msg("ShellCommand.startCommand(cmd={})".format(cmd))
         log.msg("  cmd.args = %r" % (cmd.args))
         self.cmd = cmd  # so we can interrupt it
 
@@ -1078,7 +1172,7 @@ class LoggingBuildStep(BuildStep):
         return defer.succeed(None)
 
 
-class CommandMixin(object):
+class CommandMixin:
 
     @defer.inlineCallbacks
     def _runRemoteCommand(self, cmd, abandonOnFailure, args, makeResult=None):
@@ -1114,7 +1208,7 @@ class CommandMixin(object):
             makeResult=lambda cmd: cmd.updates['files'][0])
 
 
-class ShellMixin(object):
+class ShellMixin:
 
     command = None
     env = {}
@@ -1159,8 +1253,7 @@ class ShellMixin(object):
             prohibitArgs = []
 
         def bad(arg):
-            config.error("invalid %s argument %s" %
-                         (self.__class__.__name__, arg))
+            config.error("invalid {} argument {}".format(self.__class__.__name__, arg))
         for arg in self._shellMixinArgs:
             if arg not in constructorArgs:
                 continue
@@ -1251,10 +1344,14 @@ class ShellMixin(object):
         return cmd
 
     def getResultSummary(self):
+        if self.descriptionDone is not None:
+            return super().getResultSummary()
         summary = util.command_to_string(self.command)
-        if not summary:
-            return super(ShellMixin, self).getResultSummary()
-        return {'step': summary}
+        if summary:
+            if self.results != SUCCESS:
+                summary += ' ({})'.format(Results[self.results])
+            return {'step': summary}
+        return super().getResultSummary()
 
 # Parses the logs for a list of regexs. Meant to be invoked like:
 # regexes = ((re.compile(...), FAILURE), (re.compile(...), WARNINGS))
@@ -1274,8 +1371,8 @@ def regex_log_evaluator(cmd, _, regexes):
         # we won't be changing "worst" unless possible_status is worse than it,
         # so we don't even need to check the log if that's the case
         if worst_status(worst, possible_status) == possible_status:
-            if isinstance(err, string_types):
-                err = re.compile(".*%s.*" % err, re.DOTALL)
+            if isinstance(err, str):
+                err = re.compile(".*{}.*".format(err), re.DOTALL)
             for l in cmd.logs.values():
                 if err.search(l.getText()):
                     worst = possible_status
