@@ -14,16 +14,14 @@
 # Copyright Buildbot Team Members
 
 
-import warnings
 from contextlib import contextmanager
 
 from twisted.python import deprecate
 from twisted.python import versions
 
-from buildbot import interfaces
 from buildbot import util
+from buildbot.process import buildstep
 from buildbot.process.build import Build
-from buildbot.process.buildstep import BuildStep
 from buildbot.steps.download_secret_to_worker import DownloadSecretsToWorker
 from buildbot.steps.download_secret_to_worker import RemoveWorkerFileSecret
 from buildbot.steps.shell import Compile
@@ -33,6 +31,7 @@ from buildbot.steps.shell import ShellCommand
 from buildbot.steps.shell import Test
 from buildbot.steps.source.cvs import CVS
 from buildbot.steps.source.svn import SVN
+from buildbot.warnings import warn_deprecated
 
 
 # deprecated, use BuildFactory.addStep
@@ -40,7 +39,7 @@ from buildbot.steps.source.svn import SVN
 def s(steptype, **kwargs):
     # convenience function for master.cfg files, to create step
     # specification tuples
-    return interfaces.IBuildStepFactory(steptype(**kwargs))
+    return buildstep.get_factory_from_step_or_factory(steptype(**kwargs))
 
 
 class BuildFactory(util.ComparableMixin):
@@ -72,13 +71,13 @@ class BuildFactory(util.ComparableMixin):
         return b
 
     def addStep(self, step, **kwargs):
-        if kwargs or (isinstance(step, type(BuildStep)) and issubclass(step, BuildStep)):
-            warnings.warn(
-                "Passing a BuildStep subclass to factory.addStep is "
-                "deprecated. Please pass a BuildStep instance instead.",
-                DeprecationWarning, stacklevel=2)
+        if kwargs or (isinstance(step, type(buildstep.BuildStep))
+                      and issubclass(step, buildstep.BuildStep)):
+            warn_deprecated("0.8.8",
+                            "Passing a BuildStep subclass to factory.addStep is "
+                            "deprecated. Please pass a BuildStep instance instead.")
             step = step(**kwargs)
-        self.steps.append(interfaces.IBuildStepFactory(step))
+        self.steps.append(buildstep.get_factory_from_step_or_factory(step))
 
     def addSteps(self, steps, withSecrets=None):
         if withSecrets is None:
@@ -99,7 +98,7 @@ class BuildFactory(util.ComparableMixin):
 # BuildFactory subclasses for common build tools
 
 
-class _DefaultCommand(object):
+class _DefaultCommand:
     # Used to indicate a default command to the step.
     pass
 
@@ -124,7 +123,7 @@ class GNUAutoconf(BuildFactory):
         if distcheck is _DefaultCommand:
             distcheck = ["make", "distcheck"]
 
-        BuildFactory.__init__(self, [source])
+        super().__init__([source])
 
         if reconf is True:
             reconf = ["autoreconf", "-si"]
@@ -157,7 +156,7 @@ class GNUAutoconf(BuildFactory):
 class CPAN(BuildFactory):
 
     def __init__(self, source, perl="perl"):
-        BuildFactory.__init__(self, [source])
+        super().__init__([source])
         self.addStep(Configure(command=[perl, "Makefile.PL"]))
         self.addStep(Compile(command=["make"]))
         self.addStep(PerlModuleTest(command=["make", "test"]))
@@ -166,7 +165,7 @@ class CPAN(BuildFactory):
 class Distutils(BuildFactory):
 
     def __init__(self, source, python="python", test=None):
-        BuildFactory.__init__(self, [source])
+        super().__init__([source])
         self.addStep(Compile(command=[python, "./setup.py", "build"]))
         if test is not None:
             self.addStep(Test(command=test))
@@ -192,7 +191,7 @@ class Trial(BuildFactory):
                  buildpython=None, trialpython=None, trial=None,
                  testpath=".", randomly=None, recurse=None,
                  tests=None, useTestCaseNames=False, env=None):
-        BuildFactory.__init__(self, [source])
+        super().__init__([source])
         assert tests or useTestCaseNames, "must use one or the other"
         if buildpython is None:
             buildpython = ["python"]
@@ -240,10 +239,10 @@ class BasicBuildFactory(GNUAutoconf):
             method = "copy"
         source = CVS(
             cvsroot=cvsroot, cvsmodule=cvsmodule, mode=mode, method=method)
-        GNUAutoconf.__init__(self, source,
-                             configure=configure, configureEnv=configureEnv,
-                             compile=compile,
-                             test=test)
+        super().__init__(source,
+                         configure=configure, configureEnv=configureEnv,
+                         compile=compile,
+                         test=test)
 
 
 class QuickBuildFactory(BasicBuildFactory):
@@ -257,10 +256,10 @@ class QuickBuildFactory(BasicBuildFactory):
             configureEnv = {}
         mode = "incremental"
         source = CVS(cvsroot=cvsroot, cvsmodule=cvsmodule, mode=mode)
-        GNUAutoconf.__init__(self, source,
-                             configure=configure, configureEnv=configureEnv,
-                             compile=compile,
-                             test=test)
+        super().__init__(source,
+                         configure=configure, configureEnv=configureEnv,
+                         compile=compile,
+                         test=test)
 
 
 class BasicSVN(GNUAutoconf):
@@ -272,7 +271,7 @@ class BasicSVN(GNUAutoconf):
         if configureEnv is None:
             configureEnv = {}
         source = SVN(svnurl=svnurl, mode="incremental")
-        GNUAutoconf.__init__(self, source,
-                             configure=configure, configureEnv=configureEnv,
-                             compile=compile,
-                             test=test)
+        super().__init__(source,
+                         configure=configure, configureEnv=configureEnv,
+                         compile=compile,
+                         test=test)
