@@ -18,6 +18,7 @@ from urllib.parse import quote_plus as urlquote_plus
 from twisted.internet import defer
 from twisted.python import log
 
+from buildbot import config
 from buildbot.process.properties import Interpolate
 from buildbot.process.properties import Properties
 from buildbot.process.results import CANCELLED
@@ -27,28 +28,76 @@ from buildbot.process.results import RETRY
 from buildbot.process.results import SKIPPED
 from buildbot.process.results import SUCCESS
 from buildbot.process.results import WARNINGS
-from buildbot.reporters import http
+from buildbot.reporters.base import ReporterBase
+from buildbot.reporters.generators.build import BuildStartEndStatusGenerator
+from buildbot.reporters.message import MessageFormatterRenderable
 from buildbot.util import giturlparse
 from buildbot.util import httpclientservice
+from buildbot.warnings import warn_deprecated
 
 HOSTED_BASE_URL = 'https://gitlab.com'
 
 
-class GitLabStatusPush(http.HttpStatusPushBase):
+class GitLabStatusPush(ReporterBase):
     name = "GitLabStatusPush"
-    neededDetails = dict(wantProperties=True)
+
+    def checkConfig(self, token, startDescription=None, endDescription=None,
+                    context=None, baseURL=None, verbose=False, wantProperties=True,
+                    builders=None, debug=None, verify=None,
+                    wantSteps=False, wantPreviousBuild=False, wantLogs=False, generators=None,
+                    **kwargs):
+
+        old_arg_names = {
+            'startDescription': startDescription is not None,
+            'endDescription': endDescription is not None,
+            'wantProperties': wantProperties is not True,
+            'builders': builders is not None,
+            'wantSteps': wantSteps is not False,
+            'wantPreviousBuild': wantPreviousBuild is not False,
+            'wantLogs': wantLogs is not False,
+        }
+
+        passed_old_arg_names = [k for k, v in old_arg_names.items() if v]
+
+        if passed_old_arg_names:
+
+            old_arg_names_msg = ', '.join(passed_old_arg_names)
+            if generators is not None:
+                config.error(("can't specify generators and deprecated {} arguments ({}) at the "
+                              "same time").format(self.__class__.__name__, old_arg_names_msg))
+            warn_deprecated('2.10.0',
+                            ('The arguments {} passed to {} have been deprecated. Use generators '
+                             'instead').format(old_arg_names_msg, self.__class__.__name__))
+
+        if generators is None:
+            generators = self._create_generators_from_old_args(builders, wantProperties, wantSteps,
+                                                               wantPreviousBuild, wantLogs,
+                                                               startDescription, endDescription)
+
+        super().checkConfig(generators=generators, **kwargs)
+        httpclientservice.HTTPClientService.checkAvailable(self.__class__.__name__)
 
     @defer.inlineCallbacks
     def reconfigService(self, token,
                         startDescription=None, endDescription=None,
-                        context=None, baseURL=None, verbose=False, **kwargs):
+                        context=None, baseURL=None, verbose=False, wantProperties=True,
+                        builders=None, debug=None, verify=None,
+                        wantSteps=False, wantPreviousBuild=False, wantLogs=False, generators=None,
+                        **kwargs):
 
         token = yield self.renderSecrets(token)
-        yield http.HttpStatusPushBase.reconfigService(self, **kwargs)
-
+        self.debug = debug
+        self.verify = verify
+        self.verbose = verbose
         self.context = context or Interpolate('buildbot/%(prop:buildername)s')
-        self.startDescription = startDescription or 'Build started.'
-        self.endDescription = endDescription or 'Build done.'
+
+        if generators is None:
+            generators = self._create_generators_from_old_args(builders, wantProperties, wantSteps,
+                                                               wantPreviousBuild, wantLogs,
+                                                               startDescription, endDescription)
+
+        yield super().reconfigService(generators=generators, **kwargs)
+
         if baseURL is None:
             baseURL = HOSTED_BASE_URL
         if baseURL.endswith('/'):
@@ -57,9 +106,23 @@ class GitLabStatusPush(http.HttpStatusPushBase):
         self._http = yield httpclientservice.HTTPClientService.getService(
             self.master, baseURL, headers={'PRIVATE-TOKEN': token},
             debug=self.debug, verify=self.verify)
-        self.verbose = verbose
         self.project_ids = {}
 
+    def _create_generators_from_old_args(self, builders, wantProperties, wantSteps,
+                                         wantPreviousBuild, wantLogs,
+                                         startDescription, endDescription):
+        # wantProperties is ignored, because MessageFormatterRenderable always wants properties.
+        # wantSteps and wantPreviousBuild are ignored ignored, because they are not used in
+        # this reporter.
+        start_formatter = MessageFormatterRenderable(startDescription or 'Build started.')
+        end_formatter = MessageFormatterRenderable(endDescription or 'Build done.')
+
+        return [
+            BuildStartEndStatusGenerator(builders=builders, add_logs=wantLogs,
+                                         start_formatter=start_formatter,
+                                         end_formatter=end_formatter)
+        ]
+
     def createStatus(self,
                      project_id, branch, sha, state, target_url=None,
                      description=None, context=None):
@@ -86,9 +149,8 @@ class GitLabStatusPush(http.HttpStatusPushBase):
         if context is not None:
             payload['name'] = context
 
-        return self._http.post('/api/v4/projects/%d/statuses/%s' % (
-            project_id, sha),
-            json=payload)
+        return self._http.post('/api/v4/projects/{}/statuses/{}'.format(project_id, sha),
+                json=payload)
 
     @defer.inlineCallbacks
     def getProjectId(self, sourcestamp):
@@ -96,13 +158,12 @@ class GitLabStatusPush(http.HttpStatusPushBase):
         url = giturlparse(sourcestamp['repository'])
         if url is None:
             return None
-        project_full_name = "%s/%s" % (url.owner, url.repo)
-
+        project_full_name = "{}/{}".format(url.owner, url.repo)
         # gitlab needs project name to be fully url quoted to get the project id
         project_full_name = urlquote_plus(project_full_name)
 
         if project_full_name not in self.project_ids:
-            response = yield self._http.get('/api/v4/projects/%s' % (project_full_name))
+            response = yield self._http.get('/api/v4/projects/{}'.format(project_full_name))
             proj = yield response.json()
             if response.code not in (200, ):
                 log.msg(
@@ -116,9 +177,27 @@ class GitLabStatusPush(http.HttpStatusPushBase):
 
     @defer.inlineCallbacks
     def send(self, build):
+        # the only case when this function is called is when the user derives this class, overrides
+        # send() and calls super().send(build) from there.
+        yield self._send_impl(build, self._cached_report)
+
+    @defer.inlineCallbacks
+    def sendMessage(self, reports):
+        build = reports[0]['builds'][0]
+        if self.send.__func__ is not GitLabStatusPush.send:
+            warn_deprecated('2.9.0', 'send() in reporters has been deprecated. Use sendMessage()')
+            self._cached_report = reports[0]
+            yield self.send(build)
+        else:
+            yield self._send_impl(build, reports[0])
+
+    @defer.inlineCallbacks
+    def _send_impl(self, build, report):
         props = Properties.fromDict(build['properties'])
         props.master = self.master
 
+        description = report.get('body', None)
+
         if build['complete']:
             state = {
                 SUCCESS: 'success',
@@ -129,10 +208,8 @@ class GitLabStatusPush(http.HttpStatusPushBase):
                 RETRY: 'pending',
                 CANCELLED: 'cancelled'
             }.get(build['results'], 'failed')
-            description = yield props.render(self.endDescription)
         else:
             state = 'running'
-            description = yield props.render(self.startDescription)
 
         context = yield props.render(self.context)
 
