@@ -1,13 +1,13 @@
-from future.utils import string_types
-
 from twisted.internet import defer
 
 from buildbot import config
+from buildbot import interfaces
 from buildbot.process.results import statusToString
 from buildbot.reporters import utils
 from buildbot.reporters.http import HttpStatusPushBase
 from buildbot.util import httpclientservice
 from buildbot.util.logger import Logger
+from buildbot.warnings import warn_deprecated
 
 log = Logger()
 
@@ -20,9 +20,15 @@ class HipChatStatusPush(HttpStatusPushBase):
     def checkConfig(self, auth_token, endpoint=HOSTED_BASE_URL,
                     builder_room_map=None, builder_user_map=None,
                     event_messages=None, **kwargs):
-        if not isinstance(auth_token, string_types):
+        warn_deprecated('2.10.0', 'HipChatStatusPush has been deprecated because the public ' +
+                                  'version of hipchat has been shut down. This reporter will ' +
+                                  'be removed in Buildbot 3.0 unless there is someone who will ' +
+                                  'upgrade the reporter to the new internal APIs present in ' +
+                                  'Buildbot 3.0')
+
+        if not isinstance(auth_token, str) and not interfaces.IRenderable.providedBy(auth_token):
             config.error('auth_token must be a string')
-        if not isinstance(endpoint, string_types):
+        if not isinstance(endpoint, str):
             config.error('endpoint must be a string')
         if builder_room_map and not isinstance(builder_room_map, dict):
             config.error('builder_room_map must be a dict')
@@ -34,7 +40,7 @@ class HipChatStatusPush(HttpStatusPushBase):
                         builder_room_map=None, builder_user_map=None,
                         event_messages=None, **kwargs):
         auth_token = yield self.renderSecrets(auth_token)
-        yield HttpStatusPushBase.reconfigService(self, **kwargs)
+        yield super().reconfigService(**kwargs)
         self._http = yield httpclientservice.HTTPClientService.getService(
             self.master, endpoint,
             debug=self.debug, verify=self.verify)
@@ -43,17 +49,12 @@ class HipChatStatusPush(HttpStatusPushBase):
         self.builder_room_map = builder_room_map
         self.builder_user_map = builder_user_map
 
-    # returns a Deferred that returns None
-    def buildStarted(self, key, build):
-        return self.send(build, key[2])
-
-    # returns a Deferred that returns None
-    def buildFinished(self, key, build):
-        return self.send(build, key[2])
-
     @defer.inlineCallbacks
     def getBuildDetailsAndSendMessage(self, build, key):
-        yield utils.getDetailsForBuild(self.master, build, **self.neededDetails)
+        yield utils.getDetailsForBuild(self.master, build, wantProperties=self.wantProperties,
+                                       wantSteps=self.wantSteps,
+                                       wantPreviousBuild=self.wantPreviousBuild,
+                                       wantLogs=self.wantLogs)
         postData = yield self.getRecipientList(build, key)
         postData['message'] = yield self.getMessage(build, key)
         extra_params = yield self.getExtraParams(build, key)
@@ -71,9 +72,10 @@ class HipChatStatusPush(HttpStatusPushBase):
 
     def getMessage(self, build, event_name):
         event_messages = {
-            'new': 'Buildbot started build %s here: %s' % (build['builder']['name'], build['url']),
-            'finished': 'Buildbot finished build %s with result %s here: %s'
-                        % (build['builder']['name'], statusToString(build['results']), build['url'])
+            'new': 'Buildbot started build {} here: {}'.format(build['builder']['name'],
+                                                               build['url']),
+            'finished': 'Buildbot finished build {} with result {} here: {}'.format(
+                build['builder']['name'], statusToString(build['results']), build['url'])
         }
         return event_messages.get(event_name, '')
 
@@ -83,7 +85,24 @@ class HipChatStatusPush(HttpStatusPushBase):
         return {}
 
     @defer.inlineCallbacks
-    def send(self, build, key):
+    def send(self, build):
+        # the only case when this function is called is when the user derives this class, overrides
+        # send() and calls super().send(build) from there.
+        yield self._send_impl(build)
+
+    @defer.inlineCallbacks
+    def sendMessage(self, reports):
+        build = reports[0]['builds'][0]
+        if self.send.__func__ is not HipChatStatusPush.send:
+            warn_deprecated('2.9.0', 'send() in reporters has been deprecated. Use sendMessage()')
+            yield self.send(build)
+        else:
+            yield self._send_impl(build)
+
+    @defer.inlineCallbacks
+    def _send_impl(self, build):
+        key = 'new' if build['complete'] is False else 'finished'
+
         postData = yield self.getBuildDetailsAndSendMessage(build, key)
         if not postData or 'message' not in postData or not postData['message']:
             return
@@ -95,7 +114,8 @@ class HipChatStatusPush(HttpStatusPushBase):
             urls.append('/v2/room/{}/notification'.format(postData.pop('room_id_or_name')))
 
         for url in urls:
-            response = yield self._http.post(url, params=dict(auth_token=self.auth_token), json=postData)
+            response = yield self._http.post(url, params=dict(auth_token=self.auth_token),
+                                             json=postData)
             if response.code != 200:
                 content = yield response.content()
                 log.error("{code}: unable to upload status: {content}",
