@@ -40,6 +40,7 @@ from buildbot.process.results import SUCCESS
 from buildbot.process.results import WARNINGS
 from buildbot.process.results import statusToString
 from buildbot.reporters import utils
+from buildbot.util import epoch2datetime
 from buildbot.util import service
 from buildbot.util import unicode2bytes
 
@@ -174,6 +175,7 @@ class Channel(service.AsyncService):
                 return self.workerMissing(msg)
             if key[2] == 'connected':
                 return self.workerConnected(msg)
+            return None
 
         for e, f in (("new", buildStarted),             # BuilderStarted
                      ("finished", buildFinished)):      # BuilderFinished
@@ -349,19 +351,22 @@ class Channel(service.AsyncService):
 
         return False
 
+    @defer.inlineCallbacks
     def workerMissing(self, worker):
         self.missing_workers.add(worker['workerid'])
         if self.notify_for('worker'):
-            self.send("Worker `{name}` is missing. It was seen last on {last_connection}.".format(**worker))
-        self.bot.saveMissingWorkers()
+            self.send(("Worker `{name}` is missing. It was seen last on "
+                       "{last_connection}.").format(**worker))
+        yield self.bot.saveMissingWorkers()
 
+    @defer.inlineCallbacks
     def workerConnected(self, worker):
         workerid = worker['workerid']
         if workerid in self.missing_workers:
             self.missing_workers.remove(workerid)
             if self.notify_for('worker'):
                 self.send("Worker `{name}` is back online.".format(**worker))
-            self.bot.saveMissingWorkers()
+            yield self.bot.saveMissingWorkers()
 
 
 class Contact:
@@ -464,24 +469,24 @@ class Contact:
         if not meth:
             if message[-1] == '!':
                 self.send("What you say!")
-                return
+                return None
             elif cmd.startswith(self.bot.commandPrefix):
                 self.send("I don't get this '{}'...".format(cmd))
                 meth = self.command_COMMANDS
             else:
                 if self.is_private_chat:
                     self.send("Say what?")
-                return
+                return None
 
         try:
             result = yield meth(args.strip(), **kwargs)
         except UsageError as e:
             self.send(str(e))
-            return
+            return None
         except Exception as e:
             self.bot.log_err(e)
             self.send("Something bad happened (see logs)")
-            return
+            return None
         return result
 
     def splitArgs(self, args):
@@ -490,7 +495,7 @@ class Contact:
         try:
             return shlex.split(args)
         except ValueError as e:
-            raise UsageError(e)
+            raise UsageError(e) from e
 
     def command_HELLO(self, args, **kwargs):
         """say hello"""
@@ -518,7 +523,8 @@ class Contact:
             pass
 
         if not args:
-            raise UsageError("Try '" + self.bot.commandPrefix + "list [all|N] builders|workers|changes'.")
+            raise UsageError(("Try '{}list [all|N] builders|workers|changes'."
+                              ).format(self.bot.commandPrefix))
 
         if args[0] == 'builders':
             bdicts = yield self.bot.getAllBuilders()
@@ -546,21 +552,20 @@ class Contact:
                     response.append(worker['name'])
                     response.append("[offline]")
             self.send(' '.join(response))
-            return
 
         elif args[0] == 'changes':
             if all:
                 self.send("Do you really want me to list all changes? It can be thousands!\n"
-                          "If you want to be flooded, specify the maximum number of changes to show.\n"
+                          "If you want to be flooded, specify the maximum number of changes "
+                          "to show.\n"
                           "Right now, I will show up to 100 recent changes.")
                 num = 100
-
-            changes = yield self.master.db.changes.getRecentChanges(num)
+            changes = yield self.master.data.get(('changes',), order=['-changeid'], limit=num)
 
             response = ["I found the following recent changes:"]
             for change in reversed(changes):
                 change['comment'] = change['comments'].split('\n')[0]
-                change['date'] = change['when_timestamp'].strftime('%Y-%m-%d %H:%M')
+                change['date'] = epoch2datetime(change['when_timestamp']).strftime('%Y-%m-%d %H:%M')
                 response.append(
                     "{comment})\n"
                     "Author: {author}\n"
@@ -661,6 +666,7 @@ class Contact:
         def watchForCompleteEvent(key, msg):
             if key[-1] in ('finished', 'complete'):
                 return self.channel.buildFinished(msg, watched=True)
+            return None
 
         for build in builds:
             startConsuming = self.master.mq.startConsuming
@@ -769,8 +775,9 @@ class Contact:
         else:
             self.send("Force build successfully requested.")
 
-    command_FORCE.usage = ("force build [--codebase=CODEBASE] [--branch=branch] [--revision=revision]"
-                           " [--props=prop1=val1,prop2=val2...] _which_ _reason_ - Force a build")
+    command_FORCE.usage = ("force build [--codebase=CODEBASE] [--branch=branch] "
+                           "[--revision=revision] [--props=prop1=val1,prop2=val2...] "
+                           "_which_ _reason_ - Force a build")
 
     @defer.inlineCallbacks
     @dangerousCommand
@@ -1181,7 +1188,8 @@ class StatusBot(service.AsyncMultiService):
     def getBuilder(self, buildername=None, builderid=None):
         if buildername:
             bdicts = yield self.master.data.get(('builders',),
-                                                filters=[resultspec.Filter('name', 'eq', [buildername])])
+                                                filters=[resultspec.Filter('name', 'eq',
+                                                                           [buildername])])
             if bdicts:
                 # Could there be more than one? One is enough.
                 bdict = bdicts[0]
