@@ -15,6 +15,7 @@
 
 
 import os
+import signal
 import sys
 import traceback
 
@@ -38,20 +39,20 @@ def installFile(config, target, source, overwrite=False):
         if old_contents != new_contents:
             if overwrite:
                 if not config['quiet']:
-                    print("%s has old/modified contents" % target)
+                    print("{} has old/modified contents".format(target))
                     print(" overwriting it with new contents")
                 with open(target, "wt") as f:
                     f.write(new_contents)
             else:
                 if not config['quiet']:
-                    print("%s has old/modified contents" % target)
-                    print(" writing new contents to %s.new" % target)
+                    print("{} has old/modified contents".format(target))
+                    print(" writing new contents to {}.new".format(target))
                 with open(target + ".new", "wt") as f:
                     f.write(new_contents)
         # otherwise, it's up to date
     else:
         if not config['quiet']:
-            print("creating %s" % target)
+            print("creating {}".format(target))
         with open(target, "wt") as f:
             f.write(new_contents)
 
@@ -73,18 +74,38 @@ def upgradeFiles(config):
 @defer.inlineCallbacks
 def upgradeDatabase(config, master_cfg):
     if not config['quiet']:
-        print("upgrading database (%s)"
-              % (stripUrlPassword(master_cfg.db['db_url'])))
+        print("upgrading database ({})".format(stripUrlPassword(master_cfg.db['db_url'])))
         print("Warning: Stopping this process might cause data loss")
 
-    master = BuildMaster(config['basedir'])
-    master.config = master_cfg
-    master.db.disownServiceParent()
-    db = connector.DBConnector(basedir=config['basedir'])
-    db.setServiceParent(master)
-    yield db.setup(check_version=False, verbose=not config['quiet'])
-    yield db.model.upgrade()
-    yield db.masters.setAllMastersActiveLongTimeAgo()
+    def sighandler(signum, frame):
+        msg = " ".join("""
+        WARNING: ignoring signal {}.
+        This process should not be interrupted to avoid database corruption.
+        If you really need to terminate it, use SIGKILL.
+        """.split())
+        print(msg.format(signum))
+
+    prev_handlers = {}
+    try:
+        for signame in ("SIGTERM", "SIGINT", "SIGQUIT", "SIGHUP",
+                        "SIGUSR1", "SIGUSR2", "SIGBREAK"):
+            if hasattr(signal, signame):
+                signum = getattr(signal, signame)
+                prev_handlers[signum] = signal.signal(signum, sighandler)
+
+        master = BuildMaster(config['basedir'])
+        master.config = master_cfg
+        master.db.disownServiceParent()
+        db = connector.DBConnector(basedir=config['basedir'])
+        yield db.setServiceParent(master)
+        yield db.setup(check_version=False, verbose=not config['quiet'])
+        yield db.model.upgrade()
+        yield db.masters.setAllMastersActiveLongTimeAgo()
+
+    finally:
+        # restore previous signal handlers
+        for signum, handler in prev_handlers.items():
+            signal.signal(signum, handler)
 
 
 @in_reactor
@@ -100,8 +121,8 @@ def upgradeMaster(config, _noMonkey=False):
     try:
         configFile = base.getConfigFileFromTac(config['basedir'])
     except (SyntaxError, ImportError):
-        print("Unable to load 'buildbot.tac' from '%s':" %
-              config['basedir'], file=sys.stderr)
+        print("Unable to load 'buildbot.tac' from '{}':".format(config['basedir']),
+              file=sys.stderr)
         e = traceback.format_exc()
         print(e, file=sys.stderr)
         return defer.succeed(1)
