@@ -33,14 +33,13 @@ class SecretInAFile(SecretProviderBase):
         filepath = os.path.join(dirname, secretfile)
         obs_stat = stat.S_IMODE(os.stat(filepath).st_mode)
         if (obs_stat & 0o77) != 0 and os.name == "posix":
-            config.error("Permissions %s on file %s are too open."
-                         " It is required that your secret files are NOT"
-                         " accessible by others!" % (oct(obs_stat),
-                                                     secretfile))
+            config.error(("Permissions {} on file {} are too open."
+                          " It is required that your secret files are NOT"
+                          " accessible by others!").format(oct(obs_stat), secretfile))
 
     def checkSecretDirectoryIsAvailableAndReadable(self, dirname, suffixes):
         if not os.access(dirname, os.F_OK):
-            config.error("directory %s does not exists" % dirname)
+            config.error("directory {} does not exists".format(dirname))
         for secretfile in os.listdir(dirname):
             for suffix in suffixes:
                 if secretfile.endswith(suffix):
