@@ -13,16 +13,17 @@
 #
 # Copyright Buildbot Team Members
 
+from buildbot.status.client_compat import PBListener
+from buildbot.warnings import warn_deprecated
 
-from twisted.python import log
+# This file is here to allow few remaining users of status within Buildbot to use it
+# without triggering deprecation warnings
 
-from buildbot.status import base
+_hush_pyflakes = [
+    PBListener
+]
 
-
-class PBListener(base.StatusReceiverBase):
-
-    # This class is still present in users' configs, so keep it here.
-
-    def __init__(self, port, user="statusClient", passwd="clientpw"):
-        log.msg("The PBListener status listener is unused and can be removed "
-                "from the configuration")
+warn_deprecated(
+    '0.9.0',
+    'buildbot.status.client has been deprecated, consume the buildbot.data APIs'
+)
