@@ -19,17 +19,20 @@ BuildSteps that are specific to the Twisted source tree
 
 import re
 
+from twisted.internet import defer
 from twisted.python import log
 
+from buildbot import util
+from buildbot.process import buildstep
 from buildbot.process import logobserver
 from buildbot.process.results import FAILURE
 from buildbot.process.results import SKIPPED
 from buildbot.process.results import SUCCESS
 from buildbot.process.results import WARNINGS
-from buildbot.steps.shell import ShellCommand
+from buildbot.steps import shell
 
 
-class HLint(ShellCommand):
+class HLint(buildstep.ShellMixin, buildstep.BuildStep):
 
     """I run a 'lint' checker over a set of .xhtml files. Any deviations
     from recommended style is flagged and put in the output log.
@@ -38,41 +41,57 @@ class HLint(ShellCommand):
     Lore XHTML files to check."""
 
     name = "hlint"
-    description = ["running", "hlint"]
-    descriptionDone = ["hlint"]
+    description = "running hlint"
+    descriptionDone = "hlint"
     warnOnWarnings = True
     warnOnFailure = True
     # TODO: track time, but not output
     warnings = 0
 
     def __init__(self, python=None, **kwargs):
-        ShellCommand.__init__(self, **kwargs)
+        kwargs = self.setupShellMixin(kwargs, prohibitArgs=['command'])
+        super().__init__(**kwargs)
         self.python = python
         self.warningLines = []
         self.addLogObserver(
             'stdio', logobserver.LineConsumerLogObserver(self.logConsumer))
 
-    def start(self):
+    @defer.inlineCallbacks
+    def run(self):
         # create the command
-        htmlFiles = {}
+        html_files = set()
         for f in self.build.allFiles():
             if f.endswith(".xhtml") and not f.startswith("sandbox/"):
-                htmlFiles[f] = 1
+                html_files.add(f)
         # remove duplicates
-        hlintTargets = sorted(htmlFiles.keys())
+        hlintTargets = sorted(list(html_files))
         if not hlintTargets:
             return SKIPPED
         self.hlintFiles = hlintTargets
-        c = []
+
+        command = []
         if self.python:
-            c.append(self.python)
-        c += ["bin/lore", "-p", "--output", "lint"] + self.hlintFiles
-        self.setCommand(c)
+            command.append(self.python)
+        command += ["bin/lore", "-p", "--output", "lint"] + self.hlintFiles
 
-        # add an extra log file to show the .html files we're checking
-        self.addCompleteLog("files", "\n".join(self.hlintFiles) + "\n")
+        cmd = yield self.makeRemoteShellCommand(command=command)
+        yield self.runCommand(cmd)
 
-        ShellCommand.start(self)
+        stdio_log = yield self.getLog('stdio')
+        yield stdio_log.finish()
+
+        yield self.addCompleteLog('warnings', '\n'.join(self.warningLines))
+        yield self.addCompleteLog("files", "\n".join(self.hlintFiles) + "\n")
+
+        # warnings are in stdout, rc is always 0, unless the tools break
+        if cmd.didFail():
+            return FAILURE
+
+        self.descriptionDone = "{} hlin{}".format(self.warnings, self.warnings == 1 and 't' or 'ts')
+
+        if self.warnings:
+            return WARNINGS
+        return SUCCESS
 
     def logConsumer(self):
         while True:
@@ -81,29 +100,12 @@ class HLint(ShellCommand):
                 self.warnings += 1
                 self.warningLines.append(line)
 
-    def commandComplete(self, cmd):
-        self.addCompleteLog('warnings', '\n'.join(self.warningLines))
-
-    def evaluateCommand(self, cmd):
-        # warnings are in stdout, rc is always 0, unless the tools break
-        if cmd.didFail():
-            return FAILURE
-        if self.warnings:
-            return WARNINGS
-        return SUCCESS
-
-    def getText2(self, cmd, results):
-        if cmd.didFail():
-            return ["hlint"]
-        return ["%d hlin%s" % (self.warnings,
-                               self.warnings == 1 and 't' or 'ts')]
-
 
 class TrialTestCaseCounter(logobserver.LogLineObserver):
     _line_re = re.compile(r'^(?:Doctest: )?([\w\.]+) \.\.\. \[([^\]]+)\]$')
 
     def __init__(self):
-        logobserver.LogLineObserver.__init__(self)
+        super().__init__()
         self.numTests = 0
         self.finished = False
         self.counts = {'total': None,
@@ -167,7 +169,7 @@ class TrialTestCaseCounter(logobserver.LogLineObserver):
 UNSPECIFIED = ()  # since None is a valid choice
 
 
-class Trial(ShellCommand):
+class Trial(buildstep.ShellMixin, buildstep.BuildStep):
 
     """
     There are some class attributes which may be usefully overridden
@@ -198,92 +200,18 @@ class Trial(ShellCommand):
     randomly = False
     tests = None  # required
 
+    description = 'testing'
+    descriptionDone = 'tests'
+
     def __init__(self, reactor=UNSPECIFIED, python=None, trial=None,
                  testpath=UNSPECIFIED,
                  tests=None, testChanges=None,
                  recurse=None, randomly=None,
                  trialMode=None, trialArgs=None, jobs=None,
                  **kwargs):
-        """
-        @type  testpath: string
-        @param testpath: use in PYTHONPATH when running the tests. If
-                         None, do not set PYTHONPATH. Setting this to '.' will
-                         cause the source files to be used in-place.
 
-        @type  python: string (without spaces) or list
-        @param python: which python executable to use. Will form the start of
-                       the argv array that will launch trial. If you use this,
-                       you should set 'trial' to an explicit path (like
-                       /usr/bin/trial or ./bin/trial). Defaults to None, which
-                       leaves it out entirely (running 'trial args' instead of
-                       'python ./bin/trial args'). Likely values are 'python',
-                       ['python2.2'], ['python', '-Wall'], etc.
-
-        @type  trial: string
-        @param trial: which 'trial' executable to run.
-                      Defaults to 'trial', which will cause $PATH to be
-                      searched and probably find /usr/bin/trial . If you set
-                      'python', this should be set to an explicit path (because
-                      'python2.3 trial' will not work).
-
-        @type trialMode: list of strings
-        @param trialMode: a list of arguments to pass to trial, specifically
-                          to set the reporting mode. This defaults to ['-to']
-                          which means 'verbose colorless output' to the trial
-                          that comes with Twisted-2.0.x and at least -2.1.0 .
-                          Newer versions of Twisted may come with a trial
-                          that prefers ['--reporter=bwverbose'].
-
-        @type trialArgs: list of strings
-        @param trialArgs: a list of arguments to pass to trial, available to
-                          turn on any extra flags you like. Defaults to [].
-
-        @type jobs: integer
-        @param jobs: integer to be used as trial -j/--jobs option (for
-                     running tests on several workers).  Only supported
-                     since Twisted-12.3.0.
-
-        @type  tests: list of strings
-        @param tests: a list of test modules to run, like
-                      ['twisted.test.test_defer', 'twisted.test.test_process'].
-                      If this is a string, it will be converted into a one-item
-                      list.
-
-        @type  testChanges: boolean
-        @param testChanges: if True, ignore the 'tests' parameter and instead
-                            ask the Build for all the files that make up the
-                            Changes going into this build. Pass these filenames
-                            to trial and ask it to look for test-case-name
-                            tags, running just the tests necessary to cover the
-                            changes.
-
-        @type  recurse: boolean
-        @param recurse: If True, pass the --recurse option to trial, allowing
-                        test cases to be found in deeper subdirectories of the
-                        modules listed in 'tests'. This does not appear to be
-                        necessary when using testChanges.
-
-        @type  reactor: string
-        @param reactor: which reactor to use, like 'gtk' or 'java'. If not
-                        provided, the Twisted's usual platform-dependent
-                        default is used.
-
-        @type  randomly: boolean
-        @param randomly: if True, add the --random=0 argument, which instructs
-                         trial to run the unit tests in a random order each
-                         time. This occasionally catches problems that might be
-                         masked when one module always runs before another
-                         (like failing to make registerAdapter calls before
-                         lookups are done).
-
-        @type  kwargs: dict
-        @param kwargs: parameters. The following parameters are inherited from
-                       L{ShellCommand} and may be useful to set: workdir,
-                       haltOnFailure, flunkOnWarnings, flunkOnFailure,
-                       warnOnWarnings, warnOnFailure, want_stdout, want_stderr,
-                       timeout.
-        """
-        ShellCommand.__init__(self, **kwargs)
+        kwargs = self.setupShellMixin(kwargs, prohibitArgs=['command'])
+        super().__init__(**kwargs)
 
         if python:
             self.python = python
@@ -336,6 +264,40 @@ class Trial(ShellCommand):
         if randomly is not None:
             self.randomly = randomly
 
+        if self.reactor:
+            self.description = "testing ({})".format(self.reactor)
+
+        # this counter will feed Progress along the 'test cases' metric
+        self.observer = TrialTestCaseCounter()
+        self.addLogObserver('stdio', self.observer)
+
+        # this observer consumes multiple lines in a go, so it can't be easily
+        # handled in TrialTestCaseCounter.
+        self.addLogObserver('stdio', logobserver.LineConsumerLogObserver(self.logConsumer))
+        self.problems = []
+        self.warnings = {}
+
+        # text used before commandComplete runs
+        self.text = 'running'
+
+    def setup_python_path(self):
+        if self.testpath is None:
+            return
+
+        # this bit produces a list, which can be used by buildbot_worker.runprocess.RunProcess
+        ppath = self.env.get('PYTHONPATH', self.testpath)
+        if isinstance(ppath, str):
+            ppath = [ppath]
+        if self.testpath not in ppath:
+            ppath.insert(0, self.testpath)
+        self.env['PYTHONPATH'] = ppath
+
+    @defer.inlineCallbacks
+    def run(self):
+        # choose progressMetrics and logfiles based on whether trial is being
+        # run with multiple workers or not.
+        output_observer = logobserver.OutputProgressObserver('test.log')
+
         # build up most of the command, then stash it until start()
         command = []
         if self.python:
@@ -345,58 +307,14 @@ class Trial(ShellCommand):
         if self.recurse:
             command.append("--recurse")
         if self.reactor:
-            command.append("--reactor=%s" % reactor)
+            command.append("--reactor={}".format(self.reactor))
         if self.randomly:
             command.append("--random=0")
         command.extend(self.trialArgs)
-        self.command = command
-
-        if self.reactor:
-            self.description = ["testing", "(%s)" % self.reactor]
-            self.descriptionDone = ["tests"]
-            # commandComplete adds (reactorname) to self.text
-        else:
-            self.description = ["testing"]
-            self.descriptionDone = ["tests"]
-
-        # this counter will feed Progress along the 'test cases' metric
-        self.observer = TrialTestCaseCounter()
-        self.addLogObserver('stdio', self.observer)
-
-        # this observer consumes multiple lines in a go, so it can't be easily
-        # handled in TrialTestCaseCounter.
-        self.addLogObserver(
-            'stdio', logobserver.LineConsumerLogObserver(self.logConsumer))
-        self.problems = []
-        self.warnings = {}
-
-        # text used before commandComplete runs
-        self.text = 'running'
-
-    def setupEnvironment(self, cmd):
-        ShellCommand.setupEnvironment(self, cmd)
-        if self.testpath is not None:
-            e = cmd.args['env']
-            if e is None:
-                cmd.args['env'] = {'PYTHONPATH': self.testpath}
-            else:
-                # this bit produces a list, which can be used
-                # by buildbot_worker.runprocess.RunProcess
-                ppath = e.get('PYTHONPATH', self.testpath)
-                if isinstance(ppath, str):
-                    ppath = [ppath]
-                if self.testpath not in ppath:
-                    ppath.insert(0, self.testpath)
-                e['PYTHONPATH'] = ppath
-
-    def start(self):
-        # choose progressMetrics and logfiles based on whether trial is being
-        # run with multiple workers or not.
-        output_observer = logobserver.OutputProgressObserver('test.log')
 
         if self.jobs is not None:
             self.jobs = int(self.jobs)
-            self.command.append("--jobs=%d" % self.jobs)
+            command.append("--jobs=%d" % self.jobs)
 
             # using -j/--jobs flag produces more than one test log.
             self.logfiles = {}
@@ -415,93 +333,79 @@ class Trial(ShellCommand):
         if self.testChanges:
             for f in self.build.allFiles():
                 if f.endswith(".py"):
-                    self.command.append("--testmodule=%s" % f)
+                    command.append("--testmodule={}".format(f))
         else:
-            self.command.extend(self.tests)
-        log.msg("Trial.start: command is", self.command)
+            command.extend(self.tests)
 
-        ShellCommand.start(self)
+        self.setup_python_path()
+
+        cmd = yield self.makeRemoteShellCommand(command=command)
+        yield self.runCommand(cmd)
+
+        stdio_log = yield self.getLog('stdio')
+        yield stdio_log.finish()
 
-    def commandComplete(self, cmd):
         # figure out all status, then let the various hook functions return
         # different pieces of it
 
-        counts = self.observer.counts
+        problems = '\n'.join(self.problems)
+        warnings = self.warnings
 
+        if problems:
+            yield self.addCompleteLog("problems", problems)
+
+        if warnings:
+            lines = sorted(warnings.keys())
+            yield self.addCompleteLog("warnings", "".join(lines))
+
+        return self.build_results(cmd)
+
+    def build_results(self, cmd):
+        counts = self.observer.counts
         total = counts['total']
-        failures, errors = counts['failures'], counts['errors']
+        failures = counts['failures']
+        errors = counts['errors']
         parsed = (total is not None)
-        text = []
-        text2 = ""
+
+        desc_parts = []
 
         if not cmd.didFail():
             if parsed:
                 results = SUCCESS
                 if total:
-                    text += ["%d %s" %
-                             (total,
-                              total == 1 and "test" or "tests"),
-                             "passed"]
+                    desc_parts += [str(total), total == 1 and "test" or "tests", "passed"]
                 else:
-                    text += ["no tests", "run"]
+                    desc_parts += ["no tests", "run"]
             else:
                 results = FAILURE
-                text += ["testlog", "unparseable"]
-                text2 = "tests"
+                desc_parts += ["testlog", "unparseable"]
         else:
             # something failed
             results = FAILURE
             if parsed:
-                text.append("tests")
+                desc_parts += ["tests"]
                 if failures:
-                    text.append("%d %s" %
-                                (failures,
-                                 failures == 1 and "failure" or "failures"))
+                    desc_parts += [str(failures), failures == 1 and "failure" or "failures"]
                 if errors:
-                    text.append("%d %s" %
-                                (errors,
-                                 errors == 1 and "error" or "errors"))
-                count = failures + errors
-                text2 = "%d tes%s" % (count, (count == 1 and 't' or 'ts'))
+                    desc_parts += [str(errors), errors == 1 and "error" or "errors"]
             else:
-                text += ["tests", "failed"]
-                text2 = "tests"
+                desc_parts += ["tests", "failed"]
 
         if counts['skips']:
-            text.append("%d %s" %
-                        (counts['skips'],
-                         counts['skips'] == 1 and "skip" or "skips"))
+            desc_parts += [str(counts['skips']), counts['skips'] == 1 and "skip" or "skips"]
         if counts['expectedFailures']:
-            text.append("%d %s" %
-                        (counts['expectedFailures'],
-                         counts['expectedFailures'] == 1 and "todo"
-                         or "todos"))
-            if 0:  # TODO  pylint: disable=using-constant-test
-                results = WARNINGS
-                if not text2:
-                    text2 = "todo"
-
-        if 0:  # pylint: disable=using-constant-test
-            # ignore unexpectedSuccesses for now, but it should really mark
-            # the build WARNING
-            if counts['unexpectedSuccesses']:
-                text.append("%d surprises" % counts['unexpectedSuccesses'])
-                results = WARNINGS
-                if not text2:
-                    text2 = "tests"
+            desc_parts += [str(counts['expectedFailures']),
+                           "todo" if counts['expectedFailures'] == 1 else "todos"]
 
         if self.reactor:
-            text.append(self.rtext('(%s)'))
-            if text2:
-                text2 = "%s %s" % (text2, self.rtext('(%s)'))
+            desc_parts.append(self.rtext('({})'))
 
-        self.results = results
-        self.text = text
-        self.text2 = [text2]
+        self.descriptionDone = util.join_list(desc_parts)
+        return results
 
-    def rtext(self, fmt='%s'):
+    def rtext(self, fmt='{}'):
         if self.reactor:
-            rtext = fmt % self.reactor
+            rtext = fmt.format(self.reactor)
             return rtext.replace("reactor", "")
         return ""
 
@@ -527,26 +431,9 @@ class Trial(ShellCommand):
                     self.problems.append(line)
                     stream, line = yield
 
-    def createSummary(self, loog):
-        problems = '\n'.join(self.problems)
-        warnings = self.warnings
 
-        if problems:
-            self.addCompleteLog("problems", problems)
-
-        if warnings:
-            lines = sorted(warnings.keys())
-            self.addCompleteLog("warnings", "".join(lines))
-
-    def evaluateCommand(self, cmd):
-        return self.results
-
-    def describe(self, done=False):
-        return self.text
-
-
-class RemovePYCs(ShellCommand):
-    name = "remove-.pyc"
+class RemovePYCs(shell.ShellCommandNewStyle):
+    name = "remove_pyc"
     command = ['find', '.', '-name', "'*.pyc'", '-exec', 'rm', '{}', ';']
-    description = ["removing", ".pyc", "files"]
-    descriptionDone = ["remove", ".pycs"]
+    description = "removing .pyc files"
+    descriptionDone = "remove .pycs"
