@@ -22,7 +22,7 @@ from twisted.internet import defer
 from twisted.python import log
 
 from buildbot import config
-from buildbot.interfaces import WorkerTooOldError
+from buildbot.interfaces import WorkerSetupError
 from buildbot.process import remotecommand
 from buildbot.process import remotetransfer
 from buildbot.process.buildstep import FAILURE
@@ -31,13 +31,12 @@ from buildbot.process.buildstep import SUCCESS
 from buildbot.process.buildstep import BuildStep
 from buildbot.steps.worker import CompositeStepMixin
 from buildbot.util import flatten
-from buildbot.util.eventual import eventually
 
 
 def makeStatusRemoteCommand(step, remote_command, args):
     self = remotecommand.RemoteCommand(
         remote_command, args, decodeRC={None: SUCCESS, 0: SUCCESS})
-    self.useLogDelayed('stdio', lambda arg: step.step_status.addLog('stdio'), True)
+    self.useLog(step.stdio_log)
     return self
 
 
@@ -54,34 +53,27 @@ class _TransferBuildStep(BuildStep):
     flunkOnFailure = True
 
     def __init__(self, workdir=None, **buildstep_kwargs):
-        BuildStep.__init__(self, **buildstep_kwargs)
+        super().__init__(**buildstep_kwargs)
         self.workdir = workdir
 
+    @defer.inlineCallbacks
     def runTransferCommand(self, cmd, writer=None):
         # Run a transfer step, add a callback to extract the command status,
         # add an error handler that cancels the writer.
         self.cmd = cmd
-        d = self.runCommand(cmd)
-
-        @d.addCallback
-        def checkResult(_):
-            if writer and cmd.didFail():
-                writer.cancel()
-            return FAILURE if cmd.didFail() else SUCCESS
-
-        @d.addErrback
-        def cancel(res):
+        try:
+            yield self.runCommand(cmd)
+            return cmd.results()
+        finally:
             if writer:
                 writer.cancel()
-            return res
-
-        return d
 
+    @defer.inlineCallbacks
     def interrupt(self, reason):
-        self.addCompleteLog('interrupt', str(reason))
+        yield self.addCompleteLog('interrupt', str(reason))
         if self.cmd:
-            d = self.cmd.interrupt(reason)
-            return d
+            yield self.cmd.interrupt(reason)
+        return None
 
 
 class FileUpload(_TransferBuildStep):
@@ -103,7 +95,7 @@ class FileUpload(_TransferBuildStep):
         if workersrc is None or masterdest is None:
             raise TypeError("__init__() takes at least 3 arguments")
 
-        _TransferBuildStep.__init__(self, workdir=workdir, **buildstep_kwargs)
+        super().__init__(workdir=workdir, **buildstep_kwargs)
 
         self.workersrc = workersrc
         self.masterdest = masterdest
@@ -117,14 +109,10 @@ class FileUpload(_TransferBuildStep):
         self.url = url
         self.urlText = urlText
 
-    def finished(self, results):
-        log.msg("File '{}' upload finished with results {}".format(
-            os.path.basename(self.workersrc), str(results)))
-        self.step_status.setText(self.descriptionDone)
-        _TransferBuildStep.finished(self, results)
-
-    def start(self):
+    @defer.inlineCallbacks
+    def run(self):
         self.checkWorkerHasCommand("uploadFile")
+        self.stdio_log = yield self.addLog("stdio")
 
         source = self.workersrc
         masterdest = self.masterdest
@@ -137,7 +125,7 @@ class FileUpload(_TransferBuildStep):
                 % (source, masterdest))
 
         if self.description is None:
-            self.description = ['uploading %s' % (os.path.basename(source))]
+            self.description = ['uploading {}'.format(os.path.basename(source))]
 
         if self.descriptionDone is None:
             self.descriptionDone = self.description
@@ -148,18 +136,16 @@ class FileUpload(_TransferBuildStep):
             if urlText is None:
                 urlText = os.path.basename(masterdest)
 
-            self.addURL(urlText, self.url)
-
-        self.step_status.setText(self.description)
+            yield self.addURL(urlText, self.url)
 
         # we use maxsize to limit the amount of data on both sides
         fileWriter = remotetransfer.FileWriter(
             masterdest, self.maxsize, self.mode)
 
         if self.keepstamp and self.workerVersionIsOlderThan("uploadFile", "2.13"):
-            m = ("This worker (%s) does not support preserving timestamps. "
-                 "Please upgrade the worker." % self.build.workername)
-            raise WorkerTooOldError(m)
+            m = (("This worker ({}) does not support preserving timestamps. "
+                  "Please upgrade the worker.").format(self.build.workername))
+            raise WorkerSetupError(m)
 
         # default arguments
         args = {
@@ -176,15 +162,24 @@ class FileUpload(_TransferBuildStep):
             args['workersrc'] = source
 
         cmd = makeStatusRemoteCommand(self, 'uploadFile', args)
-        d = self.runTransferCommand(cmd, fileWriter)
-        d.addCallback(self.finished).addErrback(self.failed)
+        res = yield self.runTransferCommand(cmd, fileWriter)
+
+        log.msg("File '{}' upload finished with results {}".format(
+            os.path.basename(self.workersrc), str(res)))
+
+        return res
 
 
 class DirectoryUpload(_TransferBuildStep):
 
     name = 'upload'
 
-    renderables = ['workersrc', 'masterdest', 'url']
+    renderables = [
+        'workersrc',
+        'masterdest',
+        'url',
+        'urlText'
+    ]
 
     def __init__(self, workersrc=None, masterdest=None,
                  workdir=None, maxsize=None, blocksize=16 * 1024,
@@ -195,7 +190,7 @@ class DirectoryUpload(_TransferBuildStep):
         if workersrc is None or masterdest is None:
             raise TypeError("__init__() takes at least 3 arguments")
 
-        _TransferBuildStep.__init__(self, workdir=workdir, **buildstep_kwargs)
+        super().__init__(workdir=workdir, **buildstep_kwargs)
 
         self.workersrc = workersrc
         self.masterdest = masterdest
@@ -208,8 +203,10 @@ class DirectoryUpload(_TransferBuildStep):
         self.url = url
         self.urlText = urlText
 
-    def start(self):
+    @defer.inlineCallbacks
+    def run(self):
         self.checkWorkerHasCommand("uploadDirectory")
+        self.stdio_log = yield self.addLog("stdio")
 
         source = self.workersrc
         masterdest = self.masterdest
@@ -218,17 +215,17 @@ class DirectoryUpload(_TransferBuildStep):
         # properly. TODO: maybe pass the master's basedir all the way down
         # into the BuildStep so we can do this better.
         masterdest = os.path.expanduser(masterdest)
-        log.msg("DirectoryUpload started, from worker %r to master %r"
-                % (source, masterdest))
+        log.msg("DirectoryUpload started, from worker {} to master {}".format(repr(source),
+                                                                              repr(masterdest)))
 
-        self.descriptionDone = "uploading %s" % os.path.basename(source)
+        self.descriptionDone = "uploading {}".format(os.path.basename(source))
         if self.url is not None:
             urlText = self.urlText
 
             if urlText is None:
                 urlText = os.path.basename(os.path.normpath(masterdest))
 
-            self.addURL(urlText, self.url)
+            yield self.addURL(urlText, self.url)
 
         # we use maxsize to limit the amount of data on both sides
         dirWriter = remotetransfer.DirectoryWriter(
@@ -249,8 +246,8 @@ class DirectoryUpload(_TransferBuildStep):
             args['workersrc'] = source
 
         cmd = makeStatusRemoteCommand(self, 'uploadDirectory', args)
-        d = self.runTransferCommand(cmd, dirWriter)
-        d.addCallback(self.finished).addErrback(self.failed)
+        res = yield self.runTransferCommand(cmd, dirWriter)
+        return res
 
 
 class MultipleFileUpload(_TransferBuildStep, CompositeStepMixin):
@@ -258,18 +255,23 @@ class MultipleFileUpload(_TransferBuildStep, CompositeStepMixin):
     name = 'upload'
     logEnviron = False
 
-    renderables = ['workersrcs', 'masterdest', 'url']
+    renderables = [
+        'workersrcs',
+        'masterdest',
+        'url',
+        'urlText'
+    ]
 
     def __init__(self, workersrcs=None, masterdest=None,
                  workdir=None, maxsize=None, blocksize=16 * 1024, glob=False,
-                 mode=None, compress=None, keepstamp=False, url=None,
+                 mode=None, compress=None, keepstamp=False, url=None, urlText=None,
                  **buildstep_kwargs):
 
         # Emulate that first two arguments are positional.
         if workersrcs is None or masterdest is None:
             raise TypeError("__init__() takes at least 3 arguments")
 
-        _TransferBuildStep.__init__(self, workdir=workdir, **buildstep_kwargs)
+        super().__init__(workdir=workdir, **buildstep_kwargs)
 
         self.workersrcs = workersrcs
         self.masterdest = masterdest
@@ -286,6 +288,7 @@ class MultipleFileUpload(_TransferBuildStep, CompositeStepMixin):
         self.glob = glob
         self.keepstamp = keepstamp
         self.url = url
+        self.urlText = urlText
 
     def uploadFile(self, source, masterdest):
         fileWriter = remotetransfer.FileWriter(
@@ -327,6 +330,7 @@ class MultipleFileUpload(_TransferBuildStep, CompositeStepMixin):
         cmd = makeStatusRemoteCommand(self, 'uploadDirectory', args)
         return self.runTransferCommand(cmd, dirWriter)
 
+    @defer.inlineCallbacks
     def startUpload(self, source, destdir):
         masterdest = os.path.join(destdir, os.path.basename(source))
         args = {
@@ -335,98 +339,81 @@ class MultipleFileUpload(_TransferBuildStep, CompositeStepMixin):
         }
 
         cmd = makeStatusRemoteCommand(self, 'stat', args)
-        d = self.runCommand(cmd)
+        yield self.runCommand(cmd)
+        if cmd.rc != 0:
+            msg = 'File {}/{} not available at worker'.format(self.workdir, source)
+            yield self.addCompleteLog('stderr', msg)
+            return FAILURE
+        s = cmd.updates['stat'][-1]
+        if stat.S_ISDIR(s[stat.ST_MODE]):
+            result = yield self.uploadDirectory(source, masterdest)
+        elif stat.S_ISREG(s[stat.ST_MODE]):
+            result = yield self.uploadFile(source, masterdest)
+        else:
+            msg = '{} is neither a regular file, nor a directory'.format(source)
+            yield self.addCompleteLog('stderr', msg)
+            return FAILURE
 
-        @d.addCallback
-        def checkStat(_):
-            s = cmd.updates['stat'][-1]
-            if stat.S_ISDIR(s[stat.ST_MODE]):
-                return self.uploadDirectory(source, masterdest)
-            elif stat.S_ISREG(s[stat.ST_MODE]):
-                return self.uploadFile(source, masterdest)
-            return defer.fail('%r is neither a regular file, nor a directory' % source)
-
-        @d.addCallback
-        def uploadDone(result):
-            d = defer.maybeDeferred(
-                self.uploadDone, result, source, masterdest)
-            d.addCallback(lambda _: result)
-            return d
-
-        return d
+        yield self.uploadDone(result, source, masterdest)
+        return result
 
     def uploadDone(self, result, source, masterdest):
         pass
 
+    @defer.inlineCallbacks
     def allUploadsDone(self, result, sources, masterdest):
         if self.url is not None:
-            self.addURL(
-                os.path.basename(os.path.normpath(masterdest)), self.url)
+            urlText = self.urlText
 
-    def start(self):
+            if urlText is None:
+                urlText = os.path.basename(os.path.normpath(masterdest))
+
+            yield self.addURL(urlText, self.url)
+
+    @defer.inlineCallbacks
+    def run(self):
         self.checkWorkerHasCommand("uploadDirectory")
         self.checkWorkerHasCommand("uploadFile")
         self.checkWorkerHasCommand("stat")
+        self.stdio_log = yield self.addLog("stdio")
 
         masterdest = os.path.expanduser(self.masterdest)
         sources = self.workersrcs if isinstance(self.workersrcs, list) else [self.workersrcs]
 
         if self.keepstamp and self.workerVersionIsOlderThan("uploadFile", "2.13"):
-            m = ("This worker (%s) does not support preserving timestamps. "
-                 "Please upgrade the worker." % self.build.workername)
-            raise WorkerTooOldError(m)
+            m = (("This worker ({}) does not support preserving timestamps. "
+                  "Please upgrade the worker.").format(self.build.workername))
+            raise WorkerSetupError(m)
 
         if not sources:
-            return self.finished(SKIPPED)
-
-        @defer.inlineCallbacks
-        def globSources(sources):
-            results = yield defer.gatherResults([
-                self.runGlob(
-                    os.path.join(self.workdir, source), abandonOnFailure=False) for source in sources
-            ])
-            results = [self.workerPathToMasterPath(p) for p in flatten(results)]
-            return results
-
-        @defer.inlineCallbacks
-        def uploadSources(sources):
-            if not sources:
-                return SKIPPED
-            else:
-                for source in sources:
-                    result = yield self.startUpload(source, masterdest)
-                    if result == FAILURE:
-                        return FAILURE
-                return SUCCESS
-
-        def logUpload(sources):
-            log.msg("MultipleFileUpload started, from worker %r to master %r" %
-                    (sources, masterdest))
-            nsrcs = len(sources)
-            self.descriptionDone = 'uploading %d %s' % (nsrcs, 'file'
-                                                        if nsrcs == 1 else
-                                                        'files')
-            return sources
+            return SKIPPED
 
         if self.glob:
-            s = globSources(sources)
+            results = yield defer.gatherResults([
+                self.runGlob(os.path.join(self.workdir, source), abandonOnFailure=False)
+                for source in sources
+            ])
+            sources = [self.workerPathToMasterPath(p) for p in flatten(results)]
+
+        log.msg("MultipleFileUpload started, from worker {!r} to master {!r}".format(sources,
+                                                                                     masterdest))
+
+        self.descriptionDone = ['uploading', str(len(sources)),
+                                'file' if len(sources) == 1 else 'files']
+
+        if not sources:
+            result = SKIPPED
         else:
-            s = defer.succeed(sources)
+            result = SUCCESS
+            for source in sources:
+                result_single = yield self.startUpload(source, masterdest)
+                if result_single == FAILURE:
+                    result = FAILURE
+                    break
 
-        s.addCallback(logUpload)
-        d = s.addCallback(uploadSources)
+        yield self.allUploadsDone(result, sources, masterdest)
 
-        @d.addCallback
-        def allUploadsDone(result):
-            d = defer.maybeDeferred(
-                self.allUploadsDone, result, sources, masterdest)
-            d.addCallback(lambda _: result)
-            return d
-
-        d.addCallback(self.finished).addErrback(self.failed)
-
-    def finished(self, result):
-        return BuildStep.finished(self, result)
+        return result
 
 
 class FileDownload(_TransferBuildStep):
@@ -442,7 +429,7 @@ class FileDownload(_TransferBuildStep):
         if workerdest is None:
             raise TypeError("__init__() takes at least 3 arguments")
 
-        _TransferBuildStep.__init__(self, workdir=workdir, **buildstep_kwargs)
+        super().__init__(workdir=workdir, **buildstep_kwargs)
 
         self.mastersrc = mastersrc
         self.workerdest = workerdest
@@ -453,8 +440,10 @@ class FileDownload(_TransferBuildStep):
                 'mode must be an integer or None')
         self.mode = mode
 
-    def start(self):
+    @defer.inlineCallbacks
+    def run(self):
         self.checkWorkerHasCommand("downloadFile")
+        self.stdio_log = yield self.addLog("stdio")
 
         # we are currently in the buildmaster's basedir, so any non-absolute
         # paths will be interpreted relative to that
@@ -463,20 +452,16 @@ class FileDownload(_TransferBuildStep):
         log.msg("FileDownload started, from master %r to worker %r" %
                 (source, workerdest))
 
-        self.descriptionDone = "downloading to %s" % os.path.basename(
-            workerdest)
+        self.descriptionDone = ["downloading to", os.path.basename(workerdest)]
 
         # setup structures for reading the file
         try:
             fp = open(source, 'rb')
         except IOError:
             # if file does not exist, bail out with an error
-            self.addCompleteLog('stderr',
-                                'File %r not available at master' % source)
-            # TODO: once BuildStep.start() gets rewritten to use
-            # maybeDeferred, just re-raise the exception here.
-            eventually(BuildStep.finished, self, FAILURE)
-            return
+            yield self.addCompleteLog('stderr', 'File {!r} not available at master'.format(source))
+            raise
+
         fileReader = remotetransfer.FileReader(fp)
 
         # default arguments
@@ -494,8 +479,8 @@ class FileDownload(_TransferBuildStep):
             args['workerdest'] = workerdest
 
         cmd = makeStatusRemoteCommand(self, 'downloadFile', args)
-        d = self.runTransferCommand(cmd)
-        d.addCallback(self.finished).addErrback(self.failed)
+        res = yield self.runTransferCommand(cmd)
+        return res
 
 
 class StringDownload(_TransferBuildStep):
@@ -511,21 +496,22 @@ class StringDownload(_TransferBuildStep):
         if workerdest is None:
             raise TypeError("__init__() takes at least 3 arguments")
 
-        _TransferBuildStep.__init__(self, workdir=workdir, **buildstep_kwargs)
+        super().__init__(workdir=workdir, **buildstep_kwargs)
 
         self.s = s
         self.workerdest = workerdest
         self.maxsize = maxsize
         self.blocksize = blocksize
         if not isinstance(mode, (int, type(None))):
-            config.error(
-                "StringDownload step's mode must be an integer or None,"
-                " got '%s'" % mode)
+            config.error("StringDownload step's mode must be an integer or None, got '{}'".format(
+                    mode))
         self.mode = mode
 
-    def start(self):
+    @defer.inlineCallbacks
+    def run(self):
         # we use 'downloadFile' remote command on the worker
         self.checkWorkerHasCommand("downloadFile")
+        self.stdio_log = yield self.addLog("stdio")
 
         # we are currently in the buildmaster's basedir, so any non-absolute
         # paths will be interpreted relative to that
@@ -533,8 +519,7 @@ class StringDownload(_TransferBuildStep):
         log.msg("StringDownload started, from master to worker %r" %
                 workerdest)
 
-        self.descriptionDone = "downloading to %s" % os.path.basename(
-            workerdest)
+        self.descriptionDone = ["downloading to", os.path.basename(workerdest)]
 
         # setup structures for reading the file
         fileReader = remotetransfer.StringFileReader(self.s)
@@ -554,8 +539,8 @@ class StringDownload(_TransferBuildStep):
             args['workerdest'] = workerdest
 
         cmd = makeStatusRemoteCommand(self, 'downloadFile', args)
-        d = self.runTransferCommand(cmd)
-        d.addCallback(self.finished).addErrback(self.failed)
+        res = yield self.runTransferCommand(cmd)
+        return res
 
 
 class JSONStringDownload(StringDownload):
@@ -571,8 +556,7 @@ class JSONStringDownload(StringDownload):
         if 's' in buildstep_kwargs:
             del buildstep_kwargs['s']
         s = json.dumps(o)
-        StringDownload.__init__(
-            self, s=s, workerdest=workerdest, **buildstep_kwargs)
+        super().__init__(s=s, workerdest=workerdest, **buildstep_kwargs)
 
 
 class JSONPropertiesDownload(StringDownload):
@@ -585,13 +569,12 @@ class JSONPropertiesDownload(StringDownload):
         if workerdest is None:
             raise TypeError("__init__() takes at least 2 arguments")
 
-        self.super_class = StringDownload
         if 's' in buildstep_kwargs:
             del buildstep_kwargs['s']
-        StringDownload.__init__(
-            self, s=None, workerdest=workerdest, **buildstep_kwargs)
+        super().__init__(s=None, workerdest=workerdest, **buildstep_kwargs)
 
-    def start(self):
+    @defer.inlineCallbacks
+    def run(self):
         properties = self.build.getProperties()
         props = {}
         for key, value, source in properties.asList():
@@ -603,4 +586,5 @@ class JSONPropertiesDownload(StringDownload):
                           for ss in self.build.getAllSourceStamps()],
         ),
         )
-        return self.super_class.start(self)
+        res = yield super().run()
+        return res
