@@ -17,6 +17,7 @@ from twisted.internet import defer
 from twisted.python import log
 
 from buildbot import config
+from buildbot.interfaces import IRenderable
 from buildbot.interfaces import ITriggerableScheduler
 from buildbot.process.buildstep import CANCELLED
 from buildbot.process.buildstep import EXCEPTION
@@ -24,6 +25,7 @@ from buildbot.process.buildstep import SUCCESS
 from buildbot.process.buildstep import BuildStep
 from buildbot.process.properties import Properties
 from buildbot.process.properties import Property
+from buildbot.process.results import ALL_RESULTS
 from buildbot.process.results import statusToString
 from buildbot.process.results import worst_status
 
@@ -65,10 +67,18 @@ class Trigger(BuildStep):
             config.error(
                 "You can't specify both alwaysUseLatest and updateSourceStamp"
             )
-        if not set(schedulerNames).issuperset(set(unimportantSchedulerNames)):
-            config.error(
-                "unimportantSchedulerNames must be a subset of schedulerNames"
-            )
+
+        def hasRenderable(l):
+            for s in l:
+                if IRenderable.providedBy(s):
+                    return True
+            return False
+
+        if not hasRenderable(schedulerNames) and not hasRenderable(unimportantSchedulerNames):
+            if not set(schedulerNames).issuperset(set(unimportantSchedulerNames)):
+                config.error(
+                    "unimportantSchedulerNames must be a subset of schedulerNames"
+                )
 
         self.schedulerNames = schedulerNames
         self.unimportantSchedulerNames = unimportantSchedulerNames
@@ -98,7 +108,8 @@ class Trigger(BuildStep):
         self.brids = []
         self.triggeredNames = None
         self.waitForFinishDeferred = None
-        BuildStep.__init__(self, **kwargs)
+        self._result_list = []
+        super().__init__(**kwargs)
 
     def interrupt(self, reason):
         # We cancel the buildrequests, as the data api handles
@@ -190,15 +201,16 @@ class Trigger(BuildStep):
             if isinstance(results, tuple):
                 results, brids_dict = results
 
+                # brids_dict.values() represents the list of brids kicked by a certain scheduler.
+                # We want to ignore the result of ANY brid that was kicked off
+                # by an UNimportant scheduler.
+                if set(unimportant_brids).issuperset(set(brids_dict.values())):
+                    continue
+
             if not was_cb:
                 yield self.addLogWithFailure(results)
                 results = EXCEPTION
 
-            # brids_dict.values() represents the list of brids kicked by a certain scheduler.
-            # We want to ignore the result of ANY brid that was kicked off
-            # by an UNimportant scheduler.
-            if set(unimportant_brids).issuperset(set(brids_dict.values())):
-                continue
             overall_results = worst_status(overall_results, results)
         return overall_results
 
@@ -221,8 +233,23 @@ class Trigger(BuildStep):
                             builderNames[builderid] = builderDict["name"]
                         num = build['number']
                         url = self.master.status.getURLForBuild(builderid, num)
-                        yield self.addURL("%s: %s #%d" % (statusToString(build["results"]),
-                                                          builderNames[builderid], num), url)
+                        yield self.addURL("{}: {} #{}".format(statusToString(build["results"]),
+                                                              builderNames[builderid], num),
+                                          url)
+
+    @defer.inlineCallbacks
+    def _add_results(self, brid):
+        @defer.inlineCallbacks
+        def _is_buildrequest_complete(brid):
+            buildrequest = yield self.master.db.buildrequests.getBuildRequest(brid)
+            return buildrequest['complete']
+
+        event = ('buildrequests', str(brid), 'complete')
+        yield self.master.mq.waitUntilEvent(event, lambda: _is_buildrequest_complete(brid))
+        builds = yield self.master.db.builds.getBuilds(buildrequestid=brid)
+        for build in builds:
+            self._result_list.append(build["results"])
+        self.updateSummary()
 
     @defer.inlineCallbacks
     def run(self):
@@ -237,15 +264,14 @@ class Trigger(BuildStep):
             if isinstance(element, dict):
                 schedulers_and_props_list = schedulers_and_props
                 break
-            else:
-                # Old-style back compatibility: Convert tuple to dict and make
-                # it important
-                d = {
-                    'sched_name': element[0],
-                    'props_to_set': element[1],
-                    'unimportant': False
-                }
-                schedulers_and_props_list.append(d)
+            # Old-style back compatibility: Convert tuple to dict and make
+            # it important
+            d = {
+                'sched_name': element[0],
+                'props_to_set': element[1],
+                'unimportant': False
+            }
+            schedulers_and_props_list.append(d)
 
         # post process the schedulernames, and raw properties
         # we do this out of the loop, as this can result in errors
@@ -286,7 +312,10 @@ class Trigger(BuildStep):
                 # put the url to the brids, so that we can have the status from
                 # the beginning
                 url = self.master.status.getURLForBuildrequest(brid)
-                yield self.addURL("%s #%d" % (sch.name, brid), url)
+                yield self.addURL("{} #{}".format(sch.name, brid), url)
+                # No yield since we let this happen as the builds complete
+                self._add_results(brid)
+
             dl.append(resultsDeferred)
             triggeredNames.append(sch.name)
             if self.ended:
@@ -320,4 +349,11 @@ class Trigger(BuildStep):
     def getCurrentSummary(self):
         if not self.triggeredNames:
             return {'step': 'running'}
-        return {'step': 'triggered %s' % (', '.join(self.triggeredNames))}
+        summary = ""
+        if self._result_list:
+            for status in ALL_RESULTS:
+                count = self._result_list.count(status)
+                if count:
+                    summary = summary + ", {} {}".format(self._result_list.count(status),
+                                                     statusToString(status, count))
+        return {'step': 'triggered {}{}'.format(', '.join(self.triggeredNames), summary)}
