@@ -16,10 +16,22 @@
 import os
 import sys
 import warnings
+from pkg_resources import parse_version
 
-from distutils.version import LooseVersion
+import setuptools  # force import setuptools before any other distutils imports
 
 from buildbot import monkeypatches
+from buildbot.test.util.warnings import assertProducesWarning  # noqa pylint: disable=wrong-import-position
+from buildbot.test.util.warnings import assertProducesWarnings  # noqa pylint: disable=wrong-import-position
+from buildbot.warnings import DeprecatedApiWarning  # noqa pylint: disable=wrong-import-position
+
+# import mock so we bail out early if it's not installed
+try:
+    import mock
+    [mock]
+except ImportError as e:
+    raise ImportError("\nBuildbot tests require the 'mock' module; "
+                      "try 'pip install mock'") from e
 
 # apply the same patches the buildmaster does when it starts
 monkeypatches.patch_all(for_tests=True)
@@ -27,43 +39,35 @@ monkeypatches.patch_all(for_tests=True)
 # enable deprecation warnings
 warnings.filterwarnings('always', category=DeprecationWarning)
 
-if sys.version_info[:2] < (3, 2):
-    # Setup logging unhandled messages to stderr.
-    # Since Python 3.2 similar functionality implemented through
-    # logging.lastResort handler.
-    # Significant difference between this approach and Python 3.2 last resort
-    # approach is that in the current approach only records with log level
-    # equal or above to the root logger log level will be printed (WARNING by
-    # default). For example, there still will be warnings about missing
-    # handler for log if INFO or DEBUG records will be logged (but at least
-    # WARNINGs and ERRORs will be printed).
-    import logging
-    _handler = logging.StreamHandler()
-
-    # Ignore following Exception logs:
-    #       Traceback (most recent call last):
-    #       File "[..]site-packages/sqlalchemy/pool.py", line 290, in _close_connection
-    #       self._dialect.do_close(connection)
-    #       File "[..]/sqlalchemy/engine/default.py", line 426, in do_close
-    #       dbapi_connection.close()
-    #       ProgrammingError: SQLite objects created in a thread can only be used in that same thread.
-    #       The object was created in thread id 123145306509312 and this is thread id 140735272824832
-    # sqlalchemy is closing pool connections from the main thread, which sqlite does not like
-    # the warning has been there since forever, but would be caught by the next lastResort logger
-    logging.getLogger("sqlalchemy.pool.SingletonThreadPool").addHandler(None)
-    logging.getLogger().addHandler(_handler)
-# import mock so we bail out early if it's not installed
-try:
-    import mock
-    mock = mock
-except ImportError:
-    raise ImportError("\nBuildbot tests require the 'mock' module; "
-                      "try 'pip install mock'")
-
-if LooseVersion(mock.__version__) < LooseVersion("0.8"):
+if parse_version(mock.__version__) < parse_version("0.8"):
     raise ImportError("\nBuildbot tests require mock version 0.8.0 or "
                       "higher; try 'pip install -U mock'")
 
+[setuptools]  # force use for pylint
+
+# This is where we load deprecated module-level APIs to ignore warning produced by importing them.
+# After the deprecated API has been removed, leave at least one instance of the import in a
+# commented state as reference.
+
+
+with assertProducesWarnings(DeprecatedApiWarning,
+                            messages_patterns=[
+                                r" buildbot\.status\.base has been deprecated",
+                                r" buildbot\.status\.build has been deprecated",
+                                r" buildbot\.status\.buildrequest has been deprecated",
+                                r" buildbot\.status\.event has been deprecated",
+                                r" buildbot\.status\.buildset has been deprecated",
+                                r" buildbot\.status\.master has been deprecated",
+                                r" buildbot\.status\.worker has been deprecated",
+                            ]):
+    import buildbot.status.base as _  # noqa
+    import buildbot.status.build as _  # noqa
+    import buildbot.status.buildrequest as _  # noqa
+    import buildbot.status.event as _  # noqa
+    import buildbot.status.buildset as _  # noqa
+    import buildbot.status.master as _  # noqa
+    import buildbot.status.worker as _  # noqa
+
 # All deprecated modules should be loaded, consider future warnings in tests as errors.
 # In order to not pollute the test outputs,
 # warnings in tests shall be forcefully tested with assertProducesWarning,
@@ -106,10 +110,12 @@ warnings.filterwarnings('ignore', ".*`formatargspec` is deprecated.*", Deprecati
 
 # Python 3.7 adds a deprecation importing ABCs from collection.
 # Such imports are made in dependencies (e.g moto, werzeug, pyparsing)
-warnings.filterwarnings('ignore', ".*Using or importing the ABCs from 'collections'.*", DeprecationWarning)
+warnings.filterwarnings('ignore', ".*Using or importing the ABCs from 'collections'.*",
+                        DeprecationWarning)
 
 # more 3.7 warning from moto
-warnings.filterwarnings('ignore', r".*Use 'list\(elem\)' or iteration over elem instead.*", DeprecationWarning)
+warnings.filterwarnings('ignore', r".*Use 'list\(elem\)' or iteration over elem instead.*",
+                        DeprecationWarning)
 
 # ignore ResourceWarnings for unclosed sockets for the pg8000 driver on Python 3+ (tech debt: #4508)
 if sys.version_info[0] >= 3 and "pg8000" in os.getenv("BUILDBOT_TEST_DB_URL", ""):
@@ -117,3 +123,22 @@ if sys.version_info[0] >= 3 and "pg8000" in os.getenv("BUILDBOT_TEST_DB_URL", ""
 
 # Python 3.5 on CircleCI shows this warning
 warnings.filterwarnings('ignore', ".*the imp module is deprecated in favour of importlib*")
+
+# sqlalchemy-migrate uses deprecated api from sqlalchemy https://review.openstack.org/#/c/648072/
+warnings.filterwarnings('ignore', ".*Engine.contextual_connect.*", DeprecationWarning)
+
+# ignore an attrs API warning for APIs used in dependencies
+warnings.filterwarnings('ignore', ".*The usage of `cmp` is deprecated and will be removed "
+                                  "on or after.*", DeprecationWarning)
+
+# ignore a warning emitted by pkg_resources when importing certain namespace packages
+warnings.filterwarnings('ignore', ".*Not importing directory .*/zope: missing __init__",
+                        category=ImportWarning)
+warnings.filterwarnings('ignore', ".*Not importing directory .*/sphinxcontrib: missing __init__",
+                        category=ImportWarning)
+
+# ignore warnings from importing lib2to3 via buildbot_pkg ->
+# setuptools.command.build_py -> setuptools.lib2to3_ex -> lib2to3
+# https://github.com/pypa/setuptools/issues/2086
+warnings.filterwarnings('ignore', ".*lib2to3 package is deprecated",
+                        category=PendingDeprecationWarning)
