@@ -18,6 +18,7 @@ import os
 
 from twisted.internet import defer
 from twisted.python.filepath import FilePath
+from twisted.spread import pb
 from twisted.trial.unittest import SkipTest
 
 from buildbot.process import properties
@@ -30,17 +31,18 @@ except ImportError:
     RemoteWorker = None
 
 
-class FakeWorker(object):
+class FakeWorker:
     workername = 'test'
 
     def __init__(self, master):
         self.master = master
         self.conn = fakeprotocol.FakeConnection(master, self)
         self.properties = properties.Properties()
+        self.defaultProperties = properties.Properties()
         self.workerid = 383
 
     def acquireLocks(self):
-        pass
+        return True
 
     def releaseLocks(self):
         pass
@@ -74,7 +76,68 @@ class FakeWorker(object):
         pass
 
 
-class WorkerController:
+class SeverWorkerConnectionMixin:
+
+    _connection_severed = False
+    _severed_deferreds = None
+
+    def disconnect_worker(self):
+        if not self._connection_severed:
+            return
+
+        if self._severed_deferreds is not None:
+            for d in self._severed_deferreds:
+                d.errback(pb.PBConnectionLost('lost connection'))
+
+        self._connection_severed = False
+
+    def sever_connection(self):
+        # stubs the worker connection so that it appears that the TCP connection
+        # has been severed in a way that no response is ever received, but
+        # messages don't fail immediately. All callback will be called when
+        # disconnect_worker is called
+        self._connection_severed = True
+
+        def register_deferred():
+            d = defer.Deferred()
+
+            if self._severed_deferreds is None:
+                self._severed_deferreds = []
+            self._severed_deferreds.append(d)
+
+            return d
+
+        def remotePrint(message):
+            return register_deferred()
+        self.worker.conn.remotePrint = remotePrint
+
+        def remoteGetWorkerInfo():
+            return register_deferred()
+        self.worker.conn.remoteGetWorkerInfo = remoteGetWorkerInfo
+
+        def remoteSetBuilderList(builders):
+            return register_deferred()
+        self.worker.conn.remoteSetBuilderList = remoteSetBuilderList
+
+        def remoteStartCommand(remoteCommand, builderName, commandId,
+                               commandName, args):
+            return register_deferred()
+        self.worker.conn.remoteStartCommand = remoteStartCommand
+
+        def remoteShutdown():
+            return register_deferred()
+        self.worker.conn.remoteShutdown = remoteShutdown
+
+        def remoteStartBuild(builderName):
+            return register_deferred()
+        self.worker.conn.remoteStartBuild = remoteStartBuild
+
+        def remoteInterruptCommand(builderName, commandId, why):
+            return register_deferred()
+        self.worker.conn.remoteInterruptCommand = remoteInterruptCommand
+
+
+class WorkerController(SeverWorkerConnectionMixin):
 
     """
     A controller for a ``Worker``.
@@ -103,8 +166,9 @@ class WorkerController:
         self.remote_worker.setServiceParent(self.worker)
 
     def disconnect_worker(self):
+        super().disconnect_worker()
         if self.remote_worker is None:
-            return
+            return None
         self.worker.conn, conn = None, self.worker.conn
         # LocalWorker does actually disconnect, so we must force disconnection
         # via detached
@@ -112,36 +176,3 @@ class WorkerController:
         ret = self.remote_worker.disownServiceParent()
         self.remote_worker = None
         return ret
-
-    def sever_connection(self):
-        # stubs the worker connection so that it appears that the TCP connection
-        # has been severed in a way that no response is ever received, but
-        # messages don't fail immediately.
-        def remotePrint(message):
-            return defer.Deferred()
-        self.worker.conn.remotePrint = remotePrint
-
-        def remoteGetWorkerInfo():
-            return defer.Deferred()
-        self.worker.conn.remoteGetWorkerInfo = remoteGetWorkerInfo
-
-        def remoteSetBuilderList(builders):
-            return defer.Deferred()
-        self.worker.conn.remoteSetBuilderList = remoteSetBuilderList
-
-        def remoteStartCommand(remoteCommand, builderName, commandId,
-                               commandName, args):
-            return defer.Deferred()
-        self.worker.conn.remoteStartCommand = remoteStartCommand
-
-        def remoteShutdown():
-            return defer.Deferred()
-        self.worker.conn.remoteShutdown = remoteShutdown
-
-        def remoteStartBuild(builderName):
-            return defer.Deferred()
-        self.worker.conn.remoteStartBuild = remoteStartBuild
-
-        def remoteInterruptCommand(builderName, commandId, why):
-            return defer.Deferred()
-        self.worker.conn.remoteInterruptCommand = remoteInterruptCommand
