@@ -15,6 +15,8 @@
 
 import os
 
+from parameterized import parameterized
+
 from twisted.internet import defer
 
 from buildbot.process.properties import Interpolate
@@ -24,31 +26,47 @@ from buildbot.test.util.integration import RunMasterBase
 
 
 class FakeSecretReporter(HttpStatusPush):
-    def send(self, build):
+    def sendMessage(self, reports):
         assert self.auth == ('user', 'myhttppasswd')
         self.reported = True
 
 
 class SecretsConfig(RunMasterBase):
 
+    @parameterized.expand([
+        ('with_interpolation', True),
+        ('plain_command', False),
+    ])
     @defer.inlineCallbacks
-    def test_secret(self):
-        c = masterConfig()
+    def test_secret(self, name, use_interpolation):
+        c = masterConfig(use_interpolation)
         yield self.setupConfig(c)
         build = yield self.doForceBuild(wantSteps=True, wantLogs=True)
         self.assertEqual(build['buildid'], 1)
+
+        # check the command line
         res = yield self.checkBuildStepLogExist(build, "echo <foo>")
-        if os.name == "posix":
+
+        # also check the secrets are replaced in argv
+        yield self.checkBuildStepLogExist(build, "argv:.*echo.*<foo>", regex=True)
+
+        # also check that the correct value goes to the command
+        if os.name == "posix" and use_interpolation:
             res &= yield self.checkBuildStepLogExist(build, "The password was there")
+
         self.assertTrue(res)
         # at this point, build contains all the log and steps info that is in the db
         # we check that our secret is not in there!
         self.assertNotIn("bar", repr(build))
         self.assertTrue(c['services'][0].reported)
 
+    @parameterized.expand([
+        ('with_interpolation', True),
+        ('plain_command', False),
+    ])
     @defer.inlineCallbacks
-    def test_secretReconfig(self):
-        c = masterConfig()
+    def test_secretReconfig(self, name, use_interpolation):
+        c = masterConfig(use_interpolation)
         yield self.setupConfig(c)
         c['secretsProviders'] = [FakeSecretStorage(
             secretdict={"foo": "different_value", "something": "more"})]
@@ -67,11 +85,11 @@ class SecretsConfigPB(SecretsConfig):
 
 
 # master configuration
-def masterConfig():
+def masterConfig(use_interpolation):
     c = {}
     from buildbot.config import BuilderConfig
     from buildbot.process.factory import BuildFactory
-    from buildbot.plugins import schedulers, steps
+    from buildbot.plugins import schedulers, steps, util
 
     c['services'] = [FakeSecretReporter('http://example.com/hook',
                                         auth=('user', Interpolate('%(secret:httppasswd)s')))]
@@ -83,12 +101,18 @@ def masterConfig():
     c['secretsProviders'] = [FakeSecretStorage(
         secretdict={"foo": "bar", "something": "more", 'httppasswd': 'myhttppasswd'})]
     f = BuildFactory()
-    if os.name == "posix":
-        f.addStep(steps.ShellCommand(command=Interpolate(
-            'echo %(secret:foo)s | sed "s/bar/The password was there/"')))
+
+    if use_interpolation:
+        if os.name == "posix":
+            # on posix we can also check whether the password was passed to the command
+            command = Interpolate('echo %(secret:foo)s | sed "s/bar/The password was there/"')
+        else:
+            command = Interpolate('echo %(secret:foo)s')
     else:
-        f.addStep(steps.ShellCommand(command=Interpolate(
-            'echo %(secret:foo)s')))
+        command = ['echo', util.Secret('foo')]
+
+    f.addStep(steps.ShellCommand(command=command))
+
     c['builders'] = [
         BuilderConfig(name="testy",
                       workernames=["local1"],
