@@ -14,31 +14,24 @@
 # Copyright Buildbot Team Members
 
 import traceback
-
-import mock
+from io import StringIO
 
 from twisted.internet import defer
 from twisted.internet import error
 from twisted.python import failure
-from twisted.python.compat import NativeStringIO
-from twisted.trial import unittest
 
-from buildbot import config
-from buildbot.process import builder
-from buildbot.process import buildrequest
+from buildbot.config import BuilderConfig
 from buildbot.process import buildstep
-from buildbot.process import factory
+from buildbot.process import logobserver
 from buildbot.process import results
-from buildbot.process import workerforbuilder
+from buildbot.process.factory import BuildFactory
 from buildbot.steps import shell
-from buildbot.test.fake import fakedb
-from buildbot.test.fake import fakemaster
-from buildbot.test.fake import fakeprotocol
-from buildbot.test.util.misc import TestReactorMixin
-from buildbot.worker.base import Worker
+from buildbot.test.util.integration import RunFakeMasterTestCase
+from buildbot.test.util.warnings import assertProducesWarnings
+from buildbot.warnings import DeprecatedApiWarning
 
 
-class TestLogObserver(buildstep.LogObserver):
+class TestLogObserver(logobserver.LogObserver):
 
     def __init__(self):
         self.observed = []
@@ -101,6 +94,23 @@ class Latin1ProducingCustomBuildStep(buildstep.BuildStep):
         return results.SUCCESS
 
 
+class BuildStepWithFailingLogObserver(buildstep.BuildStep):
+
+    @defer.inlineCallbacks
+    def run(self):
+        self.addLogObserver('xx', logobserver.LineConsumerLogObserver(self.log_consumer))
+
+        _log = yield self.addLog('xx')
+        yield _log.addStdout('line1\nline2\n')
+        yield _log.finish()
+
+        return results.SUCCESS
+
+    def log_consumer(self):
+        _, _ = yield
+        raise RuntimeError('fail')
+
+
 class FailingCustomStep(buildstep.LoggingBuildStep):
 
     flunkOnFailure = True
@@ -126,12 +136,12 @@ class OldBuildEPYDoc(shell.ShellCommand):
         return defer.succeed(None)
 
     def createSummary(self, log):
-        for line in NativeStringIO(log.getText()):
+        for line in StringIO(log.getText()):
             # what we do with the line isn't important to the test
             assert line in ('some\n', 'output\n')
 
 
-class OldPerlModuleTest(shell.Test):
+class OldPerlModuleTest(shell.ShellCommand):
 
     command = ['perl']
 
@@ -153,92 +163,47 @@ class OldPerlModuleTest(shell.Test):
         return results.SUCCESS
 
 
-class RunSteps(unittest.TestCase, TestReactorMixin):
+class RunSteps(RunFakeMasterTestCase):
 
     @defer.inlineCallbacks
-    def setUp(self):
-        self.setUpTestReactor()
-        self.master = fakemaster.make_master(self, wantData=True,
-                                             wantMq=True, wantDb=True)
-        self.master.db.insertTestData([
-            fakedb.Builder(id=80, name='test'), ])
+    def create_config_for_step(self, step):
+        config_dict = {
+            'builders': [
+                BuilderConfig(name="builder",
+                              workernames=["worker1"],
+                              factory=BuildFactory([step])
+                              ),
+            ],
+            'workers': [self.createLocalWorker('worker1')],
+            'protocols': {'null': {}},
+            # Disable checks about missing scheduler.
+            'multiMaster': True,
+        }
 
-        self.builder = builder.Builder('test')
-        self.builder._builderid = 80
-        self.builder.config_version = 0
-        self.builder.master = self.master
-        self.builder.botmaster = mock.Mock()
-        self.builder.botmaster.getLockFromLockAccesses = lambda l, c: []
-        yield self.builder.startService()
-
-        self.factory = factory.BuildFactory()  # will have steps added later
-        new_config = config.MasterConfig()
-        new_config.builders.append(
-            config.BuilderConfig(name='test', workername='testworker',
-                                 factory=self.factory))
-        yield self.builder.reconfigServiceWithBuildbotConfig(new_config)
-
-        self.worker = Worker('worker', 'pass')
-        self.worker.sendBuilderList = lambda: defer.succeed(None)
-        self.worker.parent = mock.Mock()
-        self.worker.master.botmaster = mock.Mock()
-        self.worker.botmaster.maybeStartBuildsForWorker = lambda w: None
-        self.worker.botmaster.getBuildersForWorker = lambda w: []
-        self.worker.parent = self.master
-        self.worker.startService()
-        self.conn = fakeprotocol.FakeConnection(self.master, self.worker)
-        yield self.worker.attached(self.conn)
-
-        wfb = self.workerforbuilder = workerforbuilder.WorkerForBuilder()
-        wfb.setBuilder(self.builder)
-        yield wfb.attached(self.worker, {})
-
-        # add the buildset/request
-        self.bsid, brids = yield self.master.db.buildsets.addBuildset(
-            sourcestamps=[{}], reason='x', properties={},
-            builderids=[80], waited_for=False)
-
-        self.brdict = \
-            yield self.master.db.buildrequests.getBuildRequest(brids[80])
-
-        self.buildrequest = \
-            yield buildrequest.BuildRequest.fromBrdict(self.master, self.brdict)
+        yield self.setup_master(config_dict)
+        builder_id = yield self.master.data.updates.findBuilderId('builder')
+        return builder_id
 
     @defer.inlineCallbacks
-    def tearDown(self):
-        yield self.worker.stopService()
-        yield self.builder.stopService()
+    def do_test_build(self, builder_id):
 
-    @defer.inlineCallbacks
-    def do_test_step(self):
-        # patch builder.buildFinished to signal us with a deferred
-        bfd = defer.Deferred()
-        old_buildFinished = self.builder.buildFinished
+        # setup waiting for build to finish
+        d_finished = defer.Deferred()
 
-        def buildFinished(*args):
-            old_buildFinished(*args)
-            bfd.callback(None)
-        self.builder.buildFinished = buildFinished
+        def on_finished(_, __):
+            if not d_finished.called:
+                d_finished.callback(None)
+        consumer = yield self.master.mq.startConsuming(on_finished, ('builds', None, 'finished'))
 
         # start the builder
-        self.assertTrue((yield self.builder.maybeStartBuild(
-            self.workerforbuilder, [self.buildrequest])))
+        yield self.create_build_request([builder_id])
 
-        # and wait for completion
-        yield bfd
-
-        # then get the BuildStatus and return it
-        return self.master.status.lastBuilderStatus.lastBuildStatus
-
-    def assertLogs(self, exp_logs):
-        got_logs = {}
-        for id, l in self.master.data.updates.logs.items():
-            self.assertTrue(l['finished'])
-            got_logs[l['name']] = ''.join(l['content'])
-        self.assertEqual(got_logs, exp_logs)
+        # and wait for build completion
+        yield d_finished
+        yield consumer.stopConsuming()
 
     @defer.inlineCallbacks
-    def doOldStyleCustomBuildStep(self, slowDB=False):
+    def doOldStyleCustomBuildStep(self, builder_id, slowDB=False):
         # patch out addLog to delay until we're ready
         newLogDeferreds = []
         oldNewLog = self.master.data.updates.addLog
@@ -258,11 +223,9 @@ class RunSteps(unittest.TestCase, TestReactorMixin):
             self.patch(OldStyleCustomBuildStep,
                        "_run_finished_hook", finishNewLog)
 
-        self.factory.addStep(OldStyleCustomBuildStep(self.reactor,
-                                                     arg1=1, arg2=2))
-        yield self.do_test_step()
+        yield self.do_test_build(builder_id)
 
-        self.assertLogs({
+        yield self.assertLogs(1, {
             'compl.html': '<blink>A very short logfile</blink>\n',
             # this is one of the things that differs independently of
             # new/old style: encoding of logs and newlines
@@ -277,58 +240,93 @@ class RunSteps(unittest.TestCase, TestReactorMixin):
             'Observer saw [' + repr('stdout\n') + ", " + repr("\u2603\n") + "]\n"
         })
 
+    @defer.inlineCallbacks
     def test_OldStyleCustomBuildStep(self):
-        return self.doOldStyleCustomBuildStep(False)
+        step = OldStyleCustomBuildStep(self.reactor, arg1=1, arg2=2)
+        builder_id = yield self.create_config_for_step(step)
+        yield self.doOldStyleCustomBuildStep(builder_id, False)
 
+    @defer.inlineCallbacks
     def test_OldStyleCustomBuildStepSlowDB(self):
-        return self.doOldStyleCustomBuildStep(True)
+        step = OldStyleCustomBuildStep(self.reactor, arg1=1, arg2=2)
+        builder_id = yield self.create_config_for_step(step)
+        yield self.doOldStyleCustomBuildStep(builder_id, True)
 
     @defer.inlineCallbacks
     def test_OldStyleCustomBuildStep_failure(self):
-        self.factory.addStep(OldStyleCustomBuildStep(self.reactor,
-                                                     arg1=1, arg2=2, doFail=1))
-        bs = yield self.do_test_step()
+        step = OldStyleCustomBuildStep(self.reactor, arg1=1, arg2=2, doFail=1)
+        builder_id = yield self.create_config_for_step(step)
+
+        yield self.do_test_build(builder_id)
+
         self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 1)
-        self.assertEqual(bs.getResults(), results.EXCEPTION)
+        yield self.assertBuildResults(1, results.EXCEPTION)
 
     @defer.inlineCallbacks
     def test_step_raising_buildstepfailed_in_start(self):
-        self.factory.addStep(FailingCustomStep())
-        bs = yield self.do_test_step()
-        self.assertEqual(bs.getResults(), results.FAILURE)
+        builder_id = yield self.create_config_for_step(FailingCustomStep())
+
+        yield self.do_test_build(builder_id)
+        yield self.assertBuildResults(1, results.FAILURE)
 
     @defer.inlineCallbacks
     def test_step_raising_exception_in_start(self):
-        self.factory.addStep(FailingCustomStep(exception=ValueError))
-        bs = yield self.do_test_step()
-        self.assertEqual(bs.getResults(), results.EXCEPTION)
+        builder_id = yield self.create_config_for_step(FailingCustomStep(exception=ValueError))
+
+        yield self.do_test_build(builder_id)
+        yield self.assertBuildResults(1, results.EXCEPTION)
         self.assertEqual(len(self.flushLoggedErrors(ValueError)), 1)
 
     @defer.inlineCallbacks
     def test_step_raising_connectionlost_in_start(self):
-        self.factory.addStep(FailingCustomStep(exception=error.ConnectionLost))
-        bs = yield self.do_test_step()
-        self.assertEqual(bs.getResults(), results.RETRY)
+        ''' Check whether we can recover from raising ConnectionLost from a step if the worker
+            did not actually disconnect
+        '''
+        step = FailingCustomStep(exception=error.ConnectionLost)
+        builder_id = yield self.create_config_for_step(step)
+
+        yield self.do_test_build(builder_id)
+        yield self.assertBuildResults(1, results.EXCEPTION)
+    test_step_raising_connectionlost_in_start.skip = "Results in infinite loop"
+
+    @defer.inlineCallbacks
+    def test_step_raising_in_log_observer(self):
+        step = BuildStepWithFailingLogObserver()
+        builder_id = yield self.create_config_for_step(step)
+
+        yield self.do_test_build(builder_id)
+        yield self.assertBuildResults(1, results.EXCEPTION)
+        yield self.assertStepStateString(2, "finished (exception)")
+        self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 1)
 
     @defer.inlineCallbacks
     def test_Latin1ProducingCustomBuildStep(self):
-        self.factory.addStep(
-            Latin1ProducingCustomBuildStep(logEncoding='latin-1'))
-        yield self.do_test_step()
-        self.assertLogs({
+        step = Latin1ProducingCustomBuildStep(logEncoding='latin-1')
+        builder_id = yield self.create_config_for_step(step)
+
+        yield self.do_test_build(builder_id)
+        yield self.assertLogs(1, {
             'xx': 'o\N{CENT SIGN}\n',
         })
 
     @defer.inlineCallbacks
     def test_OldBuildEPYDoc(self):
         # test old-style calls to log.getText, figuring readlines will be ok
-        self.factory.addStep(OldBuildEPYDoc())
-        bs = yield self.do_test_step()
-        self.assertEqual(bs.getResults(), results.FAILURE)
+        with assertProducesWarnings(DeprecatedApiWarning, num_warnings=2,
+                                    message_pattern='Subclassing old-style step'):
+            step = OldBuildEPYDoc()
+            builder_id = yield self.create_config_for_step(step)
+
+            yield self.do_test_build(builder_id)
+            yield self.assertBuildResults(1, results.FAILURE)
 
     @defer.inlineCallbacks
     def test_OldPerlModuleTest(self):
         # test old-style calls to self.getLog
-        self.factory.addStep(OldPerlModuleTest())
-        bs = yield self.do_test_step()
-        self.assertEqual(bs.getResults(), results.SUCCESS)
+        with assertProducesWarnings(DeprecatedApiWarning, num_warnings=2,
+                                    message_pattern='Subclassing old-style step'):
+            step = OldPerlModuleTest()
+            builder_id = yield self.create_config_for_step(step)
+
+            yield self.do_test_build(builder_id)
+            yield self.assertBuildResults(1, results.SUCCESS)
