@@ -47,7 +47,7 @@ class TestLog(RunMasterBase):
     @defer.inlineCallbacks
     def test_shellcommand(self):
 
-        class MyStep(steps.ShellCommand):
+        class MyStep(steps.ShellCommandNewStyle):
 
             def _newLog(obj, name, type, logid, logEncoding):
                 r = steps.ShellCommand._newLog(obj, name, type, logid, logEncoding)
@@ -61,6 +61,7 @@ class TestLog(RunMasterBase):
         change = dict(branch="master",
                       files=["foo.c"],
                       author="me@foo.com",
+                      committer="me@foo.com",
                       comments="good stuff",
                       revision="HEAD",
                       project="none")
@@ -72,10 +73,10 @@ class TestLog(RunMasterBase):
     @defer.inlineCallbacks
     def test_mastershellcommand(self):
 
-        class MyStep(steps.MasterShellCommand):
+        class MyStep(steps.MasterShellCommandNewStyle):
 
             def _newLog(obj, name, type, logid, logEncoding):
-                r = steps.MasterShellCommand._newLog(obj, name, type, logid, logEncoding)
+                r = steps.MasterShellCommandNewStyle._newLog(obj, name, type, logid, logEncoding)
                 self.curr_log = r
                 return self.curr_log
 
@@ -86,6 +87,7 @@ class TestLog(RunMasterBase):
         change = dict(branch="master",
                       files=["foo.c"],
                       author="me@foo.com",
+                      committer="me@foo.com",
                       comments="good stuff",
                       revision="HEAD",
                       project="none")
@@ -97,10 +99,10 @@ class TestLog(RunMasterBase):
     @defer.inlineCallbacks
     def test_mastershellcommand_issue(self):
 
-        class MyStep(steps.MasterShellCommand):
+        class MyStep(steps.MasterShellCommandNewStyle):
 
             def _newLog(obj, name, type, logid, logEncoding):
-                r = steps.MasterShellCommand._newLog(obj, name, type, logid, logEncoding)
+                r = steps.MasterShellCommandNewStyle._newLog(obj, name, type, logid, logEncoding)
                 self.curr_log = r
                 self.patch(r, "finish", lambda: defer.fail(RuntimeError('Could not finish')))
                 return self.curr_log
@@ -112,6 +114,7 @@ class TestLog(RunMasterBase):
         change = dict(branch="master",
                       files=["foo.c"],
                       author="me@foo.com",
+                      committer="me@foo.com",
                       comments="good stuff",
                       revision="HEAD",
                       project="none")
