@@ -15,18 +15,16 @@
 
 
 from twisted.internet import defer
-from twisted.trial.unittest import TestCase
 from zope.interface import implementer
 
 from buildbot.config import BuilderConfig
 from buildbot.interfaces import IBuildStepFactory
+from buildbot.machine.base import Machine
 from buildbot.process.buildstep import BuildStep
 from buildbot.process.factory import BuildFactory
 from buildbot.process.results import CANCELLED
 from buildbot.test.fake.latent import LatentController
-from buildbot.test.util.integration import getMaster
-from buildbot.test.util.misc import TestReactorMixin
-from buildbot.worker.local import LocalWorker
+from buildbot.test.util.integration import RunFakeMasterTestCase
 
 try:
     from buildbot_worker.bot import LocalWorker as RemoteWorker
@@ -35,7 +33,7 @@ except ImportError:
 
 
 @implementer(IBuildStepFactory)
-class StepController(object):
+class StepController:
 
     def __init__(self, **kwargs):
         self.kwargs = kwargs
@@ -54,20 +52,14 @@ class ControllableStep(BuildStep):
         return self._step_deferred
 
     def __init__(self, step_deferred, **kwargs):
-        BuildStep.__init__(self, **kwargs)
+        super().__init__(**kwargs)
         self._step_deferred = step_deferred
 
     def interrupt(self, reason):
         self._step_deferred.callback(CANCELLED)
 
 
-class Tests(TestCase, TestReactorMixin):
-
-    def setUp(self):
-        self.setUpTestReactor()
-
-    def tearDown(self):
-        self.assertFalse(self.master.running, "master is still running!")
+class Tests(RunFakeMasterTestCase):
 
     @defer.inlineCallbacks
     def test_latent_max_builds(self):
@@ -92,19 +84,19 @@ class Tests(TestCase, TestReactorMixin):
             'protocols': {'null': {}},
             'multiMaster': True,
         }
-        self.master = master = yield getMaster(self, self.reactor, config_dict)
+        yield self.setup_master(config_dict)
         builder_ids = [
-            (yield master.data.updates.findBuilderId('testy-1')),
-            (yield master.data.updates.findBuilderId('testy-2')),
+            (yield self.master.data.updates.findBuilderId('testy-1')),
+            (yield self.master.data.updates.findBuilderId('testy-2')),
         ]
 
         started_builds = []
-        yield master.mq.startConsuming(
+        yield self.master.mq.startConsuming(
             lambda key, build: started_builds.append(build),
             ('builds', None, 'new'))
 
         # Trigger a buildrequest
-        bsid, brids = yield master.data.updates.addBuildset(
+        bsid, brids = yield self.master.data.updates.addBuildset(
             waited_for=False,
             builderids=builder_ids,
             sourcestamps=[
@@ -142,23 +134,23 @@ class Tests(TestCase, TestReactorMixin):
                               factory=BuildFactory([step_controller]),
                               ),
             ],
-            'workers': [LocalWorker('local', max_builds=1)],
+            'workers': [self.createLocalWorker('local', max_builds=1)],
             'protocols': {'null': {}},
             'multiMaster': True,
         }
-        self.master = master = yield getMaster(self, self.reactor, config_dict)
+        yield self.setup_master(config_dict)
         builder_ids = [
-            (yield master.data.updates.findBuilderId('testy-1')),
-            (yield master.data.updates.findBuilderId('testy-2')),
+            (yield self.master.data.updates.findBuilderId('testy-1')),
+            (yield self.master.data.updates.findBuilderId('testy-2')),
         ]
 
         started_builds = []
-        yield master.mq.startConsuming(
+        yield self.master.mq.startConsuming(
             lambda key, build: started_builds.append(build),
             ('builds', None, 'new'))
 
         # Trigger a buildrequest
-        bsid, brids = yield master.data.updates.addBuildset(
+        bsid, brids = yield self.master.data.updates.addBuildset(
             waited_for=False,
             builderids=builder_ids,
             sourcestamps=[
@@ -172,5 +164,89 @@ class Tests(TestCase, TestReactorMixin):
 
         self.assertEqual(len(started_builds), 1)
 
+    @defer.inlineCallbacks
+    def test_worker_registered_to_machine(self):
+        worker = self.createLocalWorker('worker1', machine_name='machine1')
+        machine = Machine('machine1')
+
+        config_dict = {
+            'builders': [
+                BuilderConfig(name="builder1",
+                              workernames=["worker1"],
+                              factory=BuildFactory(),
+                              ),
+            ],
+            'workers': [worker],
+            'machines': [machine],
+            'protocols': {'null': {}},
+            'multiMaster': True,
+        }
+
+        yield self.setup_master(config_dict)
+
+        self.assertIs(worker.machine, machine)
+
+    @defer.inlineCallbacks
+    def test_worker_reconfigure_with_new_builder(self):
+        """
+        Checks if we can successfully reconfigure if we add new builders to worker.
+        """
+        config_dict = {
+            'builders': [
+                BuilderConfig(name="builder1",
+                              workernames=['local1'],
+                              factory=BuildFactory()),
+            ],
+            'workers': [self.createLocalWorker('local1', max_builds=1)],
+            'protocols': {'null': {}},
+            # Disable checks about missing scheduler.
+            'multiMaster': True,
+        }
+        yield self.setup_master(config_dict)
+
+        config_dict['builders'] += [
+            BuilderConfig(name="builder2",
+                          workernames=['local1'],
+                          factory=BuildFactory()),
+        ]
+        config_dict['workers'] = [self.createLocalWorker('local1', max_builds=2)]
+
+        # reconfig should succeed
+        yield self.reconfig_master(config_dict)
+
+    @defer.inlineCallbacks
+    def test_worker_os_release_info_roundtrip(self):
+        """
+        Checks if we can successfully get information about the platform the worker is running on.
+        This is very similar to test_worker_comm.TestWorkerComm.test_worker_info, except that
+        we check details such as whether the information is passed in correct encoding.
+        """
+        worker = self.createLocalWorker('local1')
+
+        config_dict = {
+            'builders': [
+                BuilderConfig(name="builder1",
+                              workernames=['local1'],
+                              factory=BuildFactory()),
+            ],
+            'workers': [worker],
+            'protocols': {'null': {}},
+            # Disable checks about missing scheduler.
+            'multiMaster': True,
+        }
+        yield self.setup_master(config_dict)
+
+        props = worker.worker_status.info
+
+        from buildbot_worker.base import BotBase
+
+        expected_props_dict = {}
+        BotBase._read_os_release(BotBase.os_release_file, expected_props_dict)
+
+        for key, value in expected_props_dict.items():
+            self.assertTrue(isinstance(key, str))
+            self.assertTrue(isinstance(value, str))
+            self.assertEqual(props.getProperty(key), value)
+
     if RemoteWorker is None:
         skip = "buildbot-worker package is not installed"
