@@ -14,12 +14,16 @@
 # Copyright Buildbot Team Members
 
 
+import os
+
 import mock
 
 from twisted.cred import credentials
 from twisted.internet import defer
 from twisted.internet import reactor
+from twisted.internet.endpoints import clientFromString
 from twisted.python import log
+from twisted.python import util
 from twisted.spread import pb
 from twisted.trial import unittest
 
@@ -32,9 +36,12 @@ from buildbot.process import builder
 from buildbot.process import factory
 from buildbot.status import master
 from buildbot.test.fake import fakemaster
+from buildbot.test.util.misc import TestReactorMixin
 from buildbot.util.eventual import eventually
 from buildbot.worker import manager as workermanager
 
+PKI_DIR = util.sibpath(__file__, 'pki')
+
 
 class FakeWorkerForBuilder(pb.Referenceable):
 
@@ -87,7 +94,13 @@ class FakeWorkerWorker(pb.Referenceable):
             'info': 'here',
             'worker_commands': {
                 'x': 1,
-            }
+            },
+            'numcpus': 1,
+            'none': None,
+            'os_release': b'\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88'.decode(),
+            b'\xe3\x83\xaa\xe3\x83\xaa\xe3\x83\xbc\xe3\x82\xb9\xe3'
+            b'\x83\x86\xe3\x82\xb9\xe3\x83\x88'.decode():
+                b'\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88'.decode(),
         }
 
     def remote_getVersion(self):
@@ -106,11 +119,10 @@ class FakeWorkerWorker(pb.Referenceable):
 class FakeBuilder(builder.Builder):
 
     def __init__(self, name):
-        builder.Builder.__init__(self, name)
+        super().__init__(name)
         self.builder_status = mock.Mock()
 
     def attached(self, worker, commands):
-        assert commands == {'x': 1}
         return defer.succeed(None)
 
     def detached(self, worker):
@@ -127,15 +139,15 @@ class MyWorker(worker.Worker):
 
     def attached(self, conn):
         self.detach_d = defer.Deferred()
-        return worker.Worker.attached(self, conn)
+        return super().attached(conn)
 
     def detached(self):
-        worker.Worker.detached(self)
+        super().detached()
         self.detach_d, d = None, self.detach_d
         d.callback(None)
 
 
-class TestWorkerComm(unittest.TestCase):
+class TestWorkerComm(unittest.TestCase, TestReactorMixin):
 
     """
     Test handling of connections from workers as integrated with
@@ -149,31 +161,36 @@ class TestWorkerComm(unittest.TestCase):
     @ivar worker: master-side L{Worker} instance
     @ivar workerworker: worker-side L{FakeWorkerWorker} instance
     @ivar port: TCP port to connect to
-    @ivar connector: outbound TCP connection from worker to master
+    @ivar server_connection_string: description string for the server endpoint
+    @ivar client_connection_string_tpl: description string template for the client
+                                endpoint (expects to passed 'port')
+    @ivar endpoint: endpoint controlling the outbound connection
+                    from worker to master
     """
 
     @defer.inlineCallbacks
     def setUp(self):
-        self.master = fakemaster.make_master(testcase=self, wantMq=True,
-                                             wantData=True, wantDb=True)
+        self.setUpTestReactor()
+        self.master = fakemaster.make_master(self, wantMq=True, wantData=True,
+                                             wantDb=True)
 
         # set the worker port to a loopback address with unspecified
         # port
         self.pbmanager = self.master.pbmanager = pbmanager.PBManager()
-        self.pbmanager.setServiceParent(self.master)
+        yield self.pbmanager.setServiceParent(self.master)
 
         # remove the fakeServiceParent from fake service hierarchy, and replace
         # by a real one
         yield self.master.workers.disownServiceParent()
         self.workers = self.master.workers = workermanager.WorkerManager(
             self.master)
-        self.workers.setServiceParent(self.master)
+        yield self.workers.setServiceParent(self.master)
 
         self.botmaster = botmaster.BotMaster()
-        self.botmaster.setServiceParent(self.master)
+        yield self.botmaster.setServiceParent(self.master)
 
         self.master.status = master.Status()
-        self.master.status.setServiceParent(self.master)
+        yield self.master.status.setServiceParent(self.master)
         self.master.botmaster = self.botmaster
         self.master.data.updates.workerConfigured = lambda *a, **k: None
         yield self.master.startService()
@@ -181,15 +198,21 @@ class TestWorkerComm(unittest.TestCase):
         self.buildworker = None
         self.port = None
         self.workerworker = None
-        self.connectors = []
+        self.endpoint = None
+        self.broker = None
         self._detach_deferreds = []
 
         # patch in our FakeBuilder for the regular Builder class
         self.patch(botmaster, 'Builder', FakeBuilder)
 
+        self.server_connection_string = "tcp:0:interface=127.0.0.1"
+        self.client_connection_string_tpl = "tcp:host=127.0.0.1:port={port}"
+
     def tearDown(self):
-        for connector in self.connectors:
-            connector.disconnect()
+        if self.broker:
+            del self.broker
+        if self.endpoint:
+            del self.endpoint
         deferreds = self._detach_deferreds + [
             self.pbmanager.stopService(),
             self.botmaster.stopService(),
@@ -213,10 +236,11 @@ class TestWorkerComm(unittest.TestCase):
 
         # reconfig the master to get it set up
         new_config = self.master.config
-        new_config.protocols = {"pb": {"port": "tcp:0:interface=127.0.0.1"}}
+        new_config.protocols = {"pb": {"port": self.server_connection_string}}
         new_config.workers = [self.buildworker]
-        new_config.builders = [config.BuilderConfig(name='bldr',
-                                                    workername='testworker', factory=factory.BuildFactory())]
+        new_config.builders = [config.BuilderConfig(
+            name='bldr',
+            workername='testworker', factory=factory.BuildFactory())]
 
         yield self.botmaster.reconfigServiceWithBuildbotConfig(new_config)
         yield self.workers.reconfigServiceWithBuildbotConfig(new_config)
@@ -248,18 +272,21 @@ class TestWorkerComm(unittest.TestCase):
 
             # set up to hear when the worker side disconnects
             workerworker.detach_d = defer.Deferred()
-            persp.broker.notifyOnDisconnect(lambda:
-                                            workerworker.detach_d.callback(None))
+            persp.broker.notifyOnDisconnect(
+                lambda: workerworker.detach_d.callback(None))
             self._detach_deferreds.append(workerworker.detach_d)
 
             return workerworker
 
-        self.connectors.append(
-            reactor.connectTCP("127.0.0.1", self.port, factory))
+        self.endpoint = clientFromString(
+                reactor, self.client_connection_string_tpl.format(port=self.port))
+        connected_d = self.endpoint.connect(factory)
 
-        if not waitForBuilderList:
-            return login_d
-        d = defer.DeferredList([login_d, setBuilderList_d],
+        dlist = [connected_d, login_d]
+        if waitForBuilderList:
+            dlist.append(setBuilderList_d)
+
+        d = defer.DeferredList(dlist,
                                consumeErrors=True, fireOnOneErrback=True)
         d.addCallback(lambda _: workerworker)
         return d
@@ -283,7 +310,59 @@ class TestWorkerComm(unittest.TestCase):
         yield worker.waitForDetach()
 
     @defer.inlineCallbacks
-    def test_duplicate_worker(self):
+    def test_tls_connect_disconnect(self):
+        """Test with TLS or SSL endpoint.
+
+        According to the deprecation note for the SSL client endpoint,
+        the TLS endpoint is supported from Twistd 16.0.
+
+        TODO add certificate verification (also will require some conditionals
+        on various versions, including PyOpenSSL, service_identity. The CA used
+        to generate the testing cert is in ``PKI_DIR/ca``
+        """
+        def escape_colon(path):
+            # on windows we can't have \ as it serves as the escape character for :
+            return path.replace('\\', '/').replace(':', '\\:')
+        self.server_connection_string = (
+            "ssl:port=0:certKey={pub}:privateKey={priv}:" +
+            "interface=127.0.0.1").format(
+                pub=escape_colon(os.path.join(PKI_DIR, '127.0.0.1.crt')),
+                priv=escape_colon(os.path.join(PKI_DIR, '127.0.0.1.key')))
+        self.client_connection_string_tpl = "ssl:host=127.0.0.1:port={port}"
+
+        yield self.addWorker()
+
+        # connect
+        worker = yield self.connectWorker()
+
+        # disconnect
+        self.workerSideDisconnect(worker)
+
+        # wait for the resulting detach
+        yield worker.waitForDetach()
+
+    @defer.inlineCallbacks
+    def test_worker_info(self):
+        yield self.addWorker()
+        worker = yield self.connectWorker()
+        props = self.buildworker.worker_status.info
+        # check worker info passing
+        self.assertEqual(props.getProperty("info"),
+                         "here")
+        # check worker info passing with UTF-8
+        self.assertEqual(props.getProperty("os_release"),
+                         b'\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88'.decode())
+        self.assertEqual(props.getProperty(b'\xe3\x83\xaa\xe3\x83\xaa\xe3\x83\xbc\xe3\x82'
+                                           b'\xb9\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88'.decode()),
+                         b'\xe3\x83\x86\xe3\x82\xb9\xe3\x83\x88'.decode())
+        self.assertEqual(props.getProperty("none"), None)
+        self.assertEqual(props.getProperty("numcpus"), 1)
+
+        self.workerSideDisconnect(worker)
+        yield worker.waitForDetach()
+
+    @defer.inlineCallbacks
+    def _test_duplicate_worker(self):
         yield self.addWorker()
 
         # connect first worker
@@ -306,7 +385,7 @@ class TestWorkerComm(unittest.TestCase):
         self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 1)
 
     @defer.inlineCallbacks
-    def test_duplicate_worker_old_dead(self):
+    def _test_duplicate_worker_old_dead(self):
         yield self.addWorker()
 
         # connect first worker
