@@ -28,13 +28,13 @@ from buildbot.buildbot_net_usage_data import _sendBuildbotNetUsageData
 from buildbot.buildbot_net_usage_data import computeUsageData
 from buildbot.buildbot_net_usage_data import linux_distribution
 from buildbot.config import BuilderConfig
-from buildbot.config import ConfigWarning
 from buildbot.master import BuildMaster
 from buildbot.plugins import steps
 from buildbot.process.factory import BuildFactory
 from buildbot.schedulers.forcesched import ForceScheduler
 from buildbot.test.util.integration import DictLoader
 from buildbot.test.util.warnings import assertProducesWarning
+from buildbot.warnings import ConfigWarning
 from buildbot.worker.base import Worker
 
 
@@ -76,11 +76,12 @@ class Tests(unittest.TestCase):
             master = self.getMaster(self.getBaseConfig())
         data = computeUsageData(master)
         self.assertEqual(sorted(data.keys()),
-                         sorted(['versions', 'db', 'platform', 'installid', 'mq', 'plugins', 'www_plugins']))
+                         sorted(['versions', 'db', 'platform', 'installid', 'mq', 'plugins',
+                                 'www_plugins']))
         self.assertEqual(data['plugins']['buildbot/worker/base/Worker'], 3)
         self.assertEqual(sorted(data['plugins'].keys()), sorted(
             ['buildbot/schedulers/forcesched/ForceScheduler', 'buildbot/worker/base/Worker',
-             'buildbot/steps/shell/ShellCommand', 'buildbot/config/BuilderConfig']))
+             'buildbot/steps/shell_oldstyle/ShellCommand', 'buildbot/config/BuilderConfig']))
 
     def test_full(self):
         c = self.getBaseConfig()
@@ -105,7 +106,7 @@ class Tests(unittest.TestCase):
     def test_urllib(self):
         self.patch(buildbot.buildbot_net_usage_data, '_sendWithRequests', lambda _, __: None)
 
-        class FakeRequest(object):
+        class FakeRequest:
 
             def __init__(self, *args, **kwargs):
                 self.args = args
@@ -113,7 +114,7 @@ class Tests(unittest.TestCase):
 
         open_url = []
 
-        class urlopen(object):
+        class urlopen:
 
             def __init__(self, r):
                 self.request = r
@@ -150,5 +151,5 @@ class Tests(unittest.TestCase):
         self.assertEqual(len(distro), 2)
         self.assertNotIn("unknown", distro[0])
         # Rolling distributions like Arch Linux (arch) does not have VERSION_ID
-        if distro[0] != "arch":
+        if distro[0] not in ["arch", "gentoo"]:
             self.assertNotIn("unknown", distro[1])
