@@ -38,13 +38,13 @@ from buildbot.changes import base as changes_base
 from buildbot.process import factory
 from buildbot.process import properties
 from buildbot.schedulers import base as schedulers_base
-from buildbot.status import base as status_base
 from buildbot.test.util import dirs
 from buildbot.test.util.config import ConfigErrorsMixin
 from buildbot.test.util.warnings import assertNotProducesWarnings
 from buildbot.test.util.warnings import assertProducesWarning
 from buildbot.util import service
-from buildbot.worker_transition import DeprecatedWorkerAPIWarning
+from buildbot.warnings import ConfigWarning
+from buildbot.warnings import DeprecatedApiWarning
 
 try:
     # Python 2
@@ -79,26 +79,35 @@ global_defaults = dict(
 class FakeChangeSource(changes_base.ChangeSource):
 
     def __init__(self):
-        changes_base.ChangeSource.__init__(self, name='FakeChangeSource')
-
-
-class FakeStatusReceiver(status_base.StatusReceiver):
-    pass
+        super().__init__(name='FakeChangeSource')
 
 
 @implementer(interfaces.IScheduler)
-class FakeScheduler(object):
+class FakeScheduler:
 
     def __init__(self, name):
         self.name = name
 
 
-class FakeBuilder(object):
+class FakeBuilder:
 
     def __init__(self, **kwargs):
         self.__dict__.update(kwargs)
 
 
+@implementer(interfaces.IWorker)
+class FakeWorker:
+
+    def __init__(self, **kwargs):
+        self.__dict__.update(kwargs)
+
+
+@implementer(interfaces.IMachine)
+class FakeMachine:
+    def __init__(self, **kwargs):
+        self.__dict__.update(kwargs)
+
+
 class ConfigErrors(unittest.TestCase):
 
     def test_constr(self):
@@ -362,6 +371,7 @@ class MasterConfig(ConfigErrorsMixin, dirs.DirsMixin, unittest.TestCase):
         self.assertTrue(rv.load_builders.called)
         self.assertTrue(rv.load_workers.called)
         self.assertTrue(rv.load_change_sources.called)
+        self.assertTrue(rv.load_machines.called)
         self.assertTrue(rv.load_user_managers.called)
 
         self.assertTrue(rv.check_single_master.called)
@@ -369,6 +379,7 @@ class MasterConfig(ConfigErrorsMixin, dirs.DirsMixin, unittest.TestCase):
         self.assertTrue(rv.check_locks.called)
         self.assertTrue(rv.check_builders.called)
         self.assertTrue(rv.check_ports.called)
+        self.assertTrue(rv.check_machines.called)
 
     def test_preChangeGenerator(self):
         cfg = config.MasterConfig()
@@ -450,6 +461,10 @@ class MasterConfig_loaders(ConfigErrorsMixin, unittest.TestCase):
     def test_load_global_title(self):
         self.do_test_load_global(dict(title='hi'), title='hi')
 
+    def test_load_global_title_too_long(self):
+        with assertProducesWarning(ConfigWarning, message_pattern=r"Title is too long"):
+            self.do_test_load_global(dict(title="Very very very very very long title"))
+
     def test_load_global_projectURL(self):
         self.do_test_load_global(dict(projectName='hey'), title='hey')
 
@@ -466,23 +481,21 @@ class MasterConfig_loaders(ConfigErrorsMixin, unittest.TestCase):
         self.do_test_load_global(dict(changeHorizon=None), changeHorizon=None)
 
     def test_load_global_eventHorizon(self):
-        with assertProducesWarning(
-                config.ConfigWarning,
-                message_pattern=r"`eventHorizon` is deprecated and ignored"):
+        with assertProducesWarning(DeprecatedApiWarning,
+                                   message_pattern=r"`eventHorizon` is deprecated and ignored"):
             self.do_test_load_global(
                 dict(eventHorizon=10))
 
     def test_load_global_status(self):
-        with assertProducesWarning(
-                config.ConfigWarning,
-                message_pattern=r"`status` targets are deprecated and ignored"):
+        with assertProducesWarning(DeprecatedApiWarning,
+                                   message_pattern=r"`status` targets are deprecated and ignored"):
             self.do_test_load_global(
                 dict(status=[]))
 
     def test_load_global_buildbotNetUsageData(self):
         self.patch(config, "_in_unit_tests", False)
         with assertProducesWarning(
-                config.ConfigWarning,
+                ConfigWarning,
                 message_pattern=r"`buildbotNetUsageData` is not configured and defaults to basic."):
             self.do_test_load_global(
                 dict())
@@ -615,7 +628,7 @@ class MasterConfig_loaders(ConfigErrorsMixin, unittest.TestCase):
     def test_load_db_db_poll_interval(self):
         # value is ignored, but no error
         with assertProducesWarning(
-                config.ConfigWarning,
+                DeprecatedApiWarning,
                 message_pattern=r"db_poll_interval is deprecated and will be ignored"):
             self.cfg.load_db(self.filename, dict(db_poll_interval=2))
         self.assertResults(
@@ -624,7 +637,7 @@ class MasterConfig_loaders(ConfigErrorsMixin, unittest.TestCase):
     def test_load_db_dict(self):
         # db_poll_interval value is ignored, but no error
         with assertProducesWarning(
-                config.ConfigWarning,
+                DeprecatedApiWarning,
                 message_pattern=r"db_poll_interval is deprecated and will be ignored"):
             self.cfg.load_db(self.filename,
                              dict(db=dict(db_url='abcd', db_poll_interval=10)))
@@ -632,7 +645,7 @@ class MasterConfig_loaders(ConfigErrorsMixin, unittest.TestCase):
 
     def test_load_db_unk_keys(self):
         with assertProducesWarning(
-                config.ConfigWarning,
+                DeprecatedApiWarning,
                 message_pattern=r"db_poll_interval is deprecated and will be ignored"):
             self.cfg.load_db(self.filename,
                              dict(db=dict(db_url='abcd', db_poll_interval=10, bar='bar')))
@@ -857,6 +870,27 @@ class MasterConfig_loaders(ConfigErrorsMixin, unittest.TestCase):
                                      dict(change_source=[chsrc]))
         self.assertResults(change_sources=[chsrc])
 
+    def test_load_machines_defaults(self):
+        self.cfg.load_machines(self.filename, {})
+        self.assertResults(machines=[])
+
+    def test_load_machines_not_instance(self):
+        self.cfg.load_machines(self.filename,
+                               dict(machines=[mock.Mock()]))
+        self.assertConfigError(self.errors, "must be a list of")
+
+    def test_load_machines_single(self):
+        mm = FakeMachine(name='a')
+        self.cfg.load_machines(self.filename,
+                               dict(machines=mm))
+        self.assertConfigError(self.errors, "must be a list of")
+
+    def test_load_machines_list(self):
+        mm = FakeMachine()
+        self.cfg.load_machines(self.filename,
+                               dict(machines=[mm]))
+        self.assertResults(machines=[mm])
+
     def test_load_user_managers_defaults(self):
         self.cfg.load_user_managers(self.filename, {})
         self.assertResults(user_managers=[])
@@ -975,7 +1009,7 @@ class MasterConfig_loaders(ConfigErrorsMixin, unittest.TestCase):
 
     def test_load_services_badservice(self):
 
-        class MyService(object):
+        class MyService:
             pass
         myService = MyService()
         self.cfg.load_services(self.filename, dict(
@@ -1062,11 +1096,10 @@ class MasterConfig_checkers(ConfigErrorsMixin, unittest.TestCase):
             return b
 
         def lock(name):
-            lock = mock.Mock(spec=locks.MasterLock)
-            lock.name = name
+            lock = locks.MasterLock(name)
             if bare_builder_lock:
                 return lock
-            return locks.LockAccess(lock, "counting", _skipChecks=True)
+            return locks.LockAccess(lock, "counting", count=1)
 
         b1, b2 = bldr('b1'), bldr('b2')
         self.cfg.builders = [b1, b2]
@@ -1224,6 +1257,25 @@ class MasterConfig_checkers(ConfigErrorsMixin, unittest.TestCase):
         self.assertConfigError(self.errors,
                                "Some of ports in c['protocols'] duplicated")
 
+    def test_check_machines_unknown_name(self):
+        self.cfg.workers = [
+            FakeWorker(name='wa', machine_name='unk')
+        ]
+        self.cfg.machines = [
+            FakeMachine(name='a')
+        ]
+        self.cfg.check_machines()
+        self.assertConfigError(self.errors, 'uses unknown machine')
+
+    def test_check_machines_duplicate_name(self):
+        self.cfg.machines = [
+            FakeMachine(name='a'),
+            FakeMachine(name='a')
+        ]
+        self.cfg.check_machines()
+        self.assertConfigError(self.errors,
+                               'duplicate machine name')
+
 
 class MasterConfig_old_worker_api(unittest.TestCase):
 
@@ -1233,7 +1285,7 @@ class MasterConfig_old_worker_api(unittest.TestCase):
         self.cfg = config.MasterConfig()
 
     def test_workers_new_api(self):
-        with assertNotProducesWarnings(DeprecatedWorkerAPIWarning):
+        with assertNotProducesWarnings(DeprecatedApiWarning):
             self.assertEqual(self.cfg.workers, [])
 
 
@@ -1292,7 +1344,7 @@ class BuilderConfig(ConfigErrorsMixin, unittest.TestCase):
 
     def test_bogus_category(self):
         with assertProducesWarning(
-                config.ConfigWarning,
+                DeprecatedApiWarning,
                 message_pattern=r"builder categories are deprecated and should be replaced with"):
             with self.assertRaisesConfigError("category must be a string"):
                 config.BuilderConfig(category=13,
@@ -1437,7 +1489,7 @@ class BuilderConfig(ConfigErrorsMixin, unittest.TestCase):
         self.assertEqual(cfg.workernames, ['a'])
 
     def test_init_workername_positional(self):
-        with assertNotProducesWarnings(DeprecatedWorkerAPIWarning):
+        with assertNotProducesWarnings(DeprecatedApiWarning):
             cfg = config.BuilderConfig(
                 'a b c', 'a', factory=self.factory)
 
@@ -1450,7 +1502,7 @@ class BuilderConfig(ConfigErrorsMixin, unittest.TestCase):
         self.assertEqual(cfg.workernames, ['a'])
 
     def test_init_workernames_positional(self):
-        with assertNotProducesWarnings(DeprecatedWorkerAPIWarning):
+        with assertNotProducesWarnings(DeprecatedApiWarning):
             cfg = config.BuilderConfig(
                 'a b c', None, ['a'], factory=self.factory)
 
@@ -1464,7 +1516,7 @@ class BuilderConfig(ConfigErrorsMixin, unittest.TestCase):
         self.assertEqual(cfg.workerbuilddir, 'dir')
 
     def test_init_workerbuilddir_positional(self):
-        with assertNotProducesWarnings(DeprecatedWorkerAPIWarning):
+        with assertNotProducesWarnings(DeprecatedApiWarning):
             cfg = config.BuilderConfig(
                 'a b c', 'a', None, None, 'dir', factory=self.factory)
 
@@ -1480,7 +1532,7 @@ class BuilderConfig(ConfigErrorsMixin, unittest.TestCase):
 
     def test_init_next_worker_positional(self):
         f = lambda: None
-        with assertNotProducesWarnings(DeprecatedWorkerAPIWarning):
+        with assertNotProducesWarnings(DeprecatedApiWarning):
             cfg = config.BuilderConfig(
                 'a b c', 'a', None, None, None, self.factory, None, None, f)
 
@@ -1497,8 +1549,7 @@ class FakeService(service.ReconfigurableServiceMixin,
     def reconfigServiceWithBuildbotConfig(self, new_config):
         self.called = FakeService.call_index
         FakeService.call_index += 1
-        yield service.ReconfigurableServiceMixin.reconfigServiceWithBuildbotConfig(
-            self, new_config)
+        yield super().reconfigServiceWithBuildbotConfig(new_config)
         if not self.succeed:
             raise ValueError("oh noes")
 
@@ -1508,8 +1559,7 @@ class FakeMultiService(service.ReconfigurableServiceMixin,
 
     def reconfigServiceWithBuildbotConfig(self, new_config):
         self.called = True
-        d = service.ReconfigurableServiceMixin.reconfigServiceWithBuildbotConfig(
-            self, new_config)
+        d = super().reconfigServiceWithBuildbotConfig(new_config)
         return d
 
 
@@ -1537,11 +1587,11 @@ class ReconfigurableServiceMixin(unittest.TestCase):
     def test_multiservice(self):
         svc = FakeMultiService()
         ch1 = FakeService()
-        ch1.setServiceParent(svc)
+        yield ch1.setServiceParent(svc)
         ch2 = FakeMultiService()
-        ch2.setServiceParent(svc)
+        yield ch2.setServiceParent(svc)
         ch3 = FakeService()
-        ch3.setServiceParent(ch2)
+        yield ch3.setServiceParent(ch2)
         yield svc.reconfigServiceWithBuildbotConfig(mock.Mock())
 
         self.assertTrue(svc.called)
@@ -1553,13 +1603,13 @@ class ReconfigurableServiceMixin(unittest.TestCase):
     def test_multiservice_priority(self):
         parent = FakeMultiService()
         svc128 = FakeService()
-        svc128.setServiceParent(parent)
+        yield svc128.setServiceParent(parent)
 
         services = [svc128]
         for i in range(20, 1, -1):
             svc = FakeService()
             svc.reconfig_priority = i
-            svc.setServiceParent(parent)
+            yield svc.setServiceParent(parent)
             services.append(svc)
 
         yield parent.reconfigServiceWithBuildbotConfig(mock.Mock())
@@ -1572,7 +1622,7 @@ class ReconfigurableServiceMixin(unittest.TestCase):
     def test_multiservice_nested_failure(self):
         svc = FakeMultiService()
         ch1 = FakeService()
-        ch1.setServiceParent(svc)
+        yield ch1.setServiceParent(svc)
         ch1.succeed = False
         try:
             yield svc.reconfigServiceWithBuildbotConfig(mock.Mock())
