@@ -9,25 +9,29 @@ from buildbot.test.fake import fakemaster
 from buildbot.test.fake.fakebuild import FakeBuild
 from buildbot.test.fake.secrets import FakeSecretStorage
 from buildbot.test.util.config import ConfigErrorsMixin
+from buildbot.test.util.misc import TestReactorMixin
 
 
 class FakeBuildWithMaster(FakeBuild):
 
     def __init__(self, master):
-        super(FakeBuildWithMaster, self).__init__()
+        super().__init__()
         self.master = master
 
 
-class TestInterpolateSecrets(unittest.TestCase, ConfigErrorsMixin):
+class TestInterpolateSecrets(TestReactorMixin, unittest.TestCase,
+                             ConfigErrorsMixin):
 
+    @defer.inlineCallbacks
     def setUp(self):
-        self.master = fakemaster.make_master()
+        self.setUpTestReactor()
+        self.master = fakemaster.make_master(self)
         fakeStorageService = FakeSecretStorage()
         fakeStorageService.reconfigService(secretdict={"foo": "bar",
                                                        "other": "value"})
         self.secretsrv = SecretManager()
         self.secretsrv.services = [fakeStorageService]
-        self.secretsrv.setServiceParent(self.master)
+        yield self.secretsrv.setServiceParent(self.master)
         self.build = FakeBuildWithMaster(self.master)
 
     @defer.inlineCallbacks
@@ -45,10 +49,12 @@ class TestInterpolateSecrets(unittest.TestCase, ConfigErrorsMixin):
         self.flushLoggedErrors(KeyError)
 
 
-class TestInterpolateSecretsNoService(unittest.TestCase, ConfigErrorsMixin):
+class TestInterpolateSecretsNoService(TestReactorMixin, unittest.TestCase,
+                                      ConfigErrorsMixin):
 
     def setUp(self):
-        self.master = fakemaster.make_master()
+        self.setUpTestReactor()
+        self.master = fakemaster.make_master(self)
         self.build = FakeBuildWithMaster(self.master)
 
     @defer.inlineCallbacks
@@ -60,16 +66,19 @@ class TestInterpolateSecretsNoService(unittest.TestCase, ConfigErrorsMixin):
         self.flushLoggedErrors(KeyError)
 
 
-class TestInterpolateSecretsHiddenSecrets(unittest.TestCase):
+class TestInterpolateSecretsHiddenSecrets(TestReactorMixin, unittest.TestCase):
 
+    @defer.inlineCallbacks
     def setUp(self):
-        self.master = fakemaster.make_master()
+        self.setUpTestReactor()
+        self.master = fakemaster.make_master(self)
         fakeStorageService = FakeSecretStorage()
-        fakeStorageService.reconfigService(secretdict={"foo": "bar",
-                                                       "other": "value"})
+        password = "bar"
+        secretdict = {"foo": password, "other": password + 'random'}
+        fakeStorageService.reconfigService(secretdict=secretdict)
         self.secretsrv = SecretManager()
         self.secretsrv.services = [fakeStorageService]
-        self.secretsrv.setServiceParent(self.master)
+        yield self.secretsrv.setServiceParent(self.master)
         self.build = FakeBuildWithMaster(self.master)
 
     @defer.inlineCallbacks
@@ -78,3 +87,10 @@ class TestInterpolateSecretsHiddenSecrets(unittest.TestCase):
         rendered = yield self.build.render(command)
         cleantext = self.build.build_status.properties.cleanupTextFromSecrets(rendered)
         self.assertEqual(cleantext, "echo <foo>")
+
+    @defer.inlineCallbacks
+    def test_secret_replace(self):
+        command = Interpolate("echo %(secret:foo)s %(secret:other)s")
+        rendered = yield self.build.render(command)
+        cleantext = self.build.build_status.properties.cleanupTextFromSecrets(rendered)
+        self.assertEqual(cleantext, "echo <foo> <other>")
