@@ -30,23 +30,24 @@ from buildbot import master
 from buildbot import monkeypatches
 from buildbot.db import exceptions
 from buildbot.interfaces import IConfigLoader
+from buildbot.test import fakedb
 from buildbot.test.fake import fakedata
-from buildbot.test.fake import fakedb
 from buildbot.test.fake import fakemq
 from buildbot.test.fake.botmaster import FakeBotMaster
 from buildbot.test.util import dirs
 from buildbot.test.util import logging
+from buildbot.test.util.misc import TestReactorMixin
 
 
 @implementer(IConfigLoader)
-class FailingLoader(object):
+class FailingLoader:
 
     def loadConfig(self):
         config.error('oh noes')
 
 
 @implementer(IConfigLoader)
-class DefaultLoader(object):
+class DefaultLoader:
 
     def loadConfig(self):
         return config.MasterConfig()
@@ -65,16 +66,19 @@ class InitTests(unittest.SynchronousTestCase):
 
     def test_configfile_default(self):
         """
-        If neither configfile nor config_loader are specified, The default config_loader is a `FileLoader` pointing at `"master.cfg"`.
+        If neither configfile nor config_loader are specified, The default config_loader is a
+        `FileLoader` pointing at `"master.cfg"`.
         """
         m = master.BuildMaster(".", reactor=reactor)
         self.assertEqual(m.config_loader, config.FileLoader(".", "master.cfg"))
 
 
-class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase):
+class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin,
+                         TestReactorMixin, unittest.TestCase):
 
     @defer.inlineCallbacks
     def setUp(self):
+        self.setUpTestReactor()
         self.setUpLogging()
         self.basedir = os.path.abspath('basedir')
         yield self.setUpDirs(self.basedir)
@@ -90,29 +94,21 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase
         self.patch(master, 'Status', lambda master: mock.Mock())
 
         master.BuildMaster.masterHeartbeatService = mock.Mock()
-        self.reactor = self.make_reactor()
         self.master = master.BuildMaster(
             self.basedir, reactor=self.reactor, config_loader=DefaultLoader())
         self.master.sendBuildbotNetUsageData = mock.Mock()
         self.master.botmaster = FakeBotMaster()
         self.db = self.master.db = fakedb.FakeDBConnector(self)
-        self.db.setServiceParent(self.master)
+        yield self.db.setServiceParent(self.master)
         self.mq = self.master.mq = fakemq.FakeMQConnector(self)
-        self.mq.setServiceParent(self.master)
+        yield self.mq.setServiceParent(self.master)
         self.data = self.master.data = fakedata.FakeDataConnector(
             self.master, self)
-        self.data.setServiceParent(self.master)
+        yield self.data.setServiceParent(self.master)
 
     def tearDown(self):
         return self.tearDownDirs()
 
-    def make_reactor(self):
-        r = mock.Mock()
-        r.callWhenRunning = reactor.callWhenRunning
-        r.getThreadPool = reactor.getThreadPool
-        r.callFromThread = reactor.callFromThread
-        return r
-
     # tests
     @defer.inlineCallbacks
     def test_startup_bad_config(self):
@@ -120,7 +116,7 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase
 
         yield self.master.startService()
 
-        self.reactor.stop.assert_called_with()
+        self.assertTrue(self.reactor.stop_called)
         self.assertLogged("oh noes")
         self.assertLogged("BuildMaster startup failed")
 
@@ -133,7 +129,7 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase
 
         yield self.master.startService()
 
-        self.reactor.stop.assert_called_with()
+        self.assertTrue(self.reactor.stop_called)
         self.assertLogged("GOT HERE")
         self.assertLogged("BuildMaster startup failed")
 
@@ -145,7 +141,7 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase
 
         yield self.master.startService()
 
-        self.reactor.stop.assert_called_with()
+        self.assertTrue(self.reactor.stop_called)
         self.assertEqual(len(self.flushLoggedErrors(RuntimeError)), 1)
         self.assertLogged("BuildMaster startup failed")
 
@@ -156,7 +152,7 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase
         self.assertTrue(self.master.data.updates.thisMasterActive)
         d = self.master.stopService()
         self.assertTrue(d.called)
-        self.assertFalse(self.reactor.stop.called)
+        self.assertFalse(self.reactor.stop_called)
         self.assertLogged("BuildMaster is running")
 
         # check started/stopped messages
@@ -175,7 +171,7 @@ class StartupAndReconfig(dirs.DirsMixin, logging.LoggingMixin, unittest.TestCase
         self.master.botmaster.shutdownDeferred.callback(None)
         self.assertTrue(d.called)
 
-        self.assertFalse(self.reactor.stop.called)
+        self.assertFalse(self.reactor.stop_called)
         self.assertLogged("BuildMaster is running")
 
         # check started/stopped messages
