@@ -17,10 +17,12 @@
 import inspect
 import pkg_resources
 
+import zope.interface.interface
 from twisted.trial import unittest
+from zope.interface.interface import Attribute
 
 
-class InterfaceTests(object):
+class InterfaceTests:
 
     # assertions
 
@@ -66,9 +68,8 @@ class InterfaceTests(object):
 
         def assert_same_argspec(expected, actual):
             if expected != actual:
-                msg = "Expected: %s; got: %s" % (
-                    inspect.formatargspec(*expected),
-                    inspect.formatargspec(*actual))
+                msg = "Expected: {}; got: {}".format(inspect.formatargspec(*expected),
+                                                     inspect.formatargspec(*actual))
                 self.fail(msg)
 
         actual_argspec = filter_argspec(actualMethod)
@@ -94,14 +95,11 @@ class InterfaceTests(object):
             raise unittest.SkipTest(
                 "zope.interfaces is too old to run this test")
 
-        import zope.interface.interface
-        from zope.interface.interface import Attribute
         for interface in zope.interface.implementedBy(cls):
             for attr, template_argspec in interface.namesAndDescriptions():
                 if not hasattr(cls, attr):
-                    msg = "Expected: %r; to implement: %s as specified in %r" % (
-                        cls, attr,
-                        interface)
+                    msg = ("Expected: {}; to implement: {} as specified in {}"
+                           ).format(repr(cls), attr, repr(interface))
                     self.fail(msg)
                 actual_argspec = getattr(cls, attr)
                 if isinstance(template_argspec, Attribute):
@@ -113,8 +111,7 @@ class InterfaceTests(object):
                     actual_argspec)
 
                 if actual_argspec.getSignatureInfo() != template_argspec.getSignatureInfo():
-                    msg = "%s: expected: %s; got: %s" % (
-                        attr,
-                        template_argspec.getSignatureString(),
-                        actual_argspec.getSignatureString())
+                    msg = ("{}: expected: {}; got: {}"
+                           ).format(attr, template_argspec.getSignatureString(),
+                                    actual_argspec.getSignatureString())
                     self.fail(msg)
