@@ -20,7 +20,7 @@ from buildbot.interfaces import IRenderable
 
 
 @implementer(IRenderable)
-class ConstantRenderable(object):
+class ConstantRenderable:
 
     def __init__(self, value):
         self.value = value
