@@ -28,7 +28,7 @@ from buildbot.util import config
 from buildbot.util import unicode2bytes
 
 
-class ReconfigurableServiceMixin(object):
+class ReconfigurableServiceMixin:
 
     reconfig_priority = 128
 
@@ -51,7 +51,7 @@ class ReconfigurableServiceMixin(object):
 
 # twisted 16's Service is now an new style class, better put everybody new style
 # to catch issues even on twisted < 16
-class AsyncService(service.Service, object):
+class AsyncService(service.Service):
 
     @defer.inlineCallbacks
     def setServiceParent(self, parent):
@@ -72,6 +72,9 @@ class AsyncService(service.Service, object):
 class AsyncMultiService(AsyncService, service.MultiService):
 
     def startService(self):
+        # Do NOT use super() here.
+        # The method resolution order would cause MultiService.startService() to
+        # be called which we explicitly want to override with this method.
         service.Service.startService(self)
         dl = []
         # if a service attaches another service during the reconfiguration
@@ -84,6 +87,9 @@ class AsyncMultiService(AsyncService, service.MultiService):
 
     @defer.inlineCallbacks
     def stopService(self):
+        # Do NOT use super() here.
+        # The method resolution order would cause MultiService.stopService() to
+        # be called which we explicitly want to override with this method.
         service.Service.stopService(self)
         services = list(self)
         services.reverse()
@@ -101,8 +107,8 @@ class AsyncMultiService(AsyncService, service.MultiService):
     def addService(self, service):
         if service.name is not None:
             if service.name in self.namedServices:
-                raise RuntimeError("cannot have two services with same name"
-                                   " '%s'" % service.name)
+                raise RuntimeError(("cannot have two services with same name"
+                                    " '{}'").format(service.name))
             self.namedServices[service.name] = service
         self.services.append(service)
         if self.running:
@@ -173,12 +179,11 @@ class BuildbotService(AsyncMultiService, config.ConfiguredMixin, util.Comparable
             self.name = bytes2unicode(name)
         self.checkConfig(*args, **kwargs)
         if self.name is None:
-            raise ValueError(
-                "%s: must pass a name to constructor" % type(self))
+            raise ValueError("{}: must pass a name to constructor".format(type(self)))
         self._config_args = args
         self._config_kwargs = kwargs
         self.rendered = False
-        AsyncMultiService.__init__(self)
+        super().__init__()
 
     def getConfigDict(self):
         _type = type(self)
@@ -192,7 +197,7 @@ class BuildbotService(AsyncMultiService, config.ConfiguredMixin, util.Comparable
         # only reconfigure if sibling is configured differently.
         # sibling == self is using ComparableMixin's implementation
         # only compare compare_attrs
-        if self.configured and sibling == self:
+        if self.configured and util.ComparableMixin.isEquivalent(sibling, self):
             return None
         self.configured = True
         # render renderables in parallel
@@ -227,7 +232,7 @@ class BuildbotService(AsyncMultiService, config.ConfiguredMixin, util.Comparable
                 yield self.configureService()
             except NotImplementedError:
                 pass
-        yield AsyncMultiService.startService(self)
+        yield super().startService()
 
     def checkConfig(self, *args, **kwargs):
         return defer.succeed(True)
@@ -272,7 +277,7 @@ class ClusteredBuildbotService(BuildbotService):
         self.active = False
         self._activityPollCall = None
         self._activityPollDeferred = None
-        super(ClusteredBuildbotService, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     # activity handling
 
@@ -314,7 +319,7 @@ class ClusteredBuildbotService(BuildbotService):
         # subclasses should override startService only to perform actions that should
         # run on all instances, even if they never get activated on this
         # master.
-        yield super(ClusteredBuildbotService, self).startService()
+        yield super().startService()
         self._startServiceDeferred = defer.Deferred()
         self._startActivityPolling()
         yield self._startServiceDeferred
@@ -338,9 +343,10 @@ class ClusteredBuildbotService(BuildbotService):
                 yield self.deactivate()
                 yield self._unclaimService()
             except Exception as e:
-                log.err(e, _why="Caught exception while deactivating ClusteredService(%s)" % self.name)
+                msg = "Caught exception while deactivating ClusteredService({})".format(self.name)
+                log.err(e, _why=msg)
 
-        yield super(ClusteredBuildbotService, self).stopService()
+        yield super().stopService()
 
     def _startActivityPolling(self):
         self._activityPollCall = task.LoopingCall(self._activityPoll)
@@ -359,6 +365,7 @@ class ClusteredBuildbotService(BuildbotService):
             self._activityPollCall.stop()
             self._activityPollCall = None
             return self._activityPollDeferred
+        return None
 
     def _callbackStartServiceDeferred(self):
         if self._startServiceDeferred is not None:
@@ -378,8 +385,9 @@ class ClusteredBuildbotService(BuildbotService):
             try:
                 claimed = yield self._claimService()
             except Exception:
-                log.err(
-                    _why='WARNING: ClusteredService(%s) got exception while trying to claim' % self.name)
+                msg = ('WARNING: ClusteredService({}) got exception while trying to claim'
+                       ).format(self.name)
+                log.err(_why=msg)
                 return
 
             if not claimed:
@@ -398,8 +406,8 @@ class ClusteredBuildbotService(BuildbotService):
                 yield self.activate()
             except Exception:
                 # this service is half-active, and noted as such in the db..
-                log.err(
-                    _why='WARNING: ClusteredService(%s) is only partially active' % self.name)
+                msg = 'WARNING: ClusteredService({}) is only partially active'.format(self.name)
+                log.err(_why=msg)
             finally:
                 # cannot wait for its deactivation
                 # with yield self._stopActivityPolling
@@ -412,8 +420,8 @@ class ClusteredBuildbotService(BuildbotService):
         except Exception:
             # don't pass exceptions into LoopingCall, which can cause it to
             # fail
-            log.err(
-                _why='WARNING: ClusteredService(%s) failed during activity poll' % self.name)
+            msg = 'WARNING: ClusteredService({}) failed during activity poll'.format(self.name)
+            log.err(_why=msg)
 
 
 class BuildbotServiceManager(AsyncMultiService, config.ConfiguredMixin,
@@ -439,8 +447,7 @@ class BuildbotServiceManager(AsyncMultiService, config.ConfiguredMixin,
         elif isinstance(new_config_attr, dict):
             new_by_name = new_config_attr
         else:
-            raise TypeError(
-                "config.%s should be a list or dictionary" % (self.config_attr))
+            raise TypeError("config.{} should be a list or dictionary".format(self.config_attr))
         new_set = set(new_by_name)
 
         # calculate new childs, by name, and removed childs
@@ -460,39 +467,35 @@ class BuildbotServiceManager(AsyncMultiService, config.ConfiguredMixin,
                 added_names.add(n)
             # compare using ComparableMixin if they don't support reconfig
             elif not hasattr(old, 'reconfigServiceWithBuildbotConfig'):
-                if old != new:
+                if not util.ComparableMixin.isEquivalent(old, new):
                     removed_names.add(n)
                     added_names.add(n)
 
         if removed_names or added_names:
-            log.msg("adding %d new %s, removing %d" %
-                    (len(added_names), self.config_attr, len(removed_names)))
+            log.msg("adding {} new {}, removing {}".format(len(added_names), self.config_attr,
+                                                           len(removed_names)))
 
             for n in removed_names:
                 child = old_by_name[n]
                 # disownServiceParent calls stopService after removing the relationship
                 # as child might use self.master.data to stop itself, its better to stop it first
-                # (this is related to the fact that self.master is found by recursively looking at self.parent
-                # for a master)
+                # (this is related to the fact that self.master is found by recursively looking at
+                # self.parent for a master)
                 yield child.stopService()
                 # it has already called, so do not call it again
                 child.stopService = lambda: None
                 yield child.disownServiceParent()
-                # HACK: we still keep a reference to the master for some cleanup tasks which are not waited by
-                # to stopService (like the complex worker disconnection mechanism)
-                # http://trac.buildbot.net/ticket/3583
-                child.parent = self.master
 
             for n in added_names:
                 child = new_by_name[n]
                 # setup service's objectid
                 if hasattr(child, 'objectid'):
-                    class_name = '%s.%s' % (child.__class__.__module__,
-                                            child.__class__.__name__)
+                    class_name = '{}.{}'.format(child.__class__.__module__,
+                                                child.__class__.__name__)
                     objectid = yield self.master.db.state.getObjectId(
                         child.name, class_name)
                     child.objectid = objectid
-                yield defer.maybeDeferred(child.setServiceParent, self)
+                yield child.setServiceParent(self)
 
         # As the services that were just added got
         # reconfigServiceWithSibling called by
