@@ -26,8 +26,8 @@ from buildbot.worker.docker import DockerBaseWorker
 log = Logger()
 
 
-class MarathonLatentWorker(DockerBaseWorker,
-                           CompatibleLatentWorkerMixin):
+class MarathonLatentWorker(CompatibleLatentWorkerMixin,
+                           DockerBaseWorker):
     """Marathon is a distributed docker container launcher for Mesos"""
     instance = None
     image = None
@@ -43,8 +43,7 @@ class MarathonLatentWorker(DockerBaseWorker,
                     masterFQDN=None,
                     **kwargs):
 
-        DockerBaseWorker.checkConfig(
-            self, name, image=image, masterFQDN=masterFQDN, **kwargs)
+        super().checkConfig(name, image=image, masterFQDN=masterFQDN, **kwargs)
         HTTPClientService.checkAvailable(self.__class__.__name__)
 
     @defer.inlineCallbacks
@@ -63,8 +62,7 @@ class MarathonLatentWorker(DockerBaseWorker,
 
         if 'build_wait_timeout' not in kwargs:
             kwargs['build_wait_timeout'] = 0
-        yield DockerBaseWorker.reconfigService(
-            self, name, image=image, masterFQDN=masterFQDN, **kwargs)
+        yield super().reconfigService(name, image=image, masterFQDN=masterFQDN, **kwargs)
 
         self._http = yield HTTPClientService.getService(
             self.master, marathon_url, auth=marathon_auth)
@@ -114,6 +112,7 @@ class MarathonLatentWorker(DockerBaseWorker,
         res = yield self._http.delete("/v2/apps/{}".format(
             self.getApplicationId()))
         self.instance = None
+        self.resetWorkerPropsOnStop()
 
         if res.code != 200 and reportFailure:
             res_json = yield res.json()
