@@ -16,11 +16,12 @@
 import contextlib
 
 from twisted.internet import defer
-from twisted.internet import reactor
 from twisted.python import log
 from twisted.spread import pb
 
 from buildbot.pbutil import decode
+from buildbot.util import ComparableMixin
+from buildbot.util import deferwaiter
 from buildbot.worker.protocols import base
 
 
@@ -28,7 +29,7 @@ class Listener(base.Listener):
     name = "pbListener"
 
     def __init__(self):
-        base.Listener.__init__(self)
+        super().__init__()
 
         # username : (password, portstr, PBManager registration)
         self._registrations = {}
@@ -43,21 +44,25 @@ class Listener(base.Listener):
         else:
             currentPassword, currentPortStr, currentReg = None, None, None
 
-        if currentPassword != password or currentPortStr != portStr:
-            if currentReg:
-                yield currentReg.unregister()
-                del self._registrations[username]
-            if portStr and password:
-                reg = self.master.pbmanager.register(
-                    portStr, username, password, self._getPerspective)
-                self._registrations[username] = (password, portStr, reg)
-                return reg
+        iseq = (ComparableMixin.isEquivalent(currentPassword, password) and
+                ComparableMixin.isEquivalent(currentPortStr, portStr))
+        if iseq:
+            return currentReg
+
+        if currentReg:
+            yield currentReg.unregister()
+            del self._registrations[username]
+        if portStr and password:
+            reg = yield self.master.pbmanager.register(portStr, username, password,
+                                                       self._getPerspective)
+            self._registrations[username] = (password, portStr, reg)
+            return reg
+        return currentReg
 
     @defer.inlineCallbacks
     def _getPerspective(self, mind, workerName):
         workers = self.master.workers
-        log.msg("worker '%s' attaching from %s" % (workerName,
-                                                   mind.broker.transport.getPeer()))
+        log.msg("worker '{}' attaching from {}".format(workerName, mind.broker.transport.getPeer()))
 
         # try to use TCP keepalives
         try:
@@ -104,19 +109,17 @@ class FileWriterProxy(ReferenceableProxy):
 
 class _NoSuchMethod(Exception):
     """Rewrapped pb.NoSuchMethod remote exception"""
-    pass
 
 
 @contextlib.contextmanager
 def _wrapRemoteException():
     try:
         yield
-    except pb.RemoteError as ex:
-        if ex.remoteType in (b'twisted.spread.flavors.NoSuchMethod',
-                             'twisted.spread.flavors.NoSuchMethod'):
-            raise _NoSuchMethod(ex)
-        else:
-            raise
+    except pb.RemoteError as e:
+        if e.remoteType in (b'twisted.spread.flavors.NoSuchMethod',
+                            'twisted.spread.flavors.NoSuchMethod'):
+            raise _NoSuchMethod(e) from e
+        raise
 
 
 class Connection(base.Connection, pb.Avatar):
@@ -129,8 +132,12 @@ class Connection(base.Connection, pb.Avatar):
     info = None
 
     def __init__(self, master, worker, mind):
-        base.Connection.__init__(self, master, worker)
+        super().__init__(master, worker)
         self.mind = mind
+        self._keepalive_waiter = deferwaiter.DeferWaiter()
+        self._keepalive_action_handler = \
+            deferwaiter.RepeatedActionHandler(master.reactor, self._keepalive_waiter,
+                                              self.keepalive_interval, self._do_keepalive)
 
     # methods called by the PBManager
 
@@ -149,6 +156,10 @@ class Connection(base.Connection, pb.Avatar):
         self.notifyDisconnected()
 
     # disconnection handling
+    @defer.inlineCallbacks
+    def waitShutdown(self):
+        self.stopKeepaliveTimer()
+        yield self._keepalive_waiter.wait()
 
     def loseConnection(self):
         self.stopKeepaliveTimer()
@@ -172,18 +183,15 @@ class Connection(base.Connection, pb.Avatar):
 
     # keepalive handling
 
-    def doKeepalive(self):
+    def _do_keepalive(self):
         return self.mind.callRemote('print', message="keepalive")
 
     def stopKeepaliveTimer(self):
-        if self.keepalive_timer and self.keepalive_timer.active():
-            self.keepalive_timer.cancel()
-            self.keepalive_timer = None
+        self._keepalive_action_handler.stop()
 
     def startKeepaliveTimer(self):
         assert self.keepalive_interval
-        self.keepalive_timer = reactor.callLater(self.keepalive_interval,
-                                                 self.doKeepalive)
+        self._keepalive_action_handler.start()
 
     # methods to send messages to the worker
 
@@ -284,7 +292,7 @@ class Connection(base.Connection, pb.Avatar):
 
             if d:
                 name = self.worker.workername
-                log.msg("Shutting down (old) worker: %s" % name)
+                log.msg("Shutting down (old) worker: {}".format(name))
                 # The remote shutdown call will not complete successfully since
                 # the buildbot process exits almost immediately after getting
                 # the shutdown request.
@@ -295,10 +303,9 @@ class Connection(base.Connection, pb.Avatar):
                 @d.addErrback
                 def _errback(why):
                     if why.check(pb.PBConnectionLost):
-                        log.msg("Lost connection to %s" % name)
+                        log.msg("Lost connection to {}".format(name))
                     else:
-                        log.err("Unexpected error when trying to shutdown %s"
-                                % name)
+                        log.err("Unexpected error when trying to shutdown {}".format(name))
                 return d
             log.err("Couldn't find remote builder to shut down worker")
             return defer.succeed(None)
