@@ -20,12 +20,15 @@ from sphinx.domains import Domain
 from sphinx.domains import Index
 from sphinx.domains import ObjType
 from sphinx.roles import XRefRole
+from sphinx.util import logging
 from sphinx.util import ws_re
 from sphinx.util.docfields import DocFieldTransformer
 from sphinx.util.docfields import Field
 from sphinx.util.docfields import TypedField
 from sphinx.util.nodes import make_refnode
 
+logger = logging.getLogger(__name__)
+
 
 class BBRefTargetDirective(Directive):
 
@@ -43,12 +46,25 @@ class BBRefTargetDirective(Directive):
     final_argument_whitespace = True
     option_spec = {}
     domain = 'bb'
+    doc_field_types = []
+
+    def get_field_type_map(self):
+        # This is the same as DocFieldTransformer.preprocess_fieldtype which got removed in
+        # Sphinx 4.0
+        typemap = {}
+        for fieldtype in self.doc_field_types:
+            for name in fieldtype.names:
+                typemap[name] = fieldtype, False
+            if fieldtype.is_typed:
+                for name in fieldtype.typenames:
+                    typemap[name] = fieldtype, True
+        return typemap
 
     def run(self):
         self.env = env = self.state.document.settings.env
         # normalize whitespace in fullname like XRefRole does
         fullname = ws_re.sub(' ', self.arguments[0].strip())
-        targetname = '%s-%s' % (self.ref_type, fullname)
+        targetname = '{}-{}'.format(self.ref_type, fullname)
 
         # keep the target; this may be used to generate a BBIndex later
         targets = env.domaindata['bb']['targets'].setdefault(self.ref_type, {})
@@ -87,7 +103,7 @@ class BBRefTargetDirective(Directive):
             signode = addnodes.desc_signature(fullname, '')
 
             if self.name_annotation:
-                annotation = "%s " % self.name_annotation
+                annotation = "{} ".format(self.name_annotation)
                 signode += addnodes.desc_annotation(annotation, annotation)
             signode += addnodes.desc_name(fullname, fullname)
             descnode += signode
@@ -111,7 +127,7 @@ class BBRefTargetDirective(Directive):
         try:
             todocname, targetname = targets[target]
         except KeyError:
-            env.warn(fromdocname, "Missing BB reference: bb:%s:%s" % (cls.ref_type, target),
+            env.warn(fromdocname, "Missing BB reference: bb:{}:{}".format(cls.ref_type, target),
                      node.line)
             return None
 
@@ -126,7 +142,7 @@ def make_ref_target_directive(ref_type, indextemplates=None, **kwargs):
     """
     class_vars = dict(ref_type=ref_type, indextemplates=indextemplates)
     class_vars.update(kwargs)
-    return type("BB%sRefTargetDirective" % (ref_type.capitalize(),),
+    return type("BB{}RefTargetDirective".format(ref_type.capitalize()),
                 (BBRefTargetDirective,), class_vars)
 
 
@@ -158,7 +174,7 @@ class BBIndex(Index):
         using the index's C{localname} as the content of the link.
         """
         # indexes appear to be automatically generated at doc DOMAIN-NAME
-        todocname = "bb-%s" % target
+        todocname = "bb-{}".format(target)
 
         node = nodes.reference('', '', internal=True)
         node['refuri'] = builder.get_relative_uri(fromdocname, todocname)
@@ -171,7 +187,7 @@ def make_index(name, localname):
     """
     Create and return a L{BBIndex} subclass, for use in the domain's C{indices}
     """
-    return type("BB%sIndex" % (name.capitalize(),),
+    return type("BB{}Index".format(name.capitalize()),
                 (BBIndex,),
                 dict(name=name, localname=localname))
 
@@ -185,6 +201,7 @@ class BBDomain(Domain):
         'sched': ObjType('sched', 'sched'),
         'chsrc': ObjType('chsrc', 'chsrc'),
         'step': ObjType('step', 'step'),
+        'reportgen': ObjType('reportgen', 'reportgen'),
         'reporter': ObjType('reporter', 'reporter'),
         'configurator': ObjType('configurator', 'configurator'),
         'worker': ObjType('worker', 'worker'),
@@ -193,6 +210,7 @@ class BBDomain(Domain):
         'event': ObjType('event', 'event'),
         'rtype': ObjType('rtype', 'rtype'),
         'rpath': ObjType('rpath', 'rpath'),
+        'raction': ObjType('raction', 'raction'),
     }
 
     directives = {
@@ -216,6 +234,11 @@ class BBDomain(Domain):
                                               'single: Build Steps; %s',
                                               'single: %s Build Step',
                                           ]),
+        'reportgen': make_ref_target_directive('reportgen',
+                                               indextemplates=[
+                                                   'single: Report Generators; %s',
+                                                   'single: %s Report Generator',
+                                               ]),
         'reporter': make_ref_target_directive('reporter',
                                               indextemplates=[
                                                   'single: Reporter Targets; %s',
@@ -263,7 +286,8 @@ class BBDomain(Domain):
                                            has_content=True,
                                            name_annotation='resource type:',
                                            doc_field_types=[
-                                               TypedField('attr', label='Attributes', names=('attr',),
+                                               TypedField('attr', label='Attributes',
+                                                          names=('attr',),
                                                           typenames=('type',), can_collapse=True),
                                            ]),
         'rpath': make_ref_target_directive('rpath',
@@ -295,6 +319,7 @@ class BBDomain(Domain):
         'sched': XRefRole(),
         'chsrc': XRefRole(),
         'step': XRefRole(),
+        'reportgen': XRefRole(),
         'reporter': XRefRole(),
         'configurator': XRefRole(),
         'worker': XRefRole(),
@@ -315,6 +340,7 @@ class BBDomain(Domain):
         make_index("sched", "Scheduler Index"),
         make_index("chsrc", "Change Source Index"),
         make_index("step", "Build Step Index"),
+        make_index("reportgen", "Reporter Generator Index"),
         make_index("reporter", "Reporter Target Index"),
         make_index("configurator", "Configurator Target Index"),
         make_index("worker", "Build Worker Index"),
@@ -333,7 +359,7 @@ class BBDomain(Domain):
                 if idx.name == target:
                     break
             else:
-                raise KeyError("no index named '%s'" % target)
+                raise KeyError("no index named '{}'".format(target))
             return idx.resolve_ref(self, env, fromdocname, builder, typ,
                                    target, node, contnode)
         elif typ in self.directives:
@@ -341,6 +367,39 @@ class BBDomain(Domain):
             return dir.resolve_ref(self, env, fromdocname, builder, typ,
                                    target, node, contnode)
 
+    def merge_domaindata(self, docnames, otherdata):
+        for typ in self.object_types:
+            if typ not in otherdata['targets']:
+                continue
+
+            if typ not in self.data['targets']:
+                self.data['targets'][typ] = otherdata['targets'][typ]
+                continue
+
+            self_data = self.data['targets'][typ]
+            other_data = otherdata['targets'][typ]
+
+            for target_name, target_data in other_data.items():
+                if target_name in self_data:
+                    # for some reason we end up with multiple references to the same things in
+                    # multiple domains. If both instances point to the same location, ignore it,
+                    # otherwise issue a warning.
+                    if other_data[target_name] == self_data[target_name]:
+                        continue
+
+                    self_path = '{0}#{1}'.format(self.env.doc2path(self_data[target_name][0]),
+                                                 self_data[target_name][1])
+
+                    other_path = '{0}#{1}'.format(self.env.doc2path(other_data[target_name][0]),
+                                                  other_data[target_name][1])
+
+                    logger.warning(('Duplicate index {} reference {} in {}, '
+                                    'other instance in {}').format(typ, target_name,
+                                                                   self_path, other_path))
+                else:
+                    self_data[target_name] = target_data
+
 
 def setup(app):
     app.add_domain(BBDomain)
+    return {'parallel_read_safe': True, 'parallel_write_safe': True}
