@@ -16,12 +16,11 @@
 # Method to add build step taken from here
 # https://seasonofcode.com/posts/how-to-add-custom-build-steps-and-commands-to-setuppy.html
 import datetime
-import distutils.cmd
 import os
 import re
 import subprocess
 import sys
-from distutils.version import LooseVersion
+from pkg_resources import parse_version
 from subprocess import PIPE
 from subprocess import STDOUT
 from subprocess import Popen
@@ -30,6 +29,8 @@ import setuptools.command.build_py
 import setuptools.command.egg_info
 from setuptools import setup
 
+import distutils.cmd  # isort:skip
+
 old_listdir = os.listdir
 
 
@@ -83,7 +84,7 @@ def mTimeVersion(init_file):
     return d.strftime("%Y.%m.%d")
 
 
-def getVersionFromArchiveId(git_archive_id='1592144576  (tag: v2.8.2, 2.8.x)'):
+def getVersionFromArchiveId(git_archive_id='1611922179  (tag: v2.10.1, 2.10.x)'):
     """ Extract the tag if a source is from git archive.
 
         When source is exported via `git archive`, the git_archive_id init value is modified
