@@ -2,7 +2,6 @@
 // to test this part: two different builds need to be started
 
 
-import { ForcePage } from './pages/force';
 import { BuilderPage } from './pages/builder';
 import { DashboardPage } from './pages/dashboard';
 import { HomePage } from './pages/home';
@@ -10,17 +9,15 @@ import { browser, by, element, ExpectedConditions as EC } from 'protractor';
 
 
 describe('dashboard page', function() {
-    let force = null;
     let builder = null;
     let home = null;
     let dashboard = null;
 
     beforeEach(async () => {
         builder = new BuilderPage('runtests', 'force');
-        force =  new ForcePage();
         dashboard = new DashboardPage();
         home = new HomePage();
-        await builder.goDefault();
+        await builder.goBuildersList();
     });
 
     afterEach(done =>
@@ -33,8 +30,8 @@ describe('dashboard page', function() {
     );
 
     it('should go to the dashboard page and see no error', async () => {
-        await builder.goForce();
-        await force.clickStartButton();
+        let force = await builder.goForce();
+        await force.clickStartButtonAndWaitRedirectToBuild();
         await home.waitAllBuildsFinished();
         await dashboard.go();
     });
