@@ -3,6 +3,7 @@
 
 import { BasePage } from "./base";
 import { ExpectedConditions as EC } from 'protractor';
+import { bbrowser } from '../utils';
 
 export class ForcePage extends BasePage {
     constructor() {
@@ -43,28 +44,24 @@ export class ForcePage extends BasePage {
 
     async clickStartButton() {
         let button = this.getStartButton();
-        await browser.wait(EC.elementToBeClickable(button),
-                           5000,
-                           "start button not clickable");
+        await bbrowser.wait(EC.elementToBeClickable(button),
+                            "start button not clickable");
         await button.click();
     }
 
     async clickStartButtonAndWaitRedirectToBuild() {
         let previousUrl = await browser.getCurrentUrl();
         await this.clickStartButton();
-        await browser.wait(EC.not(EC.urlIs(previousUrl)),
-                           5000,
-                           "failed to create a buildrequest");
-        await browser.wait(EC.not(EC.urlContains('redirect_to_build=true')),
-                           5000,
-                           "failed to create a build");
+        await bbrowser.wait(EC.not(EC.urlIs(previousUrl)),
+                            "failed to create a buildrequest");
+        await bbrowser.wait(EC.not(EC.urlContains('redirect_to_build=true')),
+                            "failed to create a build");
     }
 
     async clickCancelWholeQueue() {
         let button = this.getCancelWholeQueue();
-        await browser.wait(EC.elementToBeClickable(button),
-                           5000,
-                           "cancel whole queue button not clickable");
+        await bbrowser.wait(EC.elementToBeClickable(button),
+                            "cancel whole queue button not clickable");
         await button.click();
     }
 
