@@ -13,8 +13,8 @@
 #
 # Copyright Buildbot Team Members
 
-from __future__ import absolute_import
-from __future__ import print_function
+# This module is left for backward compatibility of old-named worker API.
+# It should never be imported by Buildbot.
 
 from twisted.internet import defer
 
