@@ -4,7 +4,7 @@
  * Full docs: https://github.com/decaffeinate/decaffeinate/blob/master/docs/suggestions.md
  */
 describe('http Interceptor', function() {
-    beforeEach(module("guanlecoja.ui"));
+    beforeEach(angular.mock.module("guanlecoja.ui"));
 
     it('should intercept errors', inject(function($q, $rootScope, glNotificationService, $timeout, glHttpInterceptor) {
         const d = $q.defer();
@@ -13,18 +13,19 @@ describe('http Interceptor', function() {
         d.reject("oups");
         $rootScope.$digest();
         $timeout.flush();
-        return expect(glNotificationService.network).toHaveBeenCalledWith("oups");
+        expect(glNotificationService.network).toHaveBeenCalledWith("oups");
     })
     );
 
-    return it('should intercept http errors', inject(function($q, $rootScope, glNotificationService, $timeout, glHttpInterceptor) {
+    it('should intercept http errors',
+       inject(function($q, $rootScope, glNotificationService, $timeout, glHttpInterceptor) {
         const d = $q.defer();
         const i = glHttpInterceptor(d.promise);
         spyOn(glNotificationService, "network").and.returnValue(null);
         d.reject({status: "404", data:{error:"not found"}, config:{method:"get",url:"http://foo"}});
         $rootScope.$digest();
         $timeout.flush();
-        return expect(glNotificationService.network).toHaveBeenCalledWith("404:not found when:get http://foo");
+        expect(glNotificationService.network).toHaveBeenCalledWith("404:not found when:get http://foo");
     })
     );
 });
